//---------------------------------------------------------
// QA Navigator suite
// Copyright  2003 - 2007 Ronald D. Grindle
//---------------------------------------------------------

using System;
using System.Data;
using System.IO;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Threading;
using System.Windows.Forms;
using System.Xml;

using RG.Basics;
using RG.SQL;
using RG.XMLDB;

using QN.Basics;
using QN.DataMiner;

namespace QN.CollabMgr
{
	/// <summary>
	/// QA Navigator suite
	/// Copyright  2003 - 2007 Ronald D. Grindle
	///
	/// Collaboration Services
	/// </summary>
	public class frmCollabMgr : System.Windows.Forms.Form {

		#region My Members

		private int m_butExpCmpgDT;
		private int m_butImpDatDT;
		private int m_comboLogDT;
		private int m_txtMsgDH, m_txtMsgDW;
		private int m_txtStatDT;

		private	CmpSelPresets m_CmpSelPresets;

		private QNDataMiner m_DataMiner;
		private QNGUIservice m_GUIservice;
		private QNphrase m_QNphrase;
		private QNpresets m_Presets;
		private QNTools m_QNTools;

		private static SimpleLog m_Logger;

		private string m_ApplPth;

        private RGSQL2 m_DB;
		private XMLDB m_xDB;

		//private SetList m_DocLst;
		//private string m_ExpPth;

		#endregion

		#region Form Resources

		private System.Windows.Forms.TextBox txtMsg;
		private System.Windows.Forms.ComboBox comboLog;
		private System.Windows.Forms.TextBox txtStat;
		private System.Windows.Forms.Button butExpCmpg;
		private System.Windows.Forms.Button butImpDat;
		/// <summary>
		/// Required designer variable.
		/// </summary>
		private System.ComponentModel.Container components = null;

		#endregion 

		#region Window Creating/Destroying

		public frmCollabMgr() {

			try {

				InitializeComponent();
				
				m_Logger = new SimpleLog( "CollabMgr" );

				init();
			}
			catch( Exception e ) {
		
				handleExcp( e );
			}
		}

		/// <summary>
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing ) {
			if( disposing ) {
				if (components != null) {
					components.Dispose();
				}

                m_DB.close();
			}
			base.Dispose( disposing );
		}

		#endregion

		#region Windows Form Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent() {
            System.ComponentModel.ComponentResourceManager resources = new System.ComponentModel.ComponentResourceManager( typeof( frmCollabMgr ) );
            this.txtMsg = new System.Windows.Forms.TextBox();
            this.comboLog = new System.Windows.Forms.ComboBox();
            this.txtStat = new System.Windows.Forms.TextBox();
            this.butExpCmpg = new System.Windows.Forms.Button();
            this.butImpDat = new System.Windows.Forms.Button();
            this.SuspendLayout();
            // 
            // txtMsg
            // 
            this.txtMsg.Anchor = ( (System.Windows.Forms.AnchorStyles)( ( ( ( System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Bottom )
                        | System.Windows.Forms.AnchorStyles.Left )
                        | System.Windows.Forms.AnchorStyles.Right ) ) );
            this.txtMsg.BackColor = System.Drawing.Color.LightSteelBlue;
            this.txtMsg.Font = new System.Drawing.Font( "Microsoft Sans Serif", 9.75F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ( (byte)( 0 ) ) );
            this.txtMsg.Location = new System.Drawing.Point( 5, 5 );
            this.txtMsg.Multiline = true;
            this.txtMsg.Name = "txtMsg";
            this.txtMsg.ReadOnly = true;
            this.txtMsg.ScrollBars = System.Windows.Forms.ScrollBars.Both;
            this.txtMsg.Size = new System.Drawing.Size( 773, 277 );
            this.txtMsg.TabIndex = 0;
            // 
            // comboLog
            // 
            this.comboLog.Anchor = ( (System.Windows.Forms.AnchorStyles)( ( System.Windows.Forms.AnchorStyles.Bottom | System.Windows.Forms.AnchorStyles.Left ) ) );
            this.comboLog.BackColor = System.Drawing.Color.LightGray;
            this.comboLog.DropDownStyle = System.Windows.Forms.ComboBoxStyle.DropDownList;
            this.comboLog.Font = new System.Drawing.Font( "Microsoft Sans Serif", 9.75F );
            this.comboLog.ItemHeight = 20;
            this.comboLog.Location = new System.Drawing.Point( 139, 337 );
            this.comboLog.Name = "comboLog";
            this.comboLog.Size = new System.Drawing.Size( 639, 28 );
            this.comboLog.TabIndex = 2;
            this.comboLog.TabStop = false;
            // 
            // txtStat
            // 
            this.txtStat.Anchor = ( (System.Windows.Forms.AnchorStyles)( ( System.Windows.Forms.AnchorStyles.Bottom | System.Windows.Forms.AnchorStyles.Left ) ) );
            this.txtStat.BackColor = System.Drawing.Color.Gold;
            this.txtStat.Font = new System.Drawing.Font( "Microsoft Sans Serif", 9.75F, System.Drawing.FontStyle.Bold );
            this.txtStat.Location = new System.Drawing.Point( 5, 337 );
            this.txtStat.Name = "txtStat";
            this.txtStat.ReadOnly = true;
            this.txtStat.Size = new System.Drawing.Size( 129, 26 );
            this.txtStat.TabIndex = 1;
            this.txtStat.TabStop = false;
            this.txtStat.Text = "Status";
            this.txtStat.TextAlign = System.Windows.Forms.HorizontalAlignment.Center;
            // 
            // butExpCmpg
            // 
            this.butExpCmpg.Anchor = ( (System.Windows.Forms.AnchorStyles)( ( System.Windows.Forms.AnchorStyles.Bottom | System.Windows.Forms.AnchorStyles.Left ) ) );
            this.butExpCmpg.BackColor = System.Drawing.Color.MediumBlue;
            this.butExpCmpg.Font = new System.Drawing.Font( "Microsoft Sans Serif", 9.75F, System.Drawing.FontStyle.Bold, System.Drawing.GraphicsUnit.Point, ( (byte)( 0 ) ) );
            this.butExpCmpg.ForeColor = System.Drawing.Color.LightGray;
            this.butExpCmpg.Location = new System.Drawing.Point( 5, 295 );
            this.butExpCmpg.Name = "butExpCmpg";
            this.butExpCmpg.Size = new System.Drawing.Size( 201, 33 );
            this.butExpCmpg.TabIndex = 3;
            this.butExpCmpg.Text = "kampagne exportieren";
            this.butExpCmpg.UseVisualStyleBackColor = false;
            this.butExpCmpg.MouseLeave += new System.EventHandler( this.butExpCmpg_MouseLeave );
            this.butExpCmpg.Click += new System.EventHandler( this.butExpCmpg_Click );
            this.butExpCmpg.MouseEnter += new System.EventHandler( this.butExpCmpg_MouseEnter );
            // 
            // butImpDat
            // 
            this.butImpDat.Anchor = ( (System.Windows.Forms.AnchorStyles)( ( System.Windows.Forms.AnchorStyles.Bottom | System.Windows.Forms.AnchorStyles.Left ) ) );
            this.butImpDat.BackColor = System.Drawing.Color.MediumBlue;
            this.butImpDat.Font = new System.Drawing.Font( "Microsoft Sans Serif", 9.75F, System.Drawing.FontStyle.Bold, System.Drawing.GraphicsUnit.Point, ( (byte)( 0 ) ) );
            this.butImpDat.ForeColor = System.Drawing.Color.LightGray;
            this.butImpDat.Location = new System.Drawing.Point( 211, 295 );
            this.butImpDat.Name = "butImpDat";
            this.butImpDat.Size = new System.Drawing.Size( 202, 33 );
            this.butImpDat.TabIndex = 4;
            this.butImpDat.Text = "import data";
            this.butImpDat.UseVisualStyleBackColor = false;
            this.butImpDat.MouseLeave += new System.EventHandler( this.butImpDat_MouseLeave );
            this.butImpDat.Click += new System.EventHandler( this.butImpDat_Click );
            this.butImpDat.MouseEnter += new System.EventHandler( this.butImpDat_MouseEnter );
            // 
            // frmCollabMgr
            // 
            this.AutoScaleBaseSize = new System.Drawing.Size( 6, 15 );
            this.BackColor = System.Drawing.Color.MidnightBlue;
            this.ClientSize = new System.Drawing.Size( 788, 374 );
            this.Controls.Add( this.butImpDat );
            this.Controls.Add( this.butExpCmpg );
            this.Controls.Add( this.comboLog );
            this.Controls.Add( this.txtStat );
            this.Controls.Add( this.txtMsg );
            this.Icon = ( (System.Drawing.Icon)( resources.GetObject( "$this.Icon" ) ) );
            this.Name = "frmCollabMgr";
            this.Text = "collab mgr";
            this.Resize += new System.EventHandler( this.frmCollabMgr_Resize );
            this.ResumeLayout( false );
            this.PerformLayout();

		}
		#endregion


		/// <summary>
		/// The main entry point for the application.
		/// </summary>
		[STAThread]
		static void Main() {
			Application.Run(new frmCollabMgr());
		}

		#region My Stuff

		//--------------------------
		private void addLog( IRslt p_Res ) {

			int ix;

			string msg = BTools.getNowISOTime() + " " + p_Res.getRsltLastMsg();

			ix = comboLog.Items.Add( msg );

			m_Logger.write2Log( ref p_Res );

			comboLog.SelectedIndex = ix;
		}

		//---------------------------
		private void addLog( string p_Msg ) {

			int ix;

			string msg = BTools.getNowISOTime() + " " + p_Msg;

			ix = comboLog.Items.Add( msg );

			m_Logger.write2Log( msg );

			comboLog.SelectedIndex = ix;
		}

		//-----------------------------
		private void addPRDocs( string p_TCExUID, SetList p_DocLst ) {

			int i, x;

			SetList aPRLst;
			SetList aFixLst;
			
			//ProbReport/RefTCEx( RefUID ) == p_UID
			aPRLst = m_DataMiner.getPR4TCEx( ref p_TCExUID );
			
			p_DocLst.AddRange( aPRLst );		
			
			x = aPRLst.Count;

			for( i = 0; i < x; ++i ) {

				string PR = aPRLst[ i ];
		
				//Fix/RefProbRep( RefUID )
				aFixLst = m_DataMiner.getFix4PR( ref PR ); 

				p_DocLst.AddRange( aFixLst );
			}
		}

		// ----------------------
		private RsltString cmpgSelDial() {

			RsltString cmpgUID = new RsltString();

			try {

				DataLst RsltLst;

				int x;
				DatLstEntry entry;

				frmCmpSel dialCmpSel = new frmCmpSel( m_xDB, true, m_Presets, m_Presets.m_CmpgnStatLst, m_CmpSelPresets );

				DialogResult dRes = dialCmpSel.ShowDialog();

				if( DialogResult.Cancel == dRes ) {

					writeLog( m_QNphrase.msgProcCanceled );
				}
				else {

					RsltLst = dialCmpSel.m_RsltLst;

					x = RsltLst.Count;

					if( x > 0 ) {

						entry = RsltLst[0];

						cmpgUID.theString = entry.val;					
					}
				}

				dialCmpSel.Dispose();
			}
			catch( Exception excp ) {
		
				handleExcp( excp );
			}

			return cmpgUID;
		}

		// ----------------------
		private void enableAppl( bool p_Enable ) {

			butExpCmpg.Enabled = p_Enable;
			butImpDat.Enabled = p_Enable;

		}

		// ----------------------
		private void expCmpg( ) {

			Rslt res = new Rslt( true );

			RsltString cmpgUID;
			RsltString expPth;
			RsltString val;

			SetList docLst;

			string cmpgnName;

			XMLDBNode cmpgNode;

			try {

				reset();

				setAppStat( AppStat.Proc );

				cmpgUID = cmpgSelDial();

				if( res.checkRslt( cmpgUID ) ) {

					cmpgNode = getDoc( cmpgUID.theString );

					if( res.checkRslt( cmpgNode ) ) {

						this.Refresh();

						val = cmpgNode.getAttrValFromHtml( "Title" );

						cmpgnName = val.theString;

						writeLog( String.Format( m_QNphrase.msgExpngCmpgn, cmpgnName ) );

						expPth = selectPth( "exp" );

						if( res.checkRslt( expPth ) ) {

							//m_ExpPth = expPth.theString;

							writeLog( String.Format( m_QNphrase.msgExpngTo, expPth.theString ) );

							docLst = getDocLst( ref cmpgNode.m_xDoc );

							if( res.checkRslt( docLst ) ) {

								docLst.Add( cmpgUID.theString );
							
								writeLog( String.Format( m_QNphrase.msgExpngNDoc, docLst.Count ) );

								res = expLst( expPth.theString, docLst );

								/*
								Thread loadThread = new Thread(new ThreadStart(expLst));
								loadThread.IsBackground = true;
								loadThread.Start();
								*/

								if( res.isOK() ) {

									addLog( String.Format( m_QNphrase.msgCmpgnExpd, cmpgnName ));
								}
							}
						}
					}
				}
			}
			catch( Exception excp ) {
						
				handleExcp( excp );

				res.setRsltByExcp( excp );
			}

			handleStat( res, m_QNphrase.errCmpgnExpFailed, true );
		}

		// ----------------------
		private Rslt expLst( string p_ExpPth, SetList p_DocLst ) {

			Rslt res = new Rslt( true );

			int i, x;

			RsltString val;
			RsltString alock;

			string fileName;
			string text;

			string uid = "";

			XMLDBNode docNode;

			x = p_DocLst.Count;

			for( i = 0; i < x; ++i ) {

				uid = p_DocLst[i];

				docNode = getDoc( uid );

				if( res.checkRslt( docNode ) ) {

					val = docNode.getAttrValFromHtml( "Title" );

					text = String.Format( m_QNphrase.msgExpng, val.theString );
					text += " (" + docNode.m_xElem.Name + " " + uid + ")";
					writeLog( text );

					fileName = p_ExpPth + "\\" + uid + ".xml";

					alock = docNode.addLock();

					docNode.m_xDoc.Save( fileName );
				}
				else {

					i = x;
				}
			}

			if( res.notOK() ) {

				res.addRsltAddInfo( "UID of last attempt: " + uid );
			}

			txtMsg.Text	+= m_QNphrase.msgDone + "\r\n";
			

			handleStat( res, m_QNphrase.errExpFailed, true );

			return res;
		}

		//------------------------------
		private XMLDBNode getDoc( string p_UID ) {

			XMLDBNode xDBNode;

			xDBNode = m_xDB.getXMLsubTree( ref p_UID );

			if( xDBNode.notOK()  ) {

				handleErr( xDBNode, m_QNphrase.errLoadFailed );
			}

			return xDBNode;
		}

		//-----------------------------
		// Get all the documents refered in the campaign document.
		private SetList getDocLst( ref XmlDocument p_XDoc ) {

			SetList docLst = new SetList();

			int i, j, x, y;

			RsltString xVal;

			RsltXMLNode TCnode;
			RsltXMLNode TCSnode;
			RsltXMLNode TCSListNode;

			XmlNodeList nodeLst;
			XmlNodeList TCnodeLst;
			XmlNodeList TCSnodeLst;
			
			nodeLst = p_XDoc.SelectNodes( "/Cmpgn/TCSList" );

			if( nodeLst.Count == 1 ) {

				TCSListNode = nodeLst.Item( 0 );

				TCSnodeLst = TCSListNode.m_xNode.SelectNodes( "./CmpgnTCSet" );

				x = TCSnodeLst.Count;

				for( i = 0; i < x; ++i ) {

					TCSnode = TCSnodeLst.Item( i );

					xVal = TCSnode.getAttrVal( "RefUID" );
					docLst.Add( xVal.theString );

					TCnodeLst = TCSnode.m_xNode.SelectNodes( "./RefCTC" );

					y = TCnodeLst.Count;

					for( j = 0; j < y; ++j ) {

						TCnode = TCnodeLst.Item( j );

						xVal = TCnode.getAttrVal( "RefUID" );
						docLst.Add( xVal.theString );

						getTCExDocs( TCnode, docLst );
					}
				}
			}

			return docLst;
		}


		//-----------------------------
		// Get all execution documents refering to the campaign.
		private void getTCExDocs( RsltXMLNode p_TCnode, SetList p_DocLst ) {

			int i, x;

			RsltString xVal;

			RsltXMLNode TCExNode;

			XmlNodeList TCExnodeLst;

			TCExnodeLst = p_TCnode.m_xNode.SelectNodes( "./RefTCExec" );

			x = TCExnodeLst.Count;
						
			for( i = 0; i < x; ++i ) {

				TCExNode = TCExnodeLst.Item( i );

				xVal = TCExNode.getAttrVal( "RefUID" );

				p_DocLst.Add( xVal.theString );
 				
				addPRDocs( xVal.theString, p_DocLst );
			}
		}

		//-----------------------------
		private void handleExcp( Exception p_Excp ) {
			
			try {

				setAppStat( AppStat.Error );

				addLog( "Exception '" + p_Excp.Message + "'");

				m_Logger.write2Log( ref p_Excp );

				m_GUIservice.dispExcp( p_Excp );
			}
			catch {

				string msg = "An exception has occured at a very early stage of the"
					+ " initialization of the program. So early that the program is not able"
					+ " to provide you any aid. Errors at this stage are mostly due to"
					+ " bad configuration. If the configuration has been changed recently,"
					+ " switch back to the former version, if you can. In other cases please"
					+ " refer to the list of known problems in your documentation.";

				MessageBox.Show(  msg, "Early Error Occured", MessageBoxButtons.OK, MessageBoxIcon.Error );
			}
		}

		//-----------------------------
		private void handleIfError( IRslt p_Res, string p_Msg, bool p_EmptyIsOK ) {

			if( p_Res.notOK() && !( p_Res.isInStat( RsltStatVal.isEmpty ) && p_EmptyIsOK )  ) {

				handleErr( p_Res, p_Msg );
			}
		}

		//-----------------------------
		private void handleErr( IRslt p_Res, string p_Msg ) {

			setAppStat( AppStat.Error );

			m_Logger.write2Log( ref p_Res );

			addLog( p_Msg );

			issueErr( p_Msg );
		}

		//-----------------------------
		private void handleStat( IRslt p_Res, string p_Msg, bool p_EmptyIsOK ) {

			if( p_Res.isOK() || ( p_Res.isInStat( RsltStatVal.isEmpty ) && p_EmptyIsOK )  ) {

				setAppStat( AppStat.OK );
			}
			else {

				handleErr( p_Res, p_Msg );
			}
		}

		//---------------------
		private void impDat() {

			int i, x;

			Rslt res = new Rslt( true );
			RsltString impPth;
			RsltString rStr;

			string[] fileLst;

			try {
				
				reset();

				setAppStat( AppStat.Proc );

				impPth = selectPth( "imp" );

				if( res.checkRslt( impPth ) ) {

					writeLog( String.Format( m_QNphrase.msgImpFrom, impPth.theString ) );

					fileLst = Directory.GetFiles( impPth, "*.xml" );

					x = fileLst.Length;

					for( i = 0; i < x; ++i ) {

						writeLog( String.Format( m_QNphrase.msgImping, fileLst[ i ] ) );

						rStr = m_DataMiner.importDoc( fileLst[ i ] );

						writeLog( rStr );

						if( !res.checkRslt( rStr ) ) {

							i = x;
						}
					}

					if( res.isOK() ) {

						addLog( String.Format( m_QNphrase.msgDocImpdFrom, impPth.theString ));
					}
				}
			}
			catch( Exception excp ) {
						
				handleExcp( excp );

				res.setRsltByExcp( excp );
			}

			handleStat( res, m_QNphrase.errImpFailed, true );
		}

		//---------------------
		private void init() {
			frmCollabMgr.CheckForIllegalCrossThreadCalls = false;			
			int x, y;

			Rslt res = new Rslt( true );

			m_QNTools = new QNTools();

			m_QNphrase = m_QNTools.m_QNphrase;

			m_GUIservice = new QNGUIservice( m_QNphrase );

			m_Logger.write2Log( m_QNTools.getApplInfo() );

			// resizing stuff
			x = this.Width;
			y = this.Height;

			m_txtMsgDW = x - txtMsg.Width;
			m_txtMsgDH = y - txtMsg.Height;

			m_butExpCmpgDT = y - butExpCmpg.Top;
			m_butImpDatDT = y - butImpDat.Top;

			m_txtStatDT = y - txtStat.Top;
			m_comboLogDT = y - comboLog.Top;

			setLabels();

			m_ApplPth = m_QNTools.getApplPth();

			m_Presets = new QNpresets( m_QNTools );

			if( m_Presets.notOK() ) {

				res.setRsltStat( m_Presets, m_QNphrase.errPresetFrm );
			}
			else {
			
				m_CmpSelPresets = new CmpSelPresets( m_Presets.m_Author );

				setUpAppl();

                m_DB = new RGSQL2( m_Presets.m_DBloc, m_Presets.m_DBname, m_Presets.m_DBuser, m_Presets.m_DBpw );

				if( res.checkRslt( m_DB ) ) {

    				m_xDB = new XMLDB( m_DB, m_Presets.m_UIDsrvLoc );

					m_DataMiner = new QNDataMiner( m_xDB );
				}
			}

			if( res.notOK() ) {

				handleErr( res, m_QNphrase.errInitFailed );
			}
			else {

				reset();

				setAppStat( AppStat.OK );

				addLog( BTools.getNowISODate() + " " + m_QNphrase.msgSesStart );
			}
		}
	
		//-------------------------------------------------------------------------
		private void issueErr( string msg ) {

			MessageBox.Show( msg, m_QNphrase.lblErr, MessageBoxButtons.OK, MessageBoxIcon.Error );
		}

		//-------------------------------------------------------------------------
		private void reset() {

			txtMsg.Text = "";

			//m_DocLst = null;
			//m_ExpPth = "";

			setAppStatOK( true );
		}

		//--------------------------
		private RsltString selectPth( string p_Mode ) {

			RsltString selPth = new RsltString();

			try {

				FolderBrowserDialog selPathDial;
				selPathDial = new FolderBrowserDialog();

				// Set the help text description for the FolderBrowserDialog.
				//selPathDialDescription = "Select the directory that you want to use as the default.";

				if( p_Mode.CompareTo( "exp" ) == 0 ) {

					selPathDial.Description = m_QNphrase.msgSelPath4Exp;

					selPathDial.Description += "\r\n" + m_QNphrase.msgMakSureDTD;
				}
				
				if( p_Mode.CompareTo( "imp" ) == 0 ) {

					selPathDial.Description = m_QNphrase.msgSelSrcPath;

					//selPathDial.Description += "\r\n" + m_QNphrase.msgMakSureDTD;
				}

				selPathDial.ShowNewFolderButton = false;
				selPathDial.SelectedPath = m_Presets.m_ExpPth;

				DialogResult dRes = selPathDial.ShowDialog();

				if( DialogResult.Cancel == dRes ) {

					writeLog( m_QNphrase.msgProcCanceled );
				}

				if( dRes == DialogResult.OK ) {

					selPth.theString = selPathDial.SelectedPath;
				}

				selPathDial.Dispose();
			}
			catch( Exception excp ) {
						
				handleExcp( excp );
			}

			return selPth;
		}


		//--------------------------
		private void setAppStat( AppStat p_Stat ) {

			switch( p_Stat) {

				case AppStat.OK:

					txtStat.BackColor = m_Presets.m_ClrStatOK;
					txtStat.ForeColor = Color.White;
					txtStat.Text = m_QNphrase.lblOK;

					enableAppl( true );
				break;

				case AppStat.Proc:

					enableAppl( false );

					txtStat.BackColor = m_Presets.m_ClrStatProc;
					txtStat.ForeColor = Color.Black;
					txtStat.Text = m_QNphrase.lblProcng;
				break;

				case AppStat.Error:

					txtStat.BackColor = m_Presets.m_ClrStatErr;
					txtStat.ForeColor = Color.White;
					txtStat.Text =  m_QNphrase.lblErr;

					enableAppl( true );
				break;
			}

			this.Refresh();			
		}

		//-------------------------------------------------------------------------
		private void setAppStatOK( bool p_IsOK ) {

			if( p_IsOK ) {

				setAppStat( AppStat.OK );
			}
			else {

				setAppStat( AppStat.Error );
			}
		}

		// ----------------------------------------------------
		private void setUpAppl() {

			//txtAuthor.Text = m_Presets.m_Author;	
		}

		// -----------------------
		private void writeLog( string p_Msg ) {

			txtMsg.Text	+= p_Msg + "\r\n";
			m_Logger.write2Log( p_Msg );
		}

		#endregion

		#region I18N

		// -----------------------
		private void setLabels() {

			Text = m_QNphrase.lblCollabMgr;

			butExpCmpg.Text = m_QNphrase.blblExpCmpg;
			butImpDat.Text = m_QNphrase.blblImpDat;
		}

		#endregion

		#region Event Handling

		// -----------------------
		private void butExpCmpg_Click(object sender, System.EventArgs e) {
		
			//expCmpg();

			setAppStat( AppStat.Proc );

			Thread loadThread = new Thread(new ThreadStart(expCmpg));
			loadThread.IsBackground = true;
			loadThread.Start();

			setAppStat( AppStat.OK );
		}

		// -----------------------
		private void butImpDat_Click(object sender, System.EventArgs e) {
		
			//impDat();

			setAppStat( AppStat.Proc );

			Thread loadThread = new Thread(new ThreadStart(impDat));
			loadThread.IsBackground = true;
			loadThread.Start();

			setAppStat( AppStat.OK );
		}
		
		// ----------------------------
		private void butExpCmpg_MouseLeave(object sender, System.EventArgs e) {
		
			butExpCmpg.BackColor = System.Drawing.Color.MediumBlue;
		}

		// ----------------------------
		private void butExpCmpg_MouseEnter(object sender, System.EventArgs e) {
		
			butExpCmpg.BackColor = System.Drawing.Color.Blue;
		}

		// ----------------------------
		private void butImpDat_MouseEnter(object sender, System.EventArgs e) {
		
			butImpDat.BackColor = System.Drawing.Color.Blue;
		}

		// ----------------------------
		private void butImpDat_MouseLeave(object sender, System.EventArgs e) {
		
			butImpDat.BackColor = System.Drawing.Color.MediumBlue;
		}

		// ----------------------------
		private void frmCollabMgr_Resize(object sender, System.EventArgs e) {
		
			int x, y;

			y = this.Height;
			x = this.Width;

			txtMsg.Height = y - m_txtMsgDH;
			txtMsg.Width = x - m_txtMsgDW;

			butImpDat.Top = y - m_butImpDatDT;
			butExpCmpg.Top = y - m_butExpCmpgDT;

			txtStat.Top = y - m_txtStatDT;
			comboLog.Top = y - m_comboLogDT;
		}

		
		#endregion

	}
}
