using System;
using System.Collections;

using QN.Basics;
using QN.DataMiner;

using RG.Basics;
using RG.SQL;
using RG.XMLDB;

namespace QN.Stat {

	/// <summary>
	/// Summary description for QNStatTools.
	/// </summary>
	public class QNStatTools {

		#region My Members

		private bool m_hasFailed;

		private int m_ExTime, m_TRCnt, m_TcCnt, m_TcExCnt, m_AvExTime, m_TcExecs, m_RemExTime, m_TCSets, m_TCFailed, m_EstExTime;

		private int m_ExcDurPos, m_ExpDurPos, m_ExcCodePos, m_ExcTitPos;

		private QNDataMiner qndm;

		private XMLDB m_xDB;

		#endregion

		// -------------------------
		/// <summary>
		/// 
		/// </summary>
		/// <param name="p_xDB"></param>
		public QNStatTools( XMLDB p_xDB ) {

			m_xDB = p_xDB;

			qndm = new QNDataMiner( p_xDB );
		}

		/* 2007-05-21
			public QNStatTools( XMLDB p_xDB, QNphrase p_QNphrase ) {

				m_QNphrase = p_QNphrase;
				m_xDB = p_xDB;
			}
		*/

		// ------------------------------------
		public Rslt genStat4Cmpgn( string p_CmpgnID, QNStatData p_CmpgnData, 
			QNStatData p_TCSetData, QNStatData p_TCData, QNStatData p_TC2ReqData, QNStatData p_TCExData, QNStatData p_TRData, 
			ref string p_DateTime ) {

			SetList uidLstCmpgn;
			SetList uidLstTCSet;

            string[] attrs;
			string[] headers;
			string[] addAttrVal;

			m_ExTime = 0;
			m_TRCnt = 0;
			m_TcCnt = 0;
			m_AvExTime = 0;
			m_TcExecs = 0;
			m_RemExTime = 0;
			m_TCSets = 0;

			m_ExcDurPos = -1;
			m_ExpDurPos = -1;
			m_ExcCodePos = -1;
			m_ExcTitPos = -1;

			m_TcExCnt = 0;
			
			Rslt res = new Rslt( true );
			Rslt xRes;

            attrs = new string[] { "UID", "Appl", "Author", "DateTime", "SoftRel", "Status", "Title" };
            p_CmpgnData.setUpAttrs( attrs );
			headers = new string[] {"TC Sets", "TCs", "TC Execs", "TC Execs Failed", "Est Exec Time", "Tot Exec Time", "Av Exec Time", "Rem Exec Time", "PRs", "Creation Date", "Creation Time"};
			p_CmpgnData.createAddAttrs( headers );

            attrs = new string[] { "UID", "Title", "ExpDur", "Assignd"};
            p_TCSetData.setUpAttrs( attrs );
			headers = new string[] {"Parent", "Sub TC Sets", "TCs", "TC Execs", "TC Execs Failed", "Est Exec Time", "Tot Exec Time", "Av Exec Time", "Rem Exec Time", "PRs"};
			p_TCSetData.createAddAttrs( headers );

            attrs = new string[] { "UID", "Title", "ExpDur", "Assignd"};
            p_TCData.setUpAttrs( attrs );
			headers = new string[] {"TC Set", "TC Execs", "Failed", "Tot Exec Time", "Av Exec Time", "Rem Exec Time", "PRs" };
			p_TCData.createAddAttrs( headers );


			//headers = new string[] {"TC Name", "TC UID" };
			//p_TC2ReqData.createAddAttrs( headers );

            attrs = new string[] { "UID", "Title", "Author", "Cmpgn", "Date", "Time", "ExcDur", "ExCode", "Section", "Appl" };
            p_TCExData.setUpAttrs( attrs ); 
			headers = new string[] {"TC Set","PRs"};
			p_TCExData.createAddAttrs( headers );

            attrs = new string[] { "UID", "Appl", "Assignd", "Author", "DateTime", "EstDate", "ExSys", "Fctn", "LastMod", "LastModBy", "Lock", "Prio", "ProbClass", "ProbCode", "ProbProcStat", "ProbStat", "Reqstr", "SoftRel", "Ticket", "Title"};
            p_TRData.setUpAttrs( attrs );
			headers = new string[] {"TC", "Fixes"};
			p_TRData.createAddAttrs( headers );

			// walk through the list TC Sets
			// There shouldn't be more than one TCSList, but if not, we get them all.
			uidLstTCSet = qndm.getSubList( p_CmpgnID, "TCSList", "CmpgnTCSet" );

			if( res.checkRslt( uidLstTCSet ) ) {

				xRes = genStat4ExTCSets( p_TCSetData, p_TCData, p_TC2ReqData, p_TCExData, p_TRData, uidLstTCSet, "-" );

				if( res.checkRslt( xRes ) ) {

					// get info for Campaign
					uidLstCmpgn = new SetList();

					uidLstCmpgn.Add( p_CmpgnID );

					p_CmpgnData.getAttrsForAll( uidLstCmpgn );

					if( res.checkRslt( p_CmpgnData ) ) {

						// headers = new string[] {"TC Sets", "TCs", "TC Execs", "TC Execs Failed", 
						// "Est Exec Time", "Tot Exec Time", "Av Exec Time", "Rem Exec Time", "PRs", 
						// "Creation Date", "Creation Time"};

						addAttrVal = new string[10];		
		
						addAttrVal[0] = m_TCSets.ToString();
						addAttrVal[1] = m_TcCnt.ToString();
						addAttrVal[2] = m_TcExecs.ToString();
						addAttrVal[3] = m_TCFailed.ToString();
						addAttrVal[4] = m_EstExTime.ToString();
						addAttrVal[5] = m_ExTime.ToString();
						addAttrVal[6] = m_AvExTime.ToString();
						addAttrVal[7] = m_RemExTime.ToString();
						addAttrVal[8] = m_TRCnt.ToString();
						addAttrVal[9] = p_DateTime;

						p_CmpgnData.addAddAttrs( addAttrVal );

						res.checkRslt( p_CmpgnData );
					}
				}
			}

			return res;
		}

        /// <summary>
        /// 
        /// </summary>
        /// <param name="p_Data"></param>
        /// <param name="p_Log"></param>
        /// <param name="p_DateTime"></param>
        /// <param name="p_All"></param>
        /// <returns></returns>
        public Rslt genStat4CR( QNStatData p_Data, QNStatData p_Log, ref string p_DateTime, bool p_All ) {

            bool isNew;

			int i, x, k, y;
			//RsltTable nVals;
            RsltString rVal;

			SetList uidSubLst;
			SetList uidLst;
            
            string subUID;
			string UID;
            string val;
            string[] attrs;
            string[] logAttrs;
            string[] xHeaders;
			string[] addAttrVal;

			Rslt res = new Rslt( true );

            attrs = new string[] { "UID", "AffSys", "Appl", "Assignd", "Author", "ContctBA", "ContctImp", "CostBA", "CostBAOver", "CostImp", "CostImpOver", "CostTotal", "CRProcStat", "CRStat", "DateTime", "DlvDate", "EstDate", "ExtDateTime", "ExtDocID", "ExtID", "ExtSource", "ExtStat", "OrderDate", "Prio", "RelAct", "RelPlan", "Reqstr", "RiscOver", "SoftRel", "Ticket", "Title", "TrgtEstDate", "TrgtImpDate", "Type" };

            logAttrs = new string[] { "Issue UID", "Assignd", "DateTime", "NStat" };

            xHeaders = new string[] { "Cmnt", "ExtSystms", "ExtPRs", "PRNums" };

			uidLst = getCRLst( !(p_All) );

			if( res.checkRslt( uidLst ) ) {

                p_Data.setUpAttrs( attrs );
                p_Data.createAddAttrs( xHeaders ) ;

                p_Log.setUpAttrs( logAttrs );
			}

			x = uidLst.Count;

			for(i = 0; i < x; i++) 	{

				UID = uidLst[i];

				res = p_Data.getAttrFor( UID );

                // Add data
                if(res.isOK())
                {
                    addAttrVal = new string[4];

                    rVal = m_xDB.getNodeValByParent( "Cmnt", UID );

                    val = SFormatter.decodeHTML( rVal.theString );

                    addAttrVal[0] = val.Replace( "\r\n", " " );

                    rVal = m_xDB.getNodeValByParent( "ExtSystms", UID );

                    val = SFormatter.decodeHTML( rVal.theString );

                    addAttrVal[1] = val.Replace( "\r\n", " " );

                    uidSubLst = m_xDB.getUIDLstByParent( "ExtPR", UID );

                    y = uidSubLst.Count;

                    val = "";

                    isNew = true;

                    for(k = 0; k < y; k++) {

                        subUID = uidSubLst[k];

                        rVal = m_xDB.getAttrValFromHtml( subUID, "ExtID" );

                        if(rVal.theString.Length > 0) {

                            if(isNew) {

                                isNew = false;
                            }
                            else {

                                val += ", ";
                            }

                            val += rVal.theString;
                        }
                    }

                    addAttrVal[2] = val;

                    uidSubLst = m_xDB.getUIDLstByParent( "RefProbRep", UID );

                    y = uidSubLst.Count;

                    val = "";

                    isNew = true;

                    for(k = 0; k < y; k++) {

                        subUID = uidSubLst[k];

                        rVal = m_xDB.getAttrValFromHtml( subUID, "Ticket" );

                        if(rVal.theString.Length > 0) {

                            if(isNew) {

                                isNew = false;
                            }
                            else {

                                val += ", ";
                            }

                            val += rVal.theString;
                        }
                    }

                    addAttrVal[3] = val;

                    p_Data.addAddAttrs( addAttrVal );
                }	

                // Log data
                if(res.isOK())
                {
                    uidSubLst = m_xDB.getUIDLstByParent( "ProcLog", UID );

                    y = uidSubLst.Count;

                    for(k = 0; k < y; k++)
                    {
                        addAttrVal = new string[4];

                        subUID = uidSubLst[k];

                        rVal = m_xDB.getAttrValFromHtml( subUID, "NStat" );

                        val = rVal.theString;

                        if(val.Length > 0)
                        {
                            addAttrVal[0] = UID;

                            addAttrVal[3] = val;

                            rVal = m_xDB.getAttrValFromHtml( subUID, "Assignd" );
                            addAttrVal[1] = rVal.theString;

                            rVal = m_xDB.getAttrValFromHtml( subUID, "DateTime" );
                            addAttrVal[2] = rVal.theString;

                            p_Log.addAttrs( addAttrVal );
                        }
                    }
                }	
			}

			return res;
		}

		// -------------------------------------
		private Rslt genStat4ExTCs( string p_UID, QNStatData p_TCData, QNStatData p_TC2ReqData, QNStatData p_TCExData, 
			QNStatData p_TRData, string p_ParentSetName ) {

			int i, x, p;
			int avExTime, expDurVal, tcExecs, remExTime;
			int l_TRCnt, l_AvExTime, l_TcExecs, l_ExTime, l_RemExTime, l_EstExTime;

			SetList uidLstTC;
			SetList uidLstTCEx;

			string hasFailed;
			//string tcName;
			string expDur;
			string UID;
			string[] addAttrVal;

			Rslt res = new Rslt( true );
			Rslt xRes;

			l_TRCnt = 0;
			l_EstExTime = 0;
			l_AvExTime = 0;
			l_TcExecs = 0;
			l_ExTime = 0;
			l_RemExTime = 0;

			m_TcCnt = 0;
			m_TCFailed = 0;

			expDurVal = 0;

			uidLstTC = qndm.getSubList( p_UID, "RefCTC" );

			if( res.emptyIsOK( uidLstTC ) ) {

				x = uidLstTC.Count;

				m_TcCnt = x;

				//2007-08-18 p_TCData.getAttrsForAll( uidLstTC );

				if( res.emptyIsOK( p_TCData ) ) {

					for( i = 0; i < x; i++ ) {
						
						UID = uidLstTC[i];

						//tcName = m_xDB.getAttrValFromHtml( UID, "Title" );

						avExTime = 0;
						remExTime = 0;
						m_TRCnt = 0;
						tcExecs = 0;
						m_ExTime = 0;
						
						// care about the TC Executions

						uidLstTCEx = qndm.getSubList( UID, "RefTCExec" );

						xRes = genStat4TCExData( p_TCExData, uidLstTCEx, p_ParentSetName, p_TRData );

						#region TC data -------------------

						if( m_hasFailed ) {

							m_TCFailed++;

							hasFailed = "TRUE";
						}
						else {

							hasFailed = "FALSE";
						}

						tcExecs = uidLstTCEx.Count;

						p_TCData.getAttrsFor( ref UID );

						p = p_TCData.theAttrs.Count - 1;
						expDur = getValFor( "ExpDur", p_TCData, p, ref m_ExpDurPos );

						if( expDur.Length > 0 ) {

							expDurVal = Convert.ToInt32( expDur );
						}

						l_EstExTime += expDurVal;

						if( tcExecs > 0 ) {

							avExTime = m_ExTime / tcExecs;
						}
						else {

							remExTime = expDurVal;
						}

						if( !res.checkRslt( xRes ) ) {

							i = x;
						}
						else {
						
							// headers = new string[] {"TC Set", "TC Execs", "Failed", "Tot Exec Time", "Av Exec Time", "Rem Exec Time", "PRs" };

							addAttrVal = new string[7];
							addAttrVal[0] = p_ParentSetName;
							addAttrVal[1] = tcExecs.ToString();
							addAttrVal[2] = hasFailed;
							addAttrVal[3] = m_ExTime.ToString();
							addAttrVal[4] = avExTime.ToString();
							addAttrVal[5] = remExTime.ToString();
							addAttrVal[6] = m_TRCnt.ToString();

							p_TCData.addAddAttrs( addAttrVal );
						}

						#endregion ---------------------------

						l_TRCnt += m_TRCnt;
						l_AvExTime += avExTime;
						l_TcExecs += tcExecs;
						l_ExTime += m_ExTime;
						l_RemExTime += remExTime;

						// get the linked requests
                        // fragment
						//xRes = genStat4TC2Req( p_TC2ReqData, ref UID );

						res.checkRslt( xRes );
					}
				}
			}

			m_TRCnt = l_TRCnt;
			m_AvExTime = l_AvExTime;
			m_TcExecs = l_TcExecs;
			m_ExTime = l_ExTime;
			m_RemExTime = l_RemExTime;
			m_EstExTime = l_EstExTime;

			return res;
		}

		// --------------------------------------
		private Rslt genStat4ExTCSets( QNStatData p_TCSetData, QNStatData p_TCData, QNStatData p_TC2ReqData, QNStatData p_TCExData,
			QNStatData p_TRData, SetList p_UidLstTCSet, string p_ParentSetName ) {
			
			int i, x;

			int all_TcCnt, all_TCSets, all_TRCnt, all_AvExTime, all_TcExecs, all_ExTime, all_RemExTime, all_TCFailed, all_EstExTime;

			int l_TcCnt,l_TCSets, l_TRCnt, l_AvExTime, l_TcExecs, l_ExTime, l_EstExTime, l_RemExTime, l_TCFailed;

			Rslt res = new Rslt( true );
			Rslt xRes;

			SetList uidLstTCSet;

			string tcsName;
			string UID;
			string[] addAttrVal;
			
			all_AvExTime = 0;
			all_EstExTime = 0;
			all_ExTime = 0;
			all_TRCnt = 0;
			all_RemExTime = 0;
			all_TcCnt = 0;
			all_TcExecs = 0;
			all_TCFailed = 0;
			all_TCSets = 0;

			if( res.emptyIsOK( p_UidLstTCSet ) ) {

				x = p_UidLstTCSet.Count;
				all_TCSets = x;

				// get info for the TC
				for( i = 0; i < x; i++ ) {
					
					UID = p_UidLstTCSet[i];

					uidLstTCSet = qndm.getSubList( UID, "CmpgnTCSet" );

					if( res.emptyIsOK( uidLstTCSet ) ) {

						tcsName = m_xDB.getAttrValFromHtml( UID, "Title" );

						xRes = genStat4ExTCSets( p_TCSetData, p_TCData, p_TC2ReqData, p_TCExData, p_TRData, uidLstTCSet, tcsName );

						if( res.checkRslt( xRes ) ) {							
							// sum all in path, including subpath
							l_AvExTime = m_AvExTime;
							l_EstExTime = m_EstExTime;
							l_ExTime = m_ExTime;
							l_TRCnt = m_TRCnt;
							l_RemExTime = m_RemExTime;
							l_TcCnt = m_TcCnt;
							l_TcExecs = m_TcExecs;
							l_TCFailed = m_TCFailed;
							l_TCSets = m_TCSets;

							p_TCSetData.getAttrsFor( ref UID );

							if( res.emptyIsOK( p_TCSetData ) ) {	

								xRes = genStat4ExTCs( UID, p_TCData, p_TC2ReqData, p_TCExData, p_TRData, tcsName );

								if( res.checkRslt( xRes ) ) {

									l_AvExTime += m_AvExTime;
									l_EstExTime += m_EstExTime;
									l_ExTime += m_ExTime;
									l_TRCnt += m_TRCnt;
									l_RemExTime += m_RemExTime;
									l_TcCnt += m_TcCnt;
									l_TcExecs += m_TcExecs;
									l_TCFailed += m_TCFailed;

									//headers = new string[] {"Parent", "Sub TC Sets", "TCs", "TC Execs", "TC Execs Failed", 
									// "Est Exec Time", "Tot Exec Time", "Av Exec Time", "Rem Exec Time", "PRs"};

									addAttrVal = new string[10];
									addAttrVal[0] = p_ParentSetName;
									addAttrVal[1] = l_TCSets.ToString();
									addAttrVal[2] = l_TcCnt.ToString();
									addAttrVal[3] = l_TcExecs.ToString();
									addAttrVal[4] = l_TCFailed.ToString();
									addAttrVal[5] = l_EstExTime.ToString();
									addAttrVal[6] = l_ExTime.ToString();
									addAttrVal[7] = l_AvExTime.ToString();
									addAttrVal[8] = l_RemExTime.ToString();
									addAttrVal[9] = l_TRCnt.ToString();
			
									p_TCSetData.addAddAttrs( addAttrVal );

									res.checkRslt( p_TCSetData );
								}
							}

							all_AvExTime += l_AvExTime;
							all_EstExTime += l_EstExTime;
							all_ExTime += l_ExTime;
							all_TRCnt += l_TRCnt;
							all_RemExTime += l_RemExTime;
							all_TcCnt += l_TcCnt;
							all_TcExecs += l_TcExecs;
							all_TCFailed += l_TCFailed;
							all_TCSets += l_TCSets;
						}
					}
				}
			}

			m_AvExTime = all_AvExTime;
			m_EstExTime = all_EstExTime;
			m_ExTime = all_ExTime;
			m_TRCnt = all_TRCnt;
			m_RemExTime = all_RemExTime;
			m_TcCnt = all_TcCnt;
			m_TcExecs = all_TcExecs;
			m_TCFailed = all_TCFailed;
			m_TCSets = all_TCSets;

			return res;
		}

		// --------------------------------------
		// RefProbRep* -> liste der ProbRep
		// ProbRep -> RefFix*!
		private Rslt genStat4TCExData( QNStatData p_TCExData, SetList p_UidLstRefTCEx, 
			string p_ParentSetName, QNStatData p_TRData ) {

			int i, x, p;

			Rslt res = new Rslt( true );
			Rslt xRes;

			SetList uidLstTCEx;
			SetList uidLstRefTRs;

			string exTime;
			string errStat;
			string tcName;
			string UID;
			string[] addAttrVal;

			m_hasFailed = false;

			if( res.emptyIsOK( p_UidLstRefTCEx ) ) {

				// get the real execution data
				uidLstTCEx = qndm.getLstForAll( p_UidLstRefTCEx, "RefUID" );

				x = uidLstTCEx.Count;
				m_TcExCnt = x;

				p = p_TCExData.theAddAttrs.Count;

				if( res.emptyIsOK( uidLstTCEx ) ) {

					// get info per Execution
					for( i = 0; i < x; i++ ) {

						UID = uidLstTCEx[i];

						p_TCExData.getAttrsFor( ref UID );

						uidLstRefTRs = qndm.getLstFor( UID, "RefProbRep" );

						if( res.emptyIsOK( uidLstRefTRs ) ) {

							// headers = new string[] {"TC Set","PRs"};

							addAttrVal = new string[2];
							addAttrVal[0] = p_ParentSetName;
							addAttrVal[1] = uidLstRefTRs.Count.ToString();

							p_TCExData.addAddAttrs( addAttrVal );

							p += i;

							exTime = getValFor( "ExcDur", p_TCExData, p, ref m_ExcDurPos );

							if( exTime.Length > 0 ) {

								m_ExTime += Convert.ToInt32( exTime );
							}

							m_TRCnt += uidLstRefTRs.Count;

							errStat = getValFor( "ExCode", p_TCExData, p, ref m_ExcCodePos );

							if( errStat == "er" ) {

								m_hasFailed = true;
							}

							tcName = getValFor( "Title", p_TCExData, p, ref m_ExcTitPos );

							xRes = genStat4TstTRs( p_TRData, uidLstRefTRs, tcName );
						}
					}
				}
			}

			return res;
		}

		// --------------------------------------
        /* incomplete fragment
		private Rslt genStat4TC2Req( QNStatData p_TC2ReqData, ref string p_CTCUID  ) {
			
			int i, x;

			string reqRefUID;
			string tcUID;
			string tcTitle;

			string[] addAttrs = new string[2];

			RsltString rtcUID;
			RsltString rtcTitle;

			Rslt res = new Rslt( true );

			SetList uidLstReqRefs;	

			rtcUID = m_xDB.getAttrVal( p_CTCUID, "RefUID" );

			if( res.checkRslt( rtcUID )) {

				tcUID = rtcUID.theString;

				rtcTitle = m_xDB.getAttrVal( tcUID, "Title" );

				tcTitle = rtcTitle.theString;

				addAttrs[0] = tcTitle;
				addAttrs[1] = tcUID;

				uidLstReqRefs = qndm.getSubList( tcUID, "ReqmntSrc" );

				if( res.emptyIsOK( uidLstReqRefs ) ) {

					x = uidLstReqRefs.Count;

					// get info for the TC
					for( i = 0; i < x; i++ ) {

						reqRefUID = uidLstReqRefs[i];

						p_TC2ReqData.getAttrsFor( ref reqRefUID );

						p_TC2ReqData.addAddAttrs( addAttrs );
					}
				}
			}

			return res;
		}
        */

		// --------------------------------------
		// ProbRep -> RefFix*!
		private Rslt genStat4TstTRs( QNStatData p_TRData, SetList p_UidLstRefTRs, 
			string p_ParentName /*, QNStatData p_FixData */ ) {
			
			int i, x;

			Rslt res = new Rslt( true );

			SetList uidLstRefFix;
			SetList uidLstTRs;

			string UID;
			string[] addAttrVal;

			if( res.emptyIsOK( p_UidLstRefTRs ) ) {

				// get the real execution data
				uidLstTRs = qndm.getLstForAll( p_UidLstRefTRs, "RefUID" );

				if( res.emptyIsOK( uidLstTRs ) ) {

					x = uidLstTRs.Count;

					// get info for the TC
					for( i = 0; i < x; i++ ) {

						UID = uidLstTRs[i];

						p_TRData.getAttrsFor( ref UID );

						uidLstRefFix = qndm.getLstFor( UID, "RefFix" );

						if( res.emptyIsOK( uidLstTRs ) ) {

							// headers = new string[] {"TC", "Fixes"};

							addAttrVal = new string[2];
							addAttrVal[0] = p_ParentName;
							addAttrVal[1] = uidLstRefFix.Count.ToString();

							p_TRData.addAddAttrs( addAttrVal );

							//						xRes = genStat4Fixs( UID, uidLstRefFix, "--" );
						}
					}
				}
			}

			return res;
		}

		// ------------------------------------
		/// <summary>
		/// Retrieve Statistical Data from Fixes
		/// </summary>
		/// <param name="p_TRData">The data bucket</param>
		/// <param name="p_Date"></param>
		/// <param name="p_Time"></param>
		/// <returns></returns>
		public Rslt genStat4Fix( QNStatData p_FxData, QNStatData p_FxFiles, ref string p_DateTime ) {
		
			int i, x, k, y;

			SetList uidSubLst;
			SetList uidLstFxs;
            
            string subUID;
			string UID;
            string[] attrs;
            string[] fileAttrs;
			string[] addAttrVal;
            string[] xHeaders;

			Rslt res = new Rslt( true );

            attrs = new string[] { "UID", "Appl", "Assignd", "DateTime", "DateTimeFix", "DateTimeShip", "Fctn", "FixTstDate", "FixTstSys", "FixStat", "LastMod", "LastModBy", "Lock", "PlanRel", "ProbRel", "ShipRel", "Ticket", "Title" };

            fileAttrs = new string[] { "UID", "Author", "Branch", "DateTime", "Label", "Name", "Path", "Rel", "Section", "SrcSrv" /*, "Cmnt" requires removing of semicolons and CR/LF */};

            xHeaders = new string[] { "Fix UID" };

			uidLstFxs = getFixLst( false );

			if( res.checkRslt( uidLstFxs ) ) {

                p_FxData.setUpAttrs( attrs );

				//p_TRData.createAddAttrs( xHeaders ) ;

                p_FxFiles.setUpAttrs( fileAttrs );
                p_FxFiles.createAddAttrs( xHeaders ) ;
			}

			x = uidLstFxs.Count;

			for(i = 0; i < x; i++) 	{

				UID = uidLstFxs[i];

				res = p_FxData.getAttrFor( UID );

                // Fix file data
                if(res.isOK())
                {

                    uidSubLst = m_xDB.getUIDLstByParent( "FixLst", UID );

                    if( uidSubLst.Count > 0 ) {

                        subUID = uidSubLst[0];

                        uidSubLst = m_xDB.getUIDLstByParent( "FixEntry", subUID );

                        y = uidSubLst.Count;

                        for(k = 0; k < y; k++) {

                            subUID = uidSubLst[k];

                            p_FxFiles.getAttrFor( subUID );

                            addAttrVal = new string[] { UID };

                            p_FxFiles.addAddAttrs( addAttrVal );
                        }
                    }
                }	
			}

			return res;
		}

        /// <summary>
        /// Retrieve Statistical Data from Issues
        /// </summary>
        /// <param name="p_Data"></param>
        /// <param name="p_Log"></param>
        /// <param name="p_DateTime"></param>
        /// <param name="p_All"></param>
        /// <returns></returns>
		public Rslt genStat4Iss( QNStatData p_Data, QNStatData p_Log, ref string p_DateTime, bool p_All ) {
		
			int i, x, k, y;
			//RsltTable nVals;
            RsltString rVal;

			SetList uidSubLst;
			SetList uidLst;
            
            string subUID;
			string UID;
            string val;
            string[] attrs;
            string[] logAttrs;
            //string[] xHeaders;
			string[] addAttrVal;

			Rslt res = new Rslt( true );

            attrs = new string[] { "UID", "Assignd", "Author", "DateTime", "IssNum", "IssStat", "IssType", "LastMod", "LastModBy", "Lock", "Prio", "Proj", "Reqstr", "Title" };

            logAttrs = new string[] { "Issue UID", "Assignd", "DateTime", "NStat" };

			uidLst = getIssLst( !(p_All) );

			if( res.checkRslt( uidLst ) ) {

                p_Data.setUpAttrs( attrs );

                p_Log.setUpAttrs( logAttrs );
			}

			x = uidLst.Count;

			for(i = 0; i < x; i++) 	{

				UID = uidLst[i];

				res = p_Data.getAttrFor( UID );

                // Log data
                if(res.isOK())
                {
                    uidSubLst = m_xDB.getUIDLstByParent( "ProcLog", UID );

                    y = uidSubLst.Count;

                    for(k = 0; k < y; k++)
                    {
                        addAttrVal = new string[4];

                        subUID = uidSubLst[k];

                        rVal = m_xDB.getAttrValFromHtml( subUID, "NStat" );

                        val = rVal.theString;

                        if(val.Length > 0)
                        {
                            addAttrVal[0] = UID;

                            addAttrVal[3] = val;

                            rVal = m_xDB.getAttrValFromHtml( subUID, "Assignd" );
                            addAttrVal[1] = rVal.theString;

                            rVal = m_xDB.getAttrValFromHtml( subUID, "DateTime" );
                            addAttrVal[2] = rVal.theString;

                            p_Log.addAttrs( addAttrVal );
                        }
                    }
                }	
			}

			return res;
		}

        /// <summary>
        /// Retrieve Statistical Data from Problem Reports.
        /// </summary>
        /// <param name="p_TRData"></param>
        /// <param name="p_TRLog"></param>
        /// <param name="p_DateTime"></param>
        /// <param name="p_All"></param>
        /// <returns></returns>
		public Rslt genStat4TRs( QNStatData p_TRData, QNStatData p_TRLog, ref string p_DateTime, bool p_All ) {
		
			int i, x, k, y;
			//RsltTable nVals;
            RsltString rVal;

			SetList uidSubLst;
			SetList uidLstTRs;
            
            string subUID;
			string UID;
            string val;
            string[] attrs;
            string[] logAttrs;
            string[] xHeaders;
			string[] addAttrVal;

			Rslt res = new Rslt( true );

            //attrs = new string[] { "UID", "Appl", "Assignd", "Author", "DateTime", "EstDate", "ExSys", "Fctn", "LastMod", "LastModBy", "Lock", "Prio", "ProbClass", "ProbCode", "ProbProcStat", "ProbStat", "Reqstr", "SoftRel", "Ticket", "Title" };
            attrs = new string[] { "UID", "Appl", "Assignd", "Author", "CRNum", "DateTime", "EstDate", "ExSys", "ExtID", "Fctn", "IsCR", "LastMod", "LastModBy", "Lock", "Prio", "ProbClass", "ProbCode", "ProbProcStat", "ProbStat", "PSource", "Reqstr", "SoftRel", "Ticket", "Title", "UseCase", "UCSection" };

            logAttrs = new string[] { "PR UID", "Assignd", "DateTime", "NStat" };

			//xHeaders = new string[] { "WDYD", "WWE", "WWO" }; 2008-09-23 no good, CRs in Text spoils CSV!
			// Branch:TCom PSys ##################
			xHeaders = new string[] { "PSystem", "PVersion", "POrigin", "LastShip" }; 

			uidLstTRs = getTRLst( !(p_All) );

			//2008-09-24 replace to add other data p_TRData.getAttrsForAll( uidLstTRs );

			if( res.checkRslt( uidLstTRs ) ) {

				//former generic version 2008-12-23 p_TRData.setUpAttrs( uidLstTRs[0] );
                p_TRData.setUpAttrs( attrs );

				// Branch:TCom PSys ##################
				p_TRData.createAddAttrs( xHeaders ) ;

                p_TRLog.setUpAttrs( logAttrs );
			}

			x = uidLstTRs.Count;

			for(i = 0; i < x; i++) 	{

				UID = uidLstTRs[i];

				res = p_TRData.getAttrFor( UID );

				// Branch:TCom PSys ##################

				/* Problem with carriage-returns in text: they break the csv!
				if( res.isOK() ) {

						addAttrVal = new string[] { "", "", "" };

						nVals = m_xDB.getSubNodeVals( UID, QNDefs.nodeWDYD );

						if( nVals.Count > 0 ) {

							addAttrVal[0] = (nVals[0])[0];
						}

						nVals = m_xDB.getSubNodeVals( UID, QNDefs.nodeExpctd );

						if(nVals.Count > 0) {

							addAttrVal[1] = ( nVals[0] )[0];
						}

						nVals = m_xDB.getSubNodeVals( UID, QNDefs.nodeObserv );

						if(nVals.Count > 0) {

							addAttrVal[2] = ( nVals[0] )[0];
						}

						p_TRData.addAddAttrs( addAttrVal );
				}
				*/

				// Branch:TCom PSys ##################
				if(res.isOK())
				{

					addAttrVal = new string[] { "", "", "", "" };

					uidSubLst = m_xDB.getUIDLstByParent( "PSys", UID );

					if( uidSubLst.Count > 0 ) {

						subUID = uidSubLst[0];

						rVal = m_xDB.getAttrValFromHtml( subUID, "PSystem" );
						addAttrVal[0] = rVal.theString;

						rVal = m_xDB.getAttrValFromHtml( subUID, "Version" );
						addAttrVal[1] = rVal.theString;

						rVal = m_xDB.getAttrValFromHtml( subUID, "Origin" );
						addAttrVal[2] = rVal.theString;
					}

                    addAttrVal[3] = getLastFixShpRel( UID );

					p_TRData.addAddAttrs( addAttrVal );
				}
				// Branch:TCom PSys End ##################

                // Log data
                if(res.isOK())
                {
                    uidSubLst = m_xDB.getUIDLstByParent( "ProcLog", UID );

                    y = uidSubLst.Count;

                    for(k = 0; k < y; k++)
                    {
                        addAttrVal = new string[4];

                        subUID = uidSubLst[k];

                        rVal = m_xDB.getAttrValFromHtml( subUID, "NStat" );

                        val = rVal.theString;

                        if(val.Length > 0)
                        {
                            addAttrVal[0] = UID;

                            addAttrVal[3] = val;

                            rVal = m_xDB.getAttrValFromHtml( subUID, "Assignd" );
                            addAttrVal[1] = rVal.theString;

                            rVal = m_xDB.getAttrValFromHtml( subUID, "DateTime" );
                            addAttrVal[2] = rVal.theString;

                            p_TRLog.addAttrs( addAttrVal );
                        }
                    }
                }	
			}

			return res;
		}

        /// <summary>
        /// 
        /// </summary>
        /// <param name="p_WOCl"></param>
        /// <returns></returns>
		public SetList getCRLst( bool p_WOCl ) {

			SetList theLst;
			SetList clsdLst;

			theLst = qndm.getLstFor( "ChgReq" );

			if( p_WOCl ) {
				
				clsdLst = qndm.getLstWhereAttr( "ChgReq", "CRStat", "cl" );
				theLst.subtract( clsdLst );
			}

			return theLst;
		}

    	// ----------------------
		/// <summary>
		/// Retrieves the list of all TRs w/o the closed ones.
		/// </summary>
		/// <returns></returns>
		public SetList getFixLst( bool p_WOCl ) {

			SetList theLst;
			SetList clsdLst;

			theLst = qndm.getLstFor( "Fix" );

			if( p_WOCl ) {
				
				clsdLst = qndm.getLstWhereAttr( "Fix", "FixStat", "cl" );
				theLst.subtract( clsdLst );
			}

			return theLst;
		}

		// ----------------------
		/// <summary>
		/// Retrieves the list of all Issues w/o the closed ones.
		/// </summary>
		/// <returns></returns>
		public SetList getIssLst( bool p_WOCl ) {

			SetList theLst;
			SetList clsdLst;

			theLst = qndm.getLstFor( "IssReport" );

			if( p_WOCl ) {
				
				clsdLst = qndm.getLstWhereAttr( "IssReport", "IssStat", "cl" );
				theLst.subtract( clsdLst );
			}

			return theLst;
		}

        // ----------------------
        /// <summary>
        /// 
        /// </summary>
        /// <param name="p_UID"></param>
        /// <returns></returns>
        public RsltString getLastFixShpRel( String p_UID ) {
        	
	        RsltString rel = new RsltString();
        	
	        int i, x;
        	
	        Rslt2DArray relLst;
        	
	        SetList uidLst;
	        String aUID;
        	
	        String[] row;
        	
	        rel.setRsltStat( RsltStatVal.isEmpty );
        	
	        uidLst = m_xDB.getUIDLstByParent( QNDefs.nodeRefFix, p_UID );
        	
	        x = uidLst.Count;
        	
	        relLst = new Rslt2DArray( 2, x );
        	
	        for( i = 0; i < x; ++i ) {
        		
		        aUID = uidLst[ i ];
        		
		        aUID = m_xDB.getAttrVal( aUID, QNDefs.attrRefUID ).theString;
        		
		        row = new String[2];
        		
		        row[0] = m_xDB.getAttrVal( aUID, QNDefs.attrDateTimeShip ).theString;
		        row[1] = m_xDB.getAttrVal( aUID, QNDefs.attrShipRel ).theString;
        		
		        relLst.Add( row );
	        }
        	
	        if( x > 0 ) {
        		
		        relLst.sort( 0, true, false );

                rel.theString = relLst.getCell( 1, 0 );
	        }
        	
	        return rel;
        }

		// ----------------------
		/// <summary>
		/// Retrieves the list of all TRs w/o the closed ones.
		/// </summary>
		/// <returns></returns>
		public SetList getTRLst( bool p_WOCl ) {

			SetList theLst;
			SetList clsdLst;

			theLst = qndm.getLstFor( "ProbReport" );

			if( p_WOCl ) {
				
				clsdLst = qndm.getLstWhereAttr( "ProbReport", "ProbStat", "cl" );
				theLst.subtract( clsdLst );
			}

			return theLst;
		}

		// -------------------
		private string getValFor( string p_AttrName, QNStatData p_Data, int p_Pos, ref int p_AttrPos ) {

			string val;

			if( p_AttrPos == -1 ) {

				p_AttrPos = p_Data.getPosOf( ref p_AttrName );
			}

			val = p_Data.theAttrs[p_Pos][p_AttrPos];

			//val = p_Data.getLastRecAttr( p_Pos );

			return val;

		}
	}

	// ########################################################################################
	/// <summary>
	/// Summary description for QNStatData.
	/// </summary>
	public class  QNStatData : Rslt {

		private RsltTable m_Attrs;
		private RsltTable m_AddAttrs;

		private XMLDB m_xDB;

		// ---------------------------
		public QNStatData( XMLDB p_xDB ) {

			string[] empty = new string[0];
			m_xDB = p_xDB;
			m_Attrs = new RsltTable( empty );
			m_AddAttrs = new RsltTable( empty );
			setRsltStat( RsltStatVal.isEmpty );
		}

		// ---------------------------
		public void addAddAttrs( string[] p_AddAttrs ) {

			m_AddAttrs.Add(p_AddAttrs);
		}

		// ---------------------------
		public void addAttrs( string[] p_Attrs ) {

			if( m_ResStat == RsltStatVal.isEmpty ) {

				setOK();
			}

			m_Attrs.Add( p_Attrs );
		}

		// ---------------------------
		public void addAttrs( RsltTable p_Attrs ) {

			if( m_ResStat == RsltStatVal.isEmpty ) {

				setOK();
			}

			if( checkRslt( p_Attrs ) ) {

				if( m_Attrs.m_ColNames.Length == 0 ) {

					m_Attrs = p_Attrs;
				}
				else {

					m_Attrs.merge( p_Attrs );
				}
			}
		}

		// -----------------------------
		/// <summary>
		/// Adds the list of attributes for the given UID.
		/// Drops UID attributes.
		/// </summary>
		/// <param name="p_UID">List of UIDs to data sets</param>
		public void getAttrsFor( ref string p_UID ) {

            /* 2008-12-23 no longer supported
			if( m_Attrs.m_ColNames.Length == 0 ) {

				setUpAttrs( p_UID );
			} */

			getAttrFor( p_UID );
		}
	
		// -----------------------------
		/// <summary>
		/// Adds the list of attributes for all given data sets.
		/// Drops UID attributes.
		/// </summary>
		/// <param name="p_UIDList">List of UIDs to data sets</param>
		public void getAttrsForAll( SetList p_UIDList ) {

			if( p_UIDList.Count > 0 ) {

                /* discontinued 2008-12-23
				if( m_Attrs.m_ColNames.Length == 0 ) {

					setUpAttrs( p_UIDList[0] );
				}*/

				getAttrForList( p_UIDList );
			}
		}

		// ------------------
		/// <summary>
		/// Retrieve attribute values by given list of attributes
		/// </summary>
		/// <param name="p_UIDLst">List of UID</param>
		/// <param name="p_AttrNameLst">List of attributes</param>
		/// <returns></returns>
		public Rslt getAttrFor( string p_UID ) {

			int i, x;

			Rslt res = new Rslt( true );

			RsltString val = new RsltString();

			string[] rowAttr;

			x = m_Attrs.m_ColNames.Length;

			setOK();

			rowAttr = new string[x];

			rowAttr[0] = p_UID;

			for( i = 1; i < x; i++) {

				val = m_xDB.getAttrValFromHtml( p_UID, m_Attrs.m_ColNames[i] );

				if( res.emptyIsOK( val ) ) {

					rowAttr[i] = val.theString;
				}
				else {
			
					i = x;
				}
			}

			m_Attrs.Add( rowAttr );

			m_Attrs.emptyIsOK( res );

            if(!( emptyIsOK( res ) ))
            {

                res.addRsltAddInfo( "QNStatData.getAttrFor(..) failed." );
            }

			return res; 
		}

		// ------------------
		/// <summary>
		/// Retrieve attribute values by given list of attributes
		/// </summary>
		/// <param name="p_UIDLst">List of UID</param>
		/// <param name="p_AttrNameLst">List of attributes</param>
		/// <returns></returns>
		private void getAttrForList( SetList p_UIDLst ) {

			int i, x;
			Rslt res;

			x = p_UIDLst.Count;

			for( i = 0; i < x; i++) {

				res = getAttrFor( p_UIDLst[i] );

				if( res.notOK() ) {

					i = x;
				}
			}
		}

		// --------------------------
		public string getLastRecAttr( int p_Pos ) {

			int r;

			string val;
			string[] row;

			r = m_Attrs.Count -1;

			row = m_Attrs[r];

			val = row[ p_Pos ];

			return val;
		}


		// --------------------------
		public int getPosOf( ref string p_AttrName ) {

			int i, x, pos;

			pos = -1;

			x = m_Attrs.m_ColNames.Length;

			for( i = 0; i < x; ++i ){

				if( m_Attrs.m_ColNames[i] == p_AttrName ) {

					pos = i;

					i = x;
				}
			}

			return pos;
		}

		// --------------------------
		public override bool isOK() {

			bool isOK = true;

			if( m_ResStat != RsltStatVal.isEmpty ) {

				isOK = base.isOK();
			}

			return isOK;
		}

        // -----------------------
		public void setUpAttrs( string[] p_Attrs ) {

			m_Attrs = new RsltTable( p_Attrs );
		}


		// ---------------------------
        /* former generic version
		public void setUpAttrs( string p_UID ) {

			int i, j, x;
			string attrName;
			string[] attrNameLst;
			string[] shortNameLst;

			RsltTable attrLst;

			attrLst = m_xDB.getNodeAttrs( ref p_UID );

			x = attrLst.Count;

			attrNameLst = new string[x+1];

			j = 1;

			attrNameLst[0] = "UID";

			// skip other UIDs, put 'Title' in second column
			for( i = 0; i < x; i++ ) { 

				attrName = attrLst.getCell(0, i);

				//2008-04-17 if( attrName.IndexOf( "UID" ) < 0 ) {
						
					if( attrName == "Title" ) {

						attrNameLst[j] = attrNameLst[1];

						attrNameLst[1] = attrName;
					}
					else {

						attrNameLst[j] = attrName;
					}

					j++;
				//}
			}

			x = j;

			// drop the unwanted attributes

			shortNameLst = new string[x];

			for( i = 0; i < x; i++ ) { 

				shortNameLst[i] = attrNameLst[i];
			}

			m_Attrs = new RsltTable( shortNameLst );
		}*/

		// ---------------------------
		public RsltTable theAddAttrs { 
			
			get{ 

				return m_AddAttrs;
			}
		}

		// ---------------------------
		public RsltTable theAttrs { 
			
			get{

				return m_Attrs;
			}
        }

		// ---------------------------
		public void createAddAttrs( string[] p_Headers ) {

			m_AddAttrs = new RsltTable( p_Headers );
		}
	}
}
