//---------------------------------------------------------
// QA Navigator suite
// Copyright  2003,2004 Ronald D. Grindle
//---------------------------------------------------------

using System;
using System.Collections;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.IO;
using System.Reflection;
using System.Threading;
using System.Windows.Forms;
using System.Xml;

using RG.Basics;
using RG.SQL;
using RG.XMLDB;

using QN.Basics;

namespace QN.TCEdit {

	/// <summary>
	/// QA Navigator suite
	/// Copyright  2003 - 2008 Ronald D. Grindle
	/// 
	/// QA Navigator test case editor GUI
	/// Dialog to specify Requirement has been replaced by ReqsSelector, fed from a list.
	/// </summary>
	public class frmTCEd : System.Windows.Forms.Form {

		#region my members

		private int m_butSaveNewDT, m_txtStatDT, m_comboLogDT, m_panel1DH;
		
		private QNGUIservice m_GUIservice;
		private QNphrase m_QNphrase;
		private QNpresets m_Presets;
		private QNTools m_QNTools;

		private Ref2DocLst m_PrecTCLst;
		private Ref2DocValLst m_ReqsLst;
		private Ref2FileLst m_TProcLst;
		private Ref2FileLst m_TRefLst;

		/* name, version, refUID */

		private static SimpleLog m_Logger;
		private string m_loadUID;

		private	TCSelPresets m_TCSelPresets;
		
		private string m_uidDesc;
		private string m_uidExpRslt;
		private string m_uidTC;
		private string m_uidTstProc;
		private string m_uidTSysConf;
		
        private RGSQL2 m_DB;
		private XMLDB m_xDB;

		delegate Rslt dlgtFileDialOK( ref string p_FileName );

		//delegate void dlgtOpenFileDial( object p_Sender, System.ComponentModel.CancelEventArgs p_E);

		#endregion

		#region Form Resources

		private System.Windows.Forms.MenuItem menuNew;
		private System.Windows.Forms.MainMenu mainMenu;
		private System.Windows.Forms.MenuItem menuSave;
		private System.Windows.Forms.MenuItem menuOpen;
		private System.Windows.Forms.MenuItem menuLine1;
		private System.Windows.Forms.Label lblAuthor;
		private System.Windows.Forms.MenuItem menuFile;
		private System.Windows.Forms.TextBox txtAuthor;
		private System.Windows.Forms.Label lblTitle;
		private System.Windows.Forms.TextBox txtTitle;
		private System.Windows.Forms.Label lblPrecTC;
		private System.Windows.Forms.ListBox lstPrecTC;
		private System.Windows.Forms.Label lblDate;
		private System.Windows.Forms.TextBox txtDate;
		private System.Windows.Forms.Label lblTCType;
		private System.Windows.Forms.ComboBox comboTCType;
		private System.Windows.Forms.Label lblTime;
		private System.Windows.Forms.TextBox txtTime;
		private System.Windows.Forms.Label lblPrio;
		private System.Windows.Forms.ComboBox comboTCPriority;
		private System.Windows.Forms.Label lblTSysConfig;
		private System.Windows.Forms.Label lblDesc;
		private System.Windows.Forms.TextBox txtDesc;
		private System.Windows.Forms.MenuItem menuHelp;
		private System.Windows.Forms.MenuItem menuAbout;
		private System.Windows.Forms.TextBox txtTstProc;
		private System.Windows.Forms.TextBox txtExpRslt;
		private System.Windows.Forms.TextBox txtExpDuration;
		private System.Windows.Forms.Button butSaveNew;
		private System.Windows.Forms.GroupBox grpTCAttr;
		private System.Windows.Forms.GroupBox grpEditInfo;
		private System.Windows.Forms.Label lblTCVer;
		private System.Windows.Forms.TextBox txtTCVer;
		private System.Windows.Forms.Panel panel1;
		private System.Windows.Forms.TextBox txtStat;
		private System.Windows.Forms.ComboBox comboLog;
		private System.Windows.Forms.ContextMenu ctxtMenuPrecTC;
		private System.Windows.Forms.MenuItem ctxtPrecTCAdd;
		private System.Windows.Forms.MenuItem ctxtPrecTCRem;
		private System.Windows.Forms.ContextMenu ctxtMenuTSysConf;
		private System.Windows.Forms.MenuItem ctxtTSysSel;
		private System.Windows.Forms.Label lblExpDur;
		private System.Windows.Forms.MenuItem menuExport;
		private System.Windows.Forms.MenuItem menuImport;
		private System.Windows.Forms.MenuItem menuItem1;
		private System.Windows.Forms.MenuItem menuExit;
		private System.Windows.Forms.MenuItem menuDel;
		private System.Windows.Forms.ContextMenu ctxtMenuLog;
		private System.Windows.Forms.MenuItem ctxtLogEntry;
		private System.Windows.Forms.Label lblAppl;
		private System.Windows.Forms.Label lblSection;
		private System.Windows.Forms.ComboBox comboAppl;
		private System.Windows.Forms.ComboBox comboSection;
		private System.Windows.Forms.GroupBox grpTCid;
		private System.Windows.Forms.Label lblFctn;
		private System.Windows.Forms.TextBox txtFctn;
		private System.Windows.Forms.ComboBox comboTSysConf;
		private System.Windows.Forms.Label lblSoftRel;
		private System.Windows.Forms.MenuItem menuItem2;
		private System.Windows.Forms.MenuItem menuItem3;
		private System.Windows.Forms.MenuItem menuDocu;
		private System.Windows.Forms.MenuItem menuItem5;
		private System.Windows.Forms.MenuItem menuItem4;
		private System.Windows.Forms.MenuItem menuPrint;
		private System.Windows.Forms.MenuItem menuUID2Clip;
		private System.Windows.Forms.ListBox lstReqs;
		private System.Windows.Forms.Label lblReqs;
		private System.Windows.Forms.ContextMenu ctxtMenuReqs;
		private System.Windows.Forms.MenuItem ctxtReqsEdit;
		private System.Windows.Forms.ContextMenu ctxtMenuAddFile;
		private System.Windows.Forms.MenuItem ctxtAddFileAdd;
		private System.Windows.Forms.MenuItem ctxtAddFileRem;
    private System.Windows.Forms.ListBox lstTstProcFile;
		private System.Windows.Forms.TabControl tabCtrlTstProc;
		private System.Windows.Forms.TabPage tabTstProcTxt;
		private System.Windows.Forms.TabPage tabTstProcFile;
		private System.Windows.Forms.ContextMenu ctxtMenuProcFile;
		private System.Windows.Forms.MenuItem ctxtProcAdd;
		private System.Windows.Forms.MenuItem ctxtProcRem;
		private System.Windows.Forms.ComboBox comboApplRel;
		private System.Windows.Forms.TabControl tabCtrlRslt;
		private System.Windows.Forms.TabPage tabRsltTxt;
		private System.Windows.Forms.TabPage tabRsltFile;

		#endregion

		private System.Windows.Forms.ToolTip toolTipTitle;
		private System.Windows.Forms.ListBox lstTstRefFile;
		private System.Windows.Forms.ContextMenu ctxtMenuRefFile;
		private System.Windows.Forms.MenuItem ctxtRefAdd;
		private System.Windows.Forms.MenuItem ctxtRefRem;
		private System.Windows.Forms.MenuItem menuSaveNNew;
		private System.Windows.Forms.MenuItem menuSaveAsNew;
		private System.ComponentModel.IContainer components;

		#region window construction/diposal

		public frmTCEd() {

			try {
				
				m_Logger = new SimpleLog( "TCEditor" );

				InitializeComponent();

				init();
			}
			catch( Exception e ) {
		
				handleExcp( e );
			}
		}

		// ------------------
		public frmTCEd( string p_UID ) {

			try {

				Rslt res;
				
				m_Logger = new SimpleLog( "TCEditor" );

				InitializeComponent();

				res = init();

				if( res.isOK() ) {

					loadTC( p_UID );
				}
			}
			catch( Exception e ) {
		
				handleExcp( e );
			}
		}

		/// <summary>
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing ) {
			if( disposing ) {
				if (components != null) {
					components.Dispose();
				}
                m_DB.close();
			}
			base.Dispose( disposing );
		}
		#endregion

		#region Windows Form Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent() {
			this.components = new System.ComponentModel.Container();
			System.ComponentModel.ComponentResourceManager resources = new System.ComponentModel.ComponentResourceManager( typeof( frmTCEd ) );
			this.mainMenu = new System.Windows.Forms.MainMenu( this.components );
			this.menuFile = new System.Windows.Forms.MenuItem();
			this.menuNew = new System.Windows.Forms.MenuItem();
			this.menuOpen = new System.Windows.Forms.MenuItem();
			this.menuItem2 = new System.Windows.Forms.MenuItem();
			this.menuSave = new System.Windows.Forms.MenuItem();
			this.menuSaveAsNew = new System.Windows.Forms.MenuItem();
			this.menuSaveNNew = new System.Windows.Forms.MenuItem();
			this.menuUID2Clip = new System.Windows.Forms.MenuItem();
			this.menuItem3 = new System.Windows.Forms.MenuItem();
			this.menuDel = new System.Windows.Forms.MenuItem();
			this.menuLine1 = new System.Windows.Forms.MenuItem();
			this.menuExport = new System.Windows.Forms.MenuItem();
			this.menuImport = new System.Windows.Forms.MenuItem();
			this.menuItem4 = new System.Windows.Forms.MenuItem();
			this.menuPrint = new System.Windows.Forms.MenuItem();
			this.menuItem1 = new System.Windows.Forms.MenuItem();
			this.menuExit = new System.Windows.Forms.MenuItem();
			this.menuHelp = new System.Windows.Forms.MenuItem();
			this.menuDocu = new System.Windows.Forms.MenuItem();
			this.menuItem5 = new System.Windows.Forms.MenuItem();
			this.menuAbout = new System.Windows.Forms.MenuItem();
			this.lblAuthor = new System.Windows.Forms.Label();
			this.txtAuthor = new System.Windows.Forms.TextBox();
			this.lblTitle = new System.Windows.Forms.Label();
			this.txtTitle = new System.Windows.Forms.TextBox();
			this.lblPrecTC = new System.Windows.Forms.Label();
			this.lstPrecTC = new System.Windows.Forms.ListBox();
			this.ctxtMenuPrecTC = new System.Windows.Forms.ContextMenu();
			this.ctxtPrecTCAdd = new System.Windows.Forms.MenuItem();
			this.ctxtPrecTCRem = new System.Windows.Forms.MenuItem();
			this.lblDate = new System.Windows.Forms.Label();
			this.txtDate = new System.Windows.Forms.TextBox();
			this.lblTCType = new System.Windows.Forms.Label();
			this.comboTCType = new System.Windows.Forms.ComboBox();
			this.lblTime = new System.Windows.Forms.Label();
			this.txtTime = new System.Windows.Forms.TextBox();
			this.lblPrio = new System.Windows.Forms.Label();
			this.comboTCPriority = new System.Windows.Forms.ComboBox();
			this.lblReqs = new System.Windows.Forms.Label();
			this.lstReqs = new System.Windows.Forms.ListBox();
			this.ctxtMenuReqs = new System.Windows.Forms.ContextMenu();
			this.ctxtReqsEdit = new System.Windows.Forms.MenuItem();
			this.lblTSysConfig = new System.Windows.Forms.Label();
			this.lblDesc = new System.Windows.Forms.Label();
			this.txtDesc = new System.Windows.Forms.TextBox();
			this.txtTstProc = new System.Windows.Forms.TextBox();
			this.txtExpRslt = new System.Windows.Forms.TextBox();
			this.lblExpDur = new System.Windows.Forms.Label();
			this.txtExpDuration = new System.Windows.Forms.TextBox();
			this.butSaveNew = new System.Windows.Forms.Button();
			this.grpTCAttr = new System.Windows.Forms.GroupBox();
			this.comboTSysConf = new System.Windows.Forms.ComboBox();
			this.ctxtMenuTSysConf = new System.Windows.Forms.ContextMenu();
			this.ctxtTSysSel = new System.Windows.Forms.MenuItem();
			this.grpEditInfo = new System.Windows.Forms.GroupBox();
			this.txtTCVer = new System.Windows.Forms.TextBox();
			this.lblTCVer = new System.Windows.Forms.Label();
			this.panel1 = new System.Windows.Forms.Panel();
			this.tabCtrlRslt = new System.Windows.Forms.TabControl();
			this.tabRsltTxt = new System.Windows.Forms.TabPage();
			this.tabRsltFile = new System.Windows.Forms.TabPage();
			this.lstTstRefFile = new System.Windows.Forms.ListBox();
			this.ctxtMenuRefFile = new System.Windows.Forms.ContextMenu();
			this.ctxtRefAdd = new System.Windows.Forms.MenuItem();
			this.ctxtRefRem = new System.Windows.Forms.MenuItem();
			this.tabCtrlTstProc = new System.Windows.Forms.TabControl();
			this.tabTstProcTxt = new System.Windows.Forms.TabPage();
			this.tabTstProcFile = new System.Windows.Forms.TabPage();
			this.lstTstProcFile = new System.Windows.Forms.ListBox();
			this.ctxtMenuProcFile = new System.Windows.Forms.ContextMenu();
			this.ctxtProcAdd = new System.Windows.Forms.MenuItem();
			this.ctxtProcRem = new System.Windows.Forms.MenuItem();
			this.grpTCid = new System.Windows.Forms.GroupBox();
			this.comboApplRel = new System.Windows.Forms.ComboBox();
			this.lblSoftRel = new System.Windows.Forms.Label();
			this.txtFctn = new System.Windows.Forms.TextBox();
			this.lblFctn = new System.Windows.Forms.Label();
			this.lblSection = new System.Windows.Forms.Label();
			this.lblAppl = new System.Windows.Forms.Label();
			this.comboSection = new System.Windows.Forms.ComboBox();
			this.comboAppl = new System.Windows.Forms.ComboBox();
			this.ctxtMenuAddFile = new System.Windows.Forms.ContextMenu();
			this.ctxtAddFileAdd = new System.Windows.Forms.MenuItem();
			this.ctxtAddFileRem = new System.Windows.Forms.MenuItem();
			this.txtStat = new System.Windows.Forms.TextBox();
			this.comboLog = new System.Windows.Forms.ComboBox();
			this.ctxtMenuLog = new System.Windows.Forms.ContextMenu();
			this.ctxtLogEntry = new System.Windows.Forms.MenuItem();
			this.toolTipTitle = new System.Windows.Forms.ToolTip( this.components );
			this.grpTCAttr.SuspendLayout();
			this.grpEditInfo.SuspendLayout();
			this.panel1.SuspendLayout();
			this.tabCtrlRslt.SuspendLayout();
			this.tabRsltTxt.SuspendLayout();
			this.tabRsltFile.SuspendLayout();
			this.tabCtrlTstProc.SuspendLayout();
			this.tabTstProcTxt.SuspendLayout();
			this.tabTstProcFile.SuspendLayout();
			this.grpTCid.SuspendLayout();
			this.SuspendLayout();
			// 
			// mainMenu
			// 
			this.mainMenu.MenuItems.AddRange( new System.Windows.Forms.MenuItem[] {
            this.menuFile,
            this.menuHelp} );
			// 
			// menuFile
			// 
			this.menuFile.Index = 0;
			this.menuFile.MenuItems.AddRange( new System.Windows.Forms.MenuItem[] {
            this.menuNew,
            this.menuOpen,
            this.menuItem2,
            this.menuSave,
            this.menuSaveAsNew,
            this.menuSaveNNew,
            this.menuUID2Clip,
            this.menuItem3,
            this.menuDel,
            this.menuLine1,
            this.menuExport,
            this.menuImport,
            this.menuItem4,
            this.menuPrint,
            this.menuItem1,
            this.menuExit} );
			this.menuFile.Text = "File";
			// 
			// menuNew
			// 
			this.menuNew.Index = 0;
			this.menuNew.Text = "New";
			this.menuNew.Click += new System.EventHandler( this.menuNew_Click );
			// 
			// menuOpen
			// 
			this.menuOpen.Index = 1;
			this.menuOpen.Text = "Open";
			this.menuOpen.Click += new System.EventHandler( this.menuOpen_Click );
			// 
			// menuItem2
			// 
			this.menuItem2.Index = 2;
			this.menuItem2.Text = "-";
			// 
			// menuSave
			// 
			this.menuSave.Index = 3;
			this.menuSave.Shortcut = System.Windows.Forms.Shortcut.CtrlS;
			this.menuSave.Text = "Save";
			this.menuSave.Click += new System.EventHandler( this.menuSave_Click );
			// 
			// menuSaveAsNew
			// 
			this.menuSaveAsNew.Index = 4;
			this.menuSaveAsNew.Shortcut = System.Windows.Forms.Shortcut.CtrlA;
			this.menuSaveAsNew.Text = "Save As New";
			this.menuSaveAsNew.Click += new System.EventHandler( this.menuSaveNew_Click );
			// 
			// menuSaveNNew
			// 
			this.menuSaveNNew.Index = 5;
			this.menuSaveNNew.Text = "save && new";
			this.menuSaveNNew.Click += new System.EventHandler( this.butSaveNew_Click );
			// 
			// menuUID2Clip
			// 
			this.menuUID2Clip.Index = 6;
			this.menuUID2Clip.Text = "uid to Clip";
			this.menuUID2Clip.Click += new System.EventHandler( this.menuUID2Clip_Click );
			// 
			// menuItem3
			// 
			this.menuItem3.Index = 7;
			this.menuItem3.Text = "-";
			// 
			// menuDel
			// 
			this.menuDel.Index = 8;
			this.menuDel.Text = "Delete";
			this.menuDel.Click += new System.EventHandler( this.menuDel_Click );
			// 
			// menuLine1
			// 
			this.menuLine1.Index = 9;
			this.menuLine1.Text = "-";
			// 
			// menuExport
			// 
			this.menuExport.Index = 10;
			this.menuExport.Text = "Export";
			this.menuExport.Click += new System.EventHandler( this.menuExport_Click );
			// 
			// menuImport
			// 
			this.menuImport.Index = 11;
			this.menuImport.Text = "Import";
			this.menuImport.Click += new System.EventHandler( this.menuImport_Click );
			// 
			// menuItem4
			// 
			this.menuItem4.Index = 12;
			this.menuItem4.Text = "-";
			// 
			// menuPrint
			// 
			this.menuPrint.Index = 13;
			this.menuPrint.Text = "Print";
			this.menuPrint.Click += new System.EventHandler( this.menuPrint_Click );
			// 
			// menuItem1
			// 
			this.menuItem1.Index = 14;
			this.menuItem1.Text = "-";
			// 
			// menuExit
			// 
			this.menuExit.Index = 15;
			this.menuExit.Text = "Exit";
			this.menuExit.Click += new System.EventHandler( this.menuExit_Click );
			// 
			// menuHelp
			// 
			this.menuHelp.Index = 1;
			this.menuHelp.MenuItems.AddRange( new System.Windows.Forms.MenuItem[] {
            this.menuDocu,
            this.menuItem5,
            this.menuAbout} );
			this.menuHelp.Text = "Help";
			// 
			// menuDocu
			// 
			this.menuDocu.Index = 0;
			this.menuDocu.Text = "Docu";
			this.menuDocu.Click += new System.EventHandler( this.menuDocu_Click );
			// 
			// menuItem5
			// 
			this.menuItem5.Index = 1;
			this.menuItem5.Text = "-";
			// 
			// menuAbout
			// 
			this.menuAbout.Index = 2;
			this.menuAbout.Text = "About";
			this.menuAbout.Click += new System.EventHandler( this.menuAbout_Click );
			// 
			// lblAuthor
			// 
			this.lblAuthor.Font = new System.Drawing.Font( "Microsoft Sans Serif", 9F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ( (byte)( 0 ) ) );
			this.lblAuthor.ImeMode = System.Windows.Forms.ImeMode.NoControl;
			this.lblAuthor.Location = new System.Drawing.Point( 10, 30 );
			this.lblAuthor.Name = "lblAuthor";
			this.lblAuthor.Size = new System.Drawing.Size( 86, 25 );
			this.lblAuthor.TabIndex = 0;
			this.lblAuthor.Text = "Author";
			this.lblAuthor.TextAlign = System.Drawing.ContentAlignment.MiddleLeft;
			// 
			// txtAuthor
			// 
			this.txtAuthor.BackColor = System.Drawing.SystemColors.Window;
			this.txtAuthor.Font = new System.Drawing.Font( "Microsoft Sans Serif", 9F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ( (byte)( 0 ) ) );
			this.txtAuthor.Location = new System.Drawing.Point( 106, 30 );
			this.txtAuthor.Name = "txtAuthor";
			this.txtAuthor.Size = new System.Drawing.Size( 86, 24 );
			this.txtAuthor.TabIndex = 0;
			this.txtAuthor.TabStop = false;
			this.txtAuthor.Text = "xxx";
			// 
			// lblTitle
			// 
			this.lblTitle.Font = new System.Drawing.Font( "Microsoft Sans Serif", 9F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ( (byte)( 0 ) ) );
			this.lblTitle.ImeMode = System.Windows.Forms.ImeMode.NoControl;
			this.lblTitle.Location = new System.Drawing.Point( 5, 28 );
			this.lblTitle.Name = "lblTitle";
			this.lblTitle.Size = new System.Drawing.Size( 101, 23 );
			this.lblTitle.TabIndex = 0;
			this.lblTitle.Text = "Title";
			this.lblTitle.TextAlign = System.Drawing.ContentAlignment.MiddleLeft;
			// 
			// txtTitle
			// 
			this.txtTitle.BackColor = System.Drawing.SystemColors.Window;
			this.txtTitle.Font = new System.Drawing.Font( "Microsoft Sans Serif", 9F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ( (byte)( 0 ) ) );
			this.txtTitle.Location = new System.Drawing.Point( 106, 28 );
			this.txtTitle.Name = "txtTitle";
			this.txtTitle.Size = new System.Drawing.Size( 710, 24 );
			this.txtTitle.TabIndex = 1;
			this.txtTitle.Text = "Title";
			// 
			// lblPrecTC
			// 
			this.lblPrecTC.Font = new System.Drawing.Font( "Microsoft Sans Serif", 9F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ( (byte)( 0 ) ) );
			this.lblPrecTC.ImeMode = System.Windows.Forms.ImeMode.NoControl;
			this.lblPrecTC.Location = new System.Drawing.Point( 10, 171 );
			this.lblPrecTC.Name = "lblPrecTC";
			this.lblPrecTC.Size = new System.Drawing.Size( 124, 41 );
			this.lblPrecTC.TabIndex = 0;
			this.lblPrecTC.Text = "Preceeding TCs";
			this.lblPrecTC.TextAlign = System.Drawing.ContentAlignment.MiddleLeft;
			// 
			// lstPrecTC
			// 
			this.lstPrecTC.BackColor = System.Drawing.SystemColors.Window;
			this.lstPrecTC.ContextMenu = this.ctxtMenuPrecTC;
			this.lstPrecTC.Font = new System.Drawing.Font( "Microsoft Sans Serif", 9F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ( (byte)( 0 ) ) );
			this.lstPrecTC.ItemHeight = 18;
			this.lstPrecTC.Location = new System.Drawing.Point( 144, 171 );
			this.lstPrecTC.Name = "lstPrecTC";
			this.lstPrecTC.ScrollAlwaysVisible = true;
			this.lstPrecTC.Size = new System.Drawing.Size( 466, 22 );
			this.lstPrecTC.TabIndex = 10;
			// 
			// ctxtMenuPrecTC
			// 
			this.ctxtMenuPrecTC.MenuItems.AddRange( new System.Windows.Forms.MenuItem[] {
            this.ctxtPrecTCAdd,
            this.ctxtPrecTCRem} );
			this.ctxtMenuPrecTC.Popup += new System.EventHandler( this.ctxtMenuPrecTC_Popup );
			// 
			// ctxtPrecTCAdd
			// 
			this.ctxtPrecTCAdd.Index = 0;
			this.ctxtPrecTCAdd.Text = "add";
			this.ctxtPrecTCAdd.Click += new System.EventHandler( this.ctxtPrecTCAdd_Click );
			// 
			// ctxtPrecTCRem
			// 
			this.ctxtPrecTCRem.Index = 1;
			this.ctxtPrecTCRem.Text = "remove";
			this.ctxtPrecTCRem.Click += new System.EventHandler( this.ctxtPrecTCRem_Click );
			// 
			// lblDate
			// 
			this.lblDate.Font = new System.Drawing.Font( "Microsoft Sans Serif", 9F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ( (byte)( 0 ) ) );
			this.lblDate.ImeMode = System.Windows.Forms.ImeMode.NoControl;
			this.lblDate.Location = new System.Drawing.Point( 10, 65 );
			this.lblDate.Name = "lblDate";
			this.lblDate.Size = new System.Drawing.Size( 86, 23 );
			this.lblDate.TabIndex = 0;
			this.lblDate.Text = "Date";
			this.lblDate.TextAlign = System.Drawing.ContentAlignment.MiddleLeft;
			// 
			// txtDate
			// 
			this.txtDate.BackColor = System.Drawing.SystemColors.Window;
			this.txtDate.Font = new System.Drawing.Font( "Microsoft Sans Serif", 9F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ( (byte)( 0 ) ) );
			this.txtDate.Location = new System.Drawing.Point( 106, 63 );
			this.txtDate.Name = "txtDate";
			this.txtDate.Size = new System.Drawing.Size( 86, 24 );
			this.txtDate.TabIndex = 0;
			this.txtDate.TabStop = false;
			this.txtDate.Text = "2003-12-31";
			// 
			// lblTCType
			// 
			this.lblTCType.Font = new System.Drawing.Font( "Microsoft Sans Serif", 9F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ( (byte)( 0 ) ) );
			this.lblTCType.ImeMode = System.Windows.Forms.ImeMode.NoControl;
			this.lblTCType.Location = new System.Drawing.Point( 10, 32 );
			this.lblTCType.Name = "lblTCType";
			this.lblTCType.Size = new System.Drawing.Size( 91, 26 );
			this.lblTCType.TabIndex = 0;
			this.lblTCType.Text = "TC Type";
			this.lblTCType.TextAlign = System.Drawing.ContentAlignment.MiddleLeft;
			// 
			// comboTCType
			// 
			this.comboTCType.BackColor = System.Drawing.SystemColors.Window;
			this.comboTCType.Font = new System.Drawing.Font( "Microsoft Sans Serif", 9F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ( (byte)( 0 ) ) );
			this.comboTCType.ItemHeight = 18;
			this.comboTCType.Location = new System.Drawing.Point( 106, 31 );
			this.comboTCType.Name = "comboTCType";
			this.comboTCType.Size = new System.Drawing.Size( 158, 26 );
			this.comboTCType.TabIndex = 5;
			// 
			// lblTime
			// 
			this.lblTime.Font = new System.Drawing.Font( "Microsoft Sans Serif", 9F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ( (byte)( 0 ) ) );
			this.lblTime.ImeMode = System.Windows.Forms.ImeMode.NoControl;
			this.lblTime.Location = new System.Drawing.Point( 10, 102 );
			this.lblTime.Name = "lblTime";
			this.lblTime.Size = new System.Drawing.Size( 86, 23 );
			this.lblTime.TabIndex = 0;
			this.lblTime.Text = "Time";
			this.lblTime.TextAlign = System.Drawing.ContentAlignment.MiddleLeft;
			// 
			// txtTime
			// 
			this.txtTime.BackColor = System.Drawing.SystemColors.Window;
			this.txtTime.Font = new System.Drawing.Font( "Microsoft Sans Serif", 9F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ( (byte)( 0 ) ) );
			this.txtTime.Location = new System.Drawing.Point( 106, 100 );
			this.txtTime.Name = "txtTime";
			this.txtTime.Size = new System.Drawing.Size( 48, 24 );
			this.txtTime.TabIndex = 0;
			this.txtTime.TabStop = false;
			this.txtTime.Text = "01:59";
			// 
			// lblPrio
			// 
			this.lblPrio.Font = new System.Drawing.Font( "Microsoft Sans Serif", 9F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ( (byte)( 0 ) ) );
			this.lblPrio.ImeMode = System.Windows.Forms.ImeMode.NoControl;
			this.lblPrio.Location = new System.Drawing.Point( 278, 33 );
			this.lblPrio.Name = "lblPrio";
			this.lblPrio.Size = new System.Drawing.Size( 72, 24 );
			this.lblPrio.TabIndex = 0;
			this.lblPrio.Text = "Priority";
			this.lblPrio.TextAlign = System.Drawing.ContentAlignment.MiddleLeft;
			// 
			// comboTCPriority
			// 
			this.comboTCPriority.BackColor = System.Drawing.SystemColors.Window;
			this.comboTCPriority.Font = new System.Drawing.Font( "Microsoft Sans Serif", 9F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ( (byte)( 0 ) ) );
			this.comboTCPriority.ItemHeight = 18;
			this.comboTCPriority.Location = new System.Drawing.Point( 350, 31 );
			this.comboTCPriority.Name = "comboTCPriority";
			this.comboTCPriority.Size = new System.Drawing.Size( 96, 26 );
			this.comboTCPriority.TabIndex = 6;
			// 
			// lblReqs
			// 
			this.lblReqs.Font = new System.Drawing.Font( "Microsoft Sans Serif", 9F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ( (byte)( 0 ) ) );
			this.lblReqs.ImeMode = System.Windows.Forms.ImeMode.NoControl;
			this.lblReqs.Location = new System.Drawing.Point( 10, 120 );
			this.lblReqs.Name = "lblReqs";
			this.lblReqs.Size = new System.Drawing.Size( 124, 42 );
			this.lblReqs.TabIndex = 0;
			this.lblReqs.Text = "Requirements";
			this.lblReqs.TextAlign = System.Drawing.ContentAlignment.MiddleLeft;
			// 
			// lstReqs
			// 
			this.lstReqs.BackColor = System.Drawing.SystemColors.Window;
			this.lstReqs.ContextMenu = this.ctxtMenuReqs;
			this.lstReqs.Font = new System.Drawing.Font( "Microsoft Sans Serif", 9F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ( (byte)( 0 ) ) );
			this.lstReqs.ItemHeight = 18;
			this.lstReqs.Location = new System.Drawing.Point( 144, 115 );
			this.lstReqs.Name = "lstReqs";
			this.lstReqs.ScrollAlwaysVisible = true;
			this.lstReqs.Size = new System.Drawing.Size( 466, 22 );
			this.lstReqs.TabIndex = 9;
			// 
			// ctxtMenuReqs
			// 
			this.ctxtMenuReqs.MenuItems.AddRange( new System.Windows.Forms.MenuItem[] {
            this.ctxtReqsEdit} );
			// 
			// ctxtReqsEdit
			// 
			this.ctxtReqsEdit.Index = 0;
			this.ctxtReqsEdit.Text = "edit";
			this.ctxtReqsEdit.Click += new System.EventHandler( this.ctxtReqsEdit_Click );
			// 
			// lblTSysConfig
			// 
			this.lblTSysConfig.Font = new System.Drawing.Font( "Microsoft Sans Serif", 9F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ( (byte)( 0 ) ) );
			this.lblTSysConfig.ImeMode = System.Windows.Forms.ImeMode.NoControl;
			this.lblTSysConfig.Location = new System.Drawing.Point( 10, 69 );
			this.lblTSysConfig.Name = "lblTSysConfig";
			this.lblTSysConfig.Size = new System.Drawing.Size( 124, 42 );
			this.lblTSysConfig.TabIndex = 0;
			this.lblTSysConfig.Text = "Test System Configuration";
			this.lblTSysConfig.TextAlign = System.Drawing.ContentAlignment.MiddleLeft;
			// 
			// lblDesc
			// 
			this.lblDesc.Font = new System.Drawing.Font( "Microsoft Sans Serif", 9F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ( (byte)( 0 ) ) );
			this.lblDesc.ImeMode = System.Windows.Forms.ImeMode.NoControl;
			this.lblDesc.Location = new System.Drawing.Point( 19, 397 );
			this.lblDesc.Name = "lblDesc";
			this.lblDesc.Size = new System.Drawing.Size( 168, 23 );
			this.lblDesc.TabIndex = 0;
			this.lblDesc.Text = "Description";
			this.lblDesc.TextAlign = System.Drawing.ContentAlignment.MiddleLeft;
			// 
			// txtDesc
			// 
			this.txtDesc.AcceptsReturn = true;
			this.txtDesc.AcceptsTab = true;
			this.txtDesc.BackColor = System.Drawing.SystemColors.Window;
			this.txtDesc.Font = new System.Drawing.Font( "Microsoft Sans Serif", 9F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ( (byte)( 0 ) ) );
			this.txtDesc.Location = new System.Drawing.Point( 19, 425 );
			this.txtDesc.Multiline = true;
			this.txtDesc.Name = "txtDesc";
			this.txtDesc.ScrollBars = System.Windows.Forms.ScrollBars.Vertical;
			this.txtDesc.Size = new System.Drawing.Size( 816, 97 );
			this.txtDesc.TabIndex = 11;
			this.txtDesc.Text = "Description";
			this.txtDesc.DoubleClick += new System.EventHandler( this.txtDesc_DoubleClick );
			// 
			// txtTstProc
			// 
			this.txtTstProc.AcceptsReturn = true;
			this.txtTstProc.AcceptsTab = true;
			this.txtTstProc.BackColor = System.Drawing.SystemColors.Window;
			this.txtTstProc.Font = new System.Drawing.Font( "Microsoft Sans Serif", 9F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ( (byte)( 0 ) ) );
			this.txtTstProc.Location = new System.Drawing.Point( 0, 5 );
			this.txtTstProc.Multiline = true;
			this.txtTstProc.Name = "txtTstProc";
			this.txtTstProc.ScrollBars = System.Windows.Forms.ScrollBars.Vertical;
			this.txtTstProc.Size = new System.Drawing.Size( 809, 133 );
			this.txtTstProc.TabIndex = 12;
			this.txtTstProc.Text = "Test Procedure";
			this.txtTstProc.DoubleClick += new System.EventHandler( this.txtTstProc_DoubleClick );
			// 
			// txtExpRslt
			// 
			this.txtExpRslt.AcceptsReturn = true;
			this.txtExpRslt.AcceptsTab = true;
			this.txtExpRslt.BackColor = System.Drawing.SystemColors.Window;
			this.txtExpRslt.Font = new System.Drawing.Font( "Microsoft Sans Serif", 9F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ( (byte)( 0 ) ) );
			this.txtExpRslt.Location = new System.Drawing.Point( 0, 5 );
			this.txtExpRslt.Multiline = true;
			this.txtExpRslt.Name = "txtExpRslt";
			this.txtExpRslt.ScrollBars = System.Windows.Forms.ScrollBars.Vertical;
			this.txtExpRslt.Size = new System.Drawing.Size( 809, 110 );
			this.txtExpRslt.TabIndex = 14;
			this.txtExpRslt.Text = "Expected Rslt";
			this.txtExpRslt.DoubleClick += new System.EventHandler( this.txtExpRslt_DoubleClick );
			// 
			// lblExpDur
			// 
			this.lblExpDur.Font = new System.Drawing.Font( "Microsoft Sans Serif", 9F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ( (byte)( 0 ) ) );
			this.lblExpDur.ImeMode = System.Windows.Forms.ImeMode.NoControl;
			this.lblExpDur.Location = new System.Drawing.Point( 456, 25 );
			this.lblExpDur.Name = "lblExpDur";
			this.lblExpDur.Size = new System.Drawing.Size( 101, 40 );
			this.lblExpDur.TabIndex = 0;
			this.lblExpDur.Text = "Expected Duration";
			this.lblExpDur.TextAlign = System.Drawing.ContentAlignment.MiddleLeft;
			// 
			// txtExpDuration
			// 
			this.txtExpDuration.BackColor = System.Drawing.SystemColors.Window;
			this.txtExpDuration.Font = new System.Drawing.Font( "Microsoft Sans Serif", 9F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ( (byte)( 0 ) ) );
			this.txtExpDuration.Location = new System.Drawing.Point( 557, 32 );
			this.txtExpDuration.Name = "txtExpDuration";
			this.txtExpDuration.Size = new System.Drawing.Size( 48, 24 );
			this.txtExpDuration.TabIndex = 7;
			this.txtExpDuration.Text = "12:15";
			// 
			// butSaveNew
			// 
			this.butSaveNew.BackColor = System.Drawing.SystemColors.Control;
			this.butSaveNew.Font = new System.Drawing.Font( "Microsoft Sans Serif", 9F, System.Drawing.FontStyle.Bold, System.Drawing.GraphicsUnit.Point, ( (byte)( 0 ) ) );
			this.butSaveNew.ForeColor = System.Drawing.SystemColors.WindowText;
			this.butSaveNew.ImeMode = System.Windows.Forms.ImeMode.NoControl;
			this.butSaveNew.Location = new System.Drawing.Point( 710, 891 );
			this.butSaveNew.Name = "butSaveNew";
			this.butSaveNew.Size = new System.Drawing.Size( 164, 32 );
			this.butSaveNew.TabIndex = 22;
			this.butSaveNew.Text = "Save && New";
			this.butSaveNew.UseVisualStyleBackColor = false;
			this.butSaveNew.MouseLeave += new System.EventHandler( this.butSaveNew_MouseLeave );
			this.butSaveNew.Click += new System.EventHandler( this.butSaveNew_Click );
			this.butSaveNew.MouseEnter += new System.EventHandler( this.butSaveNew_MouseEnter );
			// 
			// grpTCAttr
			// 
			this.grpTCAttr.Controls.Add( this.comboTSysConf );
			this.grpTCAttr.Controls.Add( this.comboTCType );
			this.grpTCAttr.Controls.Add( this.lblTCType );
			this.grpTCAttr.Controls.Add( this.lblPrio );
			this.grpTCAttr.Controls.Add( this.comboTCPriority );
			this.grpTCAttr.Controls.Add( this.lblReqs );
			this.grpTCAttr.Controls.Add( this.lstReqs );
			this.grpTCAttr.Controls.Add( this.lblTSysConfig );
			this.grpTCAttr.Controls.Add( this.lblExpDur );
			this.grpTCAttr.Controls.Add( this.txtExpDuration );
			this.grpTCAttr.Controls.Add( this.lstPrecTC );
			this.grpTCAttr.Controls.Add( this.lblPrecTC );
			this.grpTCAttr.Font = new System.Drawing.Font( "Microsoft Sans Serif", 9F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ( (byte)( 0 ) ) );
			this.grpTCAttr.Location = new System.Drawing.Point( 221, 189 );
			this.grpTCAttr.Name = "grpTCAttr";
			this.grpTCAttr.Size = new System.Drawing.Size( 619, 222 );
			this.grpTCAttr.TabIndex = 0;
			this.grpTCAttr.TabStop = false;
			this.grpTCAttr.Text = "TC Attributes";
			// 
			// comboTSysConf
			// 
			this.comboTSysConf.BackColor = System.Drawing.SystemColors.Window;
			this.comboTSysConf.Font = new System.Drawing.Font( "Microsoft Sans Serif", 9F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ( (byte)( 0 ) ) );
			this.comboTSysConf.ItemHeight = 18;
			this.comboTSysConf.Location = new System.Drawing.Point( 144, 69 );
			this.comboTSysConf.Name = "comboTSysConf";
			this.comboTSysConf.Size = new System.Drawing.Size( 466, 26 );
			this.comboTSysConf.TabIndex = 11;
			// 
			// ctxtMenuTSysConf
			// 
			this.ctxtMenuTSysConf.MenuItems.AddRange( new System.Windows.Forms.MenuItem[] {
            this.ctxtTSysSel} );
			// 
			// ctxtTSysSel
			// 
			this.ctxtTSysSel.Index = 0;
			this.ctxtTSysSel.Text = "select";
			this.ctxtTSysSel.Click += new System.EventHandler( this.ctxtTSysSel_Click );
			// 
			// grpEditInfo
			// 
			this.grpEditInfo.Controls.Add( this.txtTCVer );
			this.grpEditInfo.Controls.Add( this.lblTCVer );
			this.grpEditInfo.Controls.Add( this.txtTime );
			this.grpEditInfo.Controls.Add( this.lblTime );
			this.grpEditInfo.Controls.Add( this.txtAuthor );
			this.grpEditInfo.Controls.Add( this.lblAuthor );
			this.grpEditInfo.Controls.Add( this.txtDate );
			this.grpEditInfo.Controls.Add( this.lblDate );
			this.grpEditInfo.Font = new System.Drawing.Font( "Microsoft Sans Serif", 9F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ( (byte)( 0 ) ) );
			this.grpEditInfo.Location = new System.Drawing.Point( 14, 189 );
			this.grpEditInfo.Name = "grpEditInfo";
			this.grpEditInfo.Size = new System.Drawing.Size( 202, 176 );
			this.grpEditInfo.TabIndex = 0;
			this.grpEditInfo.TabStop = false;
			this.grpEditInfo.Text = "TC Info";
			// 
			// txtTCVer
			// 
			this.txtTCVer.BackColor = System.Drawing.SystemColors.Window;
			this.txtTCVer.Font = new System.Drawing.Font( "Microsoft Sans Serif", 9F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ( (byte)( 0 ) ) );
			this.txtTCVer.Location = new System.Drawing.Point( 106, 137 );
			this.txtTCVer.Name = "txtTCVer";
			this.txtTCVer.Size = new System.Drawing.Size( 48, 24 );
			this.txtTCVer.TabIndex = 0;
			this.txtTCVer.TabStop = false;
			this.txtTCVer.Text = "1.0";
			// 
			// lblTCVer
			// 
			this.lblTCVer.Font = new System.Drawing.Font( "Microsoft Sans Serif", 9F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ( (byte)( 0 ) ) );
			this.lblTCVer.ImeMode = System.Windows.Forms.ImeMode.NoControl;
			this.lblTCVer.Location = new System.Drawing.Point( 10, 138 );
			this.lblTCVer.Name = "lblTCVer";
			this.lblTCVer.Size = new System.Drawing.Size( 86, 24 );
			this.lblTCVer.TabIndex = 0;
			this.lblTCVer.Text = "Version";
			this.lblTCVer.TextAlign = System.Drawing.ContentAlignment.MiddleLeft;
			// 
			// panel1
			// 
			this.panel1.AutoScroll = true;
			this.panel1.BackColor = System.Drawing.SystemColors.Control;
			this.panel1.BorderStyle = System.Windows.Forms.BorderStyle.Fixed3D;
			this.panel1.Controls.Add( this.tabCtrlRslt );
			this.panel1.Controls.Add( this.tabCtrlTstProc );
			this.panel1.Controls.Add( this.grpTCid );
			this.panel1.Controls.Add( this.grpEditInfo );
			this.panel1.Controls.Add( this.grpTCAttr );
			this.panel1.Controls.Add( this.lblDesc );
			this.panel1.Controls.Add( this.txtDesc );
			this.panel1.Font = new System.Drawing.Font( "Microsoft Sans Serif", 9F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ( (byte)( 0 ) ) );
			this.panel1.ForeColor = System.Drawing.SystemColors.WindowText;
			this.panel1.Location = new System.Drawing.Point( 5, 5 );
			this.panel1.Name = "panel1";
			this.panel1.Size = new System.Drawing.Size( 873, 877 );
			this.panel1.TabIndex = 0;
			// 
			// tabCtrlRslt
			// 
			this.tabCtrlRslt.Controls.Add( this.tabRsltTxt );
			this.tabCtrlRslt.Controls.Add( this.tabRsltFile );
			this.tabCtrlRslt.Font = new System.Drawing.Font( "Microsoft Sans Serif", 9F );
			this.tabCtrlRslt.ItemSize = new System.Drawing.Size( 42, 20 );
			this.tabCtrlRslt.Location = new System.Drawing.Point( 19, 715 );
			this.tabCtrlRslt.Name = "tabCtrlRslt";
			this.tabCtrlRslt.SelectedIndex = 0;
			this.tabCtrlRslt.Size = new System.Drawing.Size( 821, 148 );
			this.tabCtrlRslt.TabIndex = 0;
			// 
			// tabRsltTxt
			// 
			this.tabRsltTxt.Controls.Add( this.txtExpRslt );
			this.tabRsltTxt.Font = new System.Drawing.Font( "Microsoft Sans Serif", 9F );
			this.tabRsltTxt.Location = new System.Drawing.Point( 4, 24 );
			this.tabRsltTxt.Name = "tabRsltTxt";
			this.tabRsltTxt.Size = new System.Drawing.Size( 813, 120 );
			this.tabRsltTxt.TabIndex = 0;
			this.tabRsltTxt.Text = "Rslt";
			// 
			// tabRsltFile
			// 
			this.tabRsltFile.Controls.Add( this.lstTstRefFile );
			this.tabRsltFile.Font = new System.Drawing.Font( "Microsoft Sans Serif", 9F );
			this.tabRsltFile.Location = new System.Drawing.Point( 4, 24 );
			this.tabRsltFile.Name = "tabRsltFile";
			this.tabRsltFile.Size = new System.Drawing.Size( 813, 120 );
			this.tabRsltFile.TabIndex = 1;
			this.tabRsltFile.Text = "file";
			// 
			// lstTstRefFile
			// 
			this.lstTstRefFile.BackColor = System.Drawing.SystemColors.Window;
			this.lstTstRefFile.ContextMenu = this.ctxtMenuRefFile;
			this.lstTstRefFile.Font = new System.Drawing.Font( "Microsoft Sans Serif", 9F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ( (byte)( 0 ) ) );
			this.lstTstRefFile.ItemHeight = 18;
			this.lstTstRefFile.Location = new System.Drawing.Point( 10, 9 );
			this.lstTstRefFile.Name = "lstTstRefFile";
			this.lstTstRefFile.ScrollAlwaysVisible = true;
			this.lstTstRefFile.Size = new System.Drawing.Size( 792, 40 );
			this.lstTstRefFile.TabIndex = 15;
			// 
			// ctxtMenuRefFile
			// 
			this.ctxtMenuRefFile.MenuItems.AddRange( new System.Windows.Forms.MenuItem[] {
            this.ctxtRefAdd,
            this.ctxtRefRem} );
			this.ctxtMenuRefFile.Popup += new System.EventHandler( this.ctxtMenuRefFile_Popup );
			// 
			// ctxtRefAdd
			// 
			this.ctxtRefAdd.Index = 0;
			this.ctxtRefAdd.Text = "add";
			this.ctxtRefAdd.Click += new System.EventHandler( this.ctxtRefAdd_Click );
			// 
			// ctxtRefRem
			// 
			this.ctxtRefRem.Index = 1;
			this.ctxtRefRem.Text = "remove";
			this.ctxtRefRem.Click += new System.EventHandler( this.ctxtRefRem_Click );
			// 
			// tabCtrlTstProc
			// 
			this.tabCtrlTstProc.Controls.Add( this.tabTstProcTxt );
			this.tabCtrlTstProc.Controls.Add( this.tabTstProcFile );
			this.tabCtrlTstProc.Font = new System.Drawing.Font( "Microsoft Sans Serif", 9F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ( (byte)( 0 ) ) );
			this.tabCtrlTstProc.ItemSize = new System.Drawing.Size( 68, 20 );
			this.tabCtrlTstProc.Location = new System.Drawing.Point( 19, 531 );
			this.tabCtrlTstProc.Name = "tabCtrlTstProc";
			this.tabCtrlTstProc.SelectedIndex = 0;
			this.tabCtrlTstProc.Size = new System.Drawing.Size( 821, 171 );
			this.tabCtrlTstProc.TabIndex = 0;
			// 
			// tabTstProcTxt
			// 
			this.tabTstProcTxt.Controls.Add( this.txtTstProc );
			this.tabTstProcTxt.Location = new System.Drawing.Point( 4, 24 );
			this.tabTstProcTxt.Name = "tabTstProcTxt";
			this.tabTstProcTxt.Size = new System.Drawing.Size( 813, 143 );
			this.tabTstProcTxt.TabIndex = 0;
			this.tabTstProcTxt.Text = "procedure";
			// 
			// tabTstProcFile
			// 
			this.tabTstProcFile.Controls.Add( this.lstTstProcFile );
			this.tabTstProcFile.Font = new System.Drawing.Font( "Microsoft Sans Serif", 9F );
			this.tabTstProcFile.Location = new System.Drawing.Point( 4, 24 );
			this.tabTstProcFile.Name = "tabTstProcFile";
			this.tabTstProcFile.Size = new System.Drawing.Size( 813, 143 );
			this.tabTstProcFile.TabIndex = 1;
			this.tabTstProcFile.Text = "file";
			// 
			// lstTstProcFile
			// 
			this.lstTstProcFile.BackColor = System.Drawing.SystemColors.Window;
			this.lstTstProcFile.ContextMenu = this.ctxtMenuProcFile;
			this.lstTstProcFile.Font = new System.Drawing.Font( "Microsoft Sans Serif", 9F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ( (byte)( 0 ) ) );
			this.lstTstProcFile.ItemHeight = 18;
			this.lstTstProcFile.Location = new System.Drawing.Point( 10, 9 );
			this.lstTstProcFile.Name = "lstTstProcFile";
			this.lstTstProcFile.ScrollAlwaysVisible = true;
			this.lstTstProcFile.Size = new System.Drawing.Size( 792, 40 );
			this.lstTstProcFile.TabIndex = 13;
			// 
			// ctxtMenuProcFile
			// 
			this.ctxtMenuProcFile.MenuItems.AddRange( new System.Windows.Forms.MenuItem[] {
            this.ctxtProcAdd,
            this.ctxtProcRem} );
			this.ctxtMenuProcFile.Popup += new System.EventHandler( this.ctxtMenuProcFile_Popup );
			// 
			// ctxtProcAdd
			// 
			this.ctxtProcAdd.Index = 0;
			this.ctxtProcAdd.Text = "add";
			this.ctxtProcAdd.Click += new System.EventHandler( this.ctxtProcAdd_Click );
			// 
			// ctxtProcRem
			// 
			this.ctxtProcRem.Index = 1;
			this.ctxtProcRem.Text = "remove";
			this.ctxtProcRem.Click += new System.EventHandler( this.ctxtProcRem_Click );
			// 
			// grpTCid
			// 
			this.grpTCid.Controls.Add( this.comboApplRel );
			this.grpTCid.Controls.Add( this.lblSoftRel );
			this.grpTCid.Controls.Add( this.txtFctn );
			this.grpTCid.Controls.Add( this.lblFctn );
			this.grpTCid.Controls.Add( this.lblSection );
			this.grpTCid.Controls.Add( this.lblTitle );
			this.grpTCid.Controls.Add( this.lblAppl );
			this.grpTCid.Controls.Add( this.comboSection );
			this.grpTCid.Controls.Add( this.txtTitle );
			this.grpTCid.Controls.Add( this.comboAppl );
			this.grpTCid.Font = new System.Drawing.Font( "Microsoft Sans Serif", 9F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ( (byte)( 0 ) ) );
			this.grpTCid.Location = new System.Drawing.Point( 14, 9 );
			this.grpTCid.Name = "grpTCid";
			this.grpTCid.Size = new System.Drawing.Size( 826, 171 );
			this.grpTCid.TabIndex = 0;
			this.grpTCid.TabStop = false;
			this.grpTCid.Text = "Test Case";
			// 
			// comboApplRel
			// 
			this.comboApplRel.BackColor = System.Drawing.SystemColors.Window;
			this.comboApplRel.DropDownStyle = System.Windows.Forms.ComboBoxStyle.DropDownList;
			this.comboApplRel.Font = new System.Drawing.Font( "Microsoft Sans Serif", 9F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ( (byte)( 0 ) ) );
			this.comboApplRel.ItemHeight = 18;
			this.comboApplRel.Location = new System.Drawing.Point( 662, 102 );
			this.comboApplRel.Name = "comboApplRel";
			this.comboApplRel.Size = new System.Drawing.Size( 154, 26 );
			this.comboApplRel.TabIndex = 27;
			// 
			// lblSoftRel
			// 
			this.lblSoftRel.Font = new System.Drawing.Font( "Microsoft Sans Serif", 9F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ( (byte)( 0 ) ) );
			this.lblSoftRel.ImeMode = System.Windows.Forms.ImeMode.NoControl;
			this.lblSoftRel.Location = new System.Drawing.Point( 576, 103 );
			this.lblSoftRel.Name = "lblSoftRel";
			this.lblSoftRel.Size = new System.Drawing.Size( 82, 23 );
			this.lblSoftRel.TabIndex = 26;
			this.lblSoftRel.Text = "Release";
			this.lblSoftRel.TextAlign = System.Drawing.ContentAlignment.MiddleLeft;
			// 
			// txtFctn
			// 
			this.txtFctn.BackColor = System.Drawing.SystemColors.Window;
			this.txtFctn.Font = new System.Drawing.Font( "Microsoft Sans Serif", 9F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ( (byte)( 0 ) ) );
			this.txtFctn.Location = new System.Drawing.Point( 106, 134 );
			this.txtFctn.Name = "txtFctn";
			this.txtFctn.Size = new System.Drawing.Size( 710, 24 );
			this.txtFctn.TabIndex = 4;
			this.txtFctn.Text = "Function";
			// 
			// lblFctn
			// 
			this.lblFctn.Font = new System.Drawing.Font( "Microsoft Sans Serif", 9F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ( (byte)( 0 ) ) );
			this.lblFctn.ImeMode = System.Windows.Forms.ImeMode.NoControl;
			this.lblFctn.Location = new System.Drawing.Point( 5, 138 );
			this.lblFctn.Name = "lblFctn";
			this.lblFctn.Size = new System.Drawing.Size( 101, 24 );
			this.lblFctn.TabIndex = 0;
			this.lblFctn.Text = "Function";
			this.lblFctn.TextAlign = System.Drawing.ContentAlignment.MiddleLeft;
			// 
			// lblSection
			// 
			this.lblSection.Font = new System.Drawing.Font( "Microsoft Sans Serif", 9F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ( (byte)( 0 ) ) );
			this.lblSection.ImeMode = System.Windows.Forms.ImeMode.NoControl;
			this.lblSection.Location = new System.Drawing.Point( 5, 67 );
			this.lblSection.Name = "lblSection";
			this.lblSection.Size = new System.Drawing.Size( 101, 23 );
			this.lblSection.TabIndex = 0;
			this.lblSection.Text = "Section";
			this.lblSection.TextAlign = System.Drawing.ContentAlignment.MiddleLeft;
			// 
			// lblAppl
			// 
			this.lblAppl.Font = new System.Drawing.Font( "Microsoft Sans Serif", 9F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ( (byte)( 0 ) ) );
			this.lblAppl.ImeMode = System.Windows.Forms.ImeMode.NoControl;
			this.lblAppl.Location = new System.Drawing.Point( 5, 103 );
			this.lblAppl.Name = "lblAppl";
			this.lblAppl.Size = new System.Drawing.Size( 101, 23 );
			this.lblAppl.TabIndex = 0;
			this.lblAppl.Text = "Application";
			this.lblAppl.TextAlign = System.Drawing.ContentAlignment.MiddleLeft;
			// 
			// comboSection
			// 
			this.comboSection.BackColor = System.Drawing.SystemColors.Window;
			this.comboSection.DropDownStyle = System.Windows.Forms.ComboBoxStyle.DropDownList;
			this.comboSection.Font = new System.Drawing.Font( "Microsoft Sans Serif", 9F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ( (byte)( 0 ) ) );
			this.comboSection.ItemHeight = 18;
			this.comboSection.Location = new System.Drawing.Point( 106, 65 );
			this.comboSection.Name = "comboSection";
			this.comboSection.Size = new System.Drawing.Size( 460, 26 );
			this.comboSection.TabIndex = 2;
			// 
			// comboAppl
			// 
			this.comboAppl.BackColor = System.Drawing.SystemColors.Window;
			this.comboAppl.DropDownStyle = System.Windows.Forms.ComboBoxStyle.DropDownList;
			this.comboAppl.Font = new System.Drawing.Font( "Microsoft Sans Serif", 9F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ( (byte)( 0 ) ) );
			this.comboAppl.ItemHeight = 18;
			this.comboAppl.Location = new System.Drawing.Point( 106, 100 );
			this.comboAppl.Name = "comboAppl";
			this.comboAppl.Size = new System.Drawing.Size( 460, 26 );
			this.comboAppl.TabIndex = 3;
			// 
			// ctxtMenuAddFile
			// 
			this.ctxtMenuAddFile.MenuItems.AddRange( new System.Windows.Forms.MenuItem[] {
            this.ctxtAddFileAdd,
            this.ctxtAddFileRem} );
			// 
			// ctxtAddFileAdd
			// 
			this.ctxtAddFileAdd.Index = 0;
			this.ctxtAddFileAdd.Text = "add";
			this.ctxtAddFileAdd.Click += new System.EventHandler( this.ctxtAddFileAdd_Click );
			// 
			// ctxtAddFileRem
			// 
			this.ctxtAddFileRem.Index = 1;
			this.ctxtAddFileRem.Text = "remove";
			this.ctxtAddFileRem.Click += new System.EventHandler( this.ctxtAddFileRem_Click );
			// 
			// txtStat
			// 
			this.txtStat.BackColor = System.Drawing.Color.Yellow;
			this.txtStat.Font = new System.Drawing.Font( "Microsoft Sans Serif", 9F, System.Drawing.FontStyle.Bold, System.Drawing.GraphicsUnit.Point, ( (byte)( 0 ) ) );
			this.txtStat.Location = new System.Drawing.Point( 10, 891 );
			this.txtStat.Name = "txtStat";
			this.txtStat.ReadOnly = true;
			this.txtStat.Size = new System.Drawing.Size( 129, 24 );
			this.txtStat.TabIndex = 0;
			this.txtStat.TabStop = false;
			this.txtStat.Text = "Initializing...";
			this.txtStat.TextAlign = System.Windows.Forms.HorizontalAlignment.Center;
			// 
			// comboLog
			// 
			this.comboLog.BackColor = System.Drawing.SystemColors.Window;
			this.comboLog.ContextMenu = this.ctxtMenuLog;
			this.comboLog.DropDownStyle = System.Windows.Forms.ComboBoxStyle.DropDownList;
			this.comboLog.Font = new System.Drawing.Font( "Microsoft Sans Serif", 9F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ( (byte)( 0 ) ) );
			this.comboLog.ItemHeight = 18;
			this.comboLog.Location = new System.Drawing.Point( 144, 891 );
			this.comboLog.Name = "comboLog";
			this.comboLog.Size = new System.Drawing.Size( 557, 26 );
			this.comboLog.TabIndex = 16;
			this.comboLog.TabStop = false;
			// 
			// ctxtMenuLog
			// 
			this.ctxtMenuLog.MenuItems.AddRange( new System.Windows.Forms.MenuItem[] {
            this.ctxtLogEntry} );
			// 
			// ctxtLogEntry
			// 
			this.ctxtLogEntry.Index = 0;
			this.ctxtLogEntry.Text = "Display Entry";
			this.ctxtLogEntry.Click += new System.EventHandler( this.ctxtLogEntry_Click );
			// 
			// frmTCEd
			// 
			this.AutoScaleBaseSize = new System.Drawing.Size( 6, 15 );
			this.BackColor = System.Drawing.SystemColors.Control;
			this.ClientSize = new System.Drawing.Size( 890, 931 );
			this.Controls.Add( this.comboLog );
			this.Controls.Add( this.txtStat );
			this.Controls.Add( this.butSaveNew );
			this.Controls.Add( this.panel1 );
			this.Icon = ( (System.Drawing.Icon)( resources.GetObject( "$this.Icon" ) ) );
			this.MaximizeBox = false;
			this.MaximumSize = new System.Drawing.Size( 898, 1006 );
			this.Menu = this.mainMenu;
			this.MinimumSize = new System.Drawing.Size( 898, 92 );
			this.Name = "frmTCEd";
			this.Text = "TC Editor";
			this.Resize += new System.EventHandler( this.frmTCEd_Resize );
			this.grpTCAttr.ResumeLayout( false );
			this.grpTCAttr.PerformLayout();
			this.grpEditInfo.ResumeLayout( false );
			this.grpEditInfo.PerformLayout();
			this.panel1.ResumeLayout( false );
			this.panel1.PerformLayout();
			this.tabCtrlRslt.ResumeLayout( false );
			this.tabRsltTxt.ResumeLayout( false );
			this.tabRsltTxt.PerformLayout();
			this.tabRsltFile.ResumeLayout( false );
			this.tabCtrlTstProc.ResumeLayout( false );
			this.tabTstProcTxt.ResumeLayout( false );
			this.tabTstProcTxt.PerformLayout();
			this.tabTstProcFile.ResumeLayout( false );
			this.grpTCid.ResumeLayout( false );
			this.grpTCid.PerformLayout();
			this.ResumeLayout( false );
			this.PerformLayout();

		}

		/// <summary>
		/// The main entry point for the application.
		/// </summary>
		[STAThread]
		static void Main( string[] args ) {

			if( args.Length < 1 ) {

				Application.Run(new frmTCEd());
			}
			else {

				Application.Run(new frmTCEd( args[0] ));
			}
		}
		#endregion

		#region My Stuff

		//---------------------------------
		private void addLog( IRslt p_Res ) {

			int ix;

			string msg = BTools.getNowISOTime() + " " + p_Res.getRsltLastMsg();

			ix = comboLog.Items.Add( msg );

			m_Logger.write2Log( ref p_Res );

			comboLog.SelectedIndex = ix;
		}

		//----------------------------------
		private void addLog( string p_Msg ) {

			int ix;

			string msg = BTools.getNowISOTime() + " " + p_Msg;

			ix = comboLog.Items.Add( msg );

			m_Logger.write2Log( msg );

			comboLog.SelectedIndex = ix;
		}

		// --------------------------------
		private Rslt appProcFile( ref string p_FileName ) {

			DateTime sDT;

			Ref2File r2f;

			Rslt res = new Rslt( true );

            RsltString fUID;

			string shortName;
			string fType;
			string dt;

			sDT = File.GetLastWriteTime( p_FileName );

			dt = BTools.getISODateTime( sDT );

			shortName = Path.GetFileName( p_FileName );

			fType = Path.GetExtension( p_FileName );

            m_uidTC = m_xDB.getUID( ref m_uidTC ).theString;

            fUID = m_xDB.saveFile( "", ref m_uidTC, ref p_FileName );

            if(res.checkRslt( fUID ))
            {

                r2f = new Ref2File( shortName, fUID.theString, fType, dt, p_FileName);

                m_TProcLst.Add( r2f );

                lstTstProcFile.Items.Add( shortName );

                lstTstProcFile.Refresh();
            }
	
			return res;
		}

		// --------------------------------
		private Rslt appRefFile( ref string p_FileName ) {

			DateTime sDT;

			Ref2File r2f;

			Rslt res = new Rslt( true );

			string shortName;
			string fType;
			string dt;

			sDT = File.GetLastWriteTime( p_FileName );

			dt = BTools.getISODateTime( sDT );

			shortName = Path.GetFileName( p_FileName );

			fType = Path.GetExtension( p_FileName );
			
			r2f = new Ref2File( shortName, "", fType, dt, p_FileName );

			m_TRefLst.Add( r2f );

			lstTstRefFile.Items.Add( shortName );

			lstTstRefFile.Refresh();
	
			return res;
		}
		
		// --------------------------------
		private RsltXMLNode bldTCdoc() {

			RsltXMLNode aNode;
			RsltXMLNode TCNode;
			
			string val;
			RsltString rVal;

			/*
				<!ELEMENT TestCase ( Desc, RefTC+, ReqmntSrc+, TSysConfigRef, TestProc, ExpRslt )>
					<!ATTLIST TestCase
						Appl			CDATA #IMPLIED
						Author		CDATA #REQUIRED
						DateTime	NMTOKEN #REQUIRED
						ExpDur		NMTOKEN #IMPLIED
						Fctn			CDATA #IMPLIED
						Section		CDATA #IMPLIED
						SoftRel		CDATA #IMPLIED
						TCType		(ac|co|do|fi|fu|io|lo|mo|pe|se|un|us|vo|aid|prep) ""
						Title			CDATA #REQUIRED
						UID				NMTOKEN #IMPLIED
						Ver				NMTOKEN #REQUIRED
						Prio			NMTOKEN #IMPLIED
					>
			*/

			string nodeName = "TestCase";
			TCNode = new RsltXMLNode( nodeName );

			if( TCNode.isOK() ) {

				//<?xml version="1.0"?>
				TCNode.createXmlDecl( "1.0", "", "" );

				//<!DOCTYPE TestCase SYSTEM "RG.TestCaseV01.dtd">
				TCNode.addSysDocTypeInfo( nodeName, QNDefs.dtdNameTC );

				//<!-- Generated by QA Navigator Test Case Editor V1.0.0.0-->
				string genCom = QNDefs.phrGenBy + m_QNTools.getApplInfo();
				TCNode.addComment( genCom );

				TCNode.addUID( m_uidTC );
				
				val = m_QNTools.getComboSelVal( ref comboAppl );
				TCNode.addAttribAsHtml( "Appl", val );

				TCNode.addAttribAsHtml( "Author", txtAuthor.Text );

				rVal = m_QNTools.chkVal( txtDate.Text, lblDate.Text );

				if( TCNode.checkRslt( rVal ) ) {

					val = rVal.theString;

					rVal = m_QNTools.chkVal( txtTime.Text, lblTime.Text );

					if( TCNode.checkRslt( rVal ) ) {

						val += "T" + rVal.theString;

						TCNode.addAttrib( "DateTime", val );
					}				
				}
			}

			if( TCNode.checkRslt( m_QNTools.chkNumVal( txtExpDuration.Text, lblExpDur.Text, false) ) ) {

				TCNode.addAttrib( "ExpDur", txtExpDuration.Text );

				TCNode.addAttribAsHtml( "Fctn", txtFctn.Text );

				val = m_QNTools.getComboSelVal( ref comboSection );
				TCNode.addAttribAsHtml( "Section", val );

				val = m_QNTools.getComboSelVal( ref comboApplRel );
				TCNode.addAttribAsHtml( "SoftRel", val );

				val = m_QNTools.getComboSelVal( ref comboTCPriority );
				TCNode.addAttribAsHtml( "Prio", val );

				val = m_QNTools.getComboSelVal( ref comboTCType );
				TCNode.addAttribAsHtml( "TCType", val );

				rVal = m_QNTools.chkVal( txtTitle.Text, lblTitle.Text );

				if( TCNode.checkRslt( rVal ) ) {

					TCNode.addAttribAsHtml( "Title", rVal );
				}

				TCNode.addAttribAsHtml( "Ver", txtTCVer.Text );
			}

			if( TCNode.isOK() ) {

				m_PrecTCLst.loadYSUp( TCNode );
			}

			/* So far there's only a dialog asking for title and location */
			if( TCNode.isOK() ) {

				m_ReqsLst.loadYSUp( TCNode );
			}
			
			if( TCNode.isOK() ) {

				/* <!ELEMENT TSysConfigRef (EMPTY)>
						<!ATTLIST TSysConfigRef
							RefUID	NMTOKEN #IMPLIED
							Title		CDATA #IMPLIED
							UID			NMTOKEN #IMPLIED
						> */

				aNode = TCNode.appNode( "TSysConfigRef" );

				val = m_QNTools.getComboSelText( ref comboTSysConf );

				aNode.addAttribAsHtml( "Title", val );

				aNode.addUID( m_uidTSysConf );
			}
			
			if( TCNode.isOK() ) {
				/*	Desc, TestProc, ExpResult */

				aNode = TCNode.appTxtNodeAsHTML( "Desc", txtDesc.Text );
				aNode.addUID( m_uidDesc );

				aNode = TCNode.appTxtNodeAsHTML( "TestProc", txtTstProc.Text );
				aNode.addUID( m_uidTstProc );


				/*
					<!-- Data for test execution. Descriptive or for automation 
							DatUID is the pointer to the BLOB -->
					<!ELEMENT TestProcData (EMPTY)>
						<!ATTLIST TestProcData
							ContentType	NMTOKEN #IMPLIED
							DateTime		NMTOKEN #IMPLIED
							BlobUID			NMTOKEN #IMPLIED
							FileName		NMTOKEN #IMPLIED
							Title				NMTOKEN #REQUIRED
							UID					NMTOKEN #IMPLIED
						>*/

				m_TProcLst.loadYSUp( TCNode );

				/*
		
					<!-- Data for test Rslt reference. Descriptive or for automation
							DatUID is the pointer to the BLOB -->
					<!ELEMENT TestRsltData (EMPTY)>
						<!ATTLIST TestRsltData
							ContentType	NMTOKEN #IMPLIED
							DateTime		NMTOKEN #IMPLIED
							BlobUID			NMTOKEN #IMPLIED
							FileName		NMTOKEN #IMPLIED
							Title				NMTOKEN #REQUIRED
							UID					NMTOKEN #IMPLIED
						>
				*/

				aNode = TCNode.appTxtNodeAsHTML( "ExpResult", txtExpRslt.Text );
				aNode.addUID( m_uidExpRslt );


				m_TRefLst.loadYSUp( TCNode );

			}

			return TCNode;
		}

		//-----------------------------------
		private void doDialForAnyType( System.ComponentModel.CancelEventHandler p_openFileDial_FileOkHandler ) {
		
			try {

				setAppStat( AppStat.Proc );

				OpenFileDialog openFileDial;
				openFileDial = new OpenFileDialog();
				openFileDial.FileOk += p_openFileDial_FileOkHandler;

				openFileDial.Title = m_Presets.m_QNphrase.lblOpenTC;
				openFileDial.Filter = "All files (*.*)|*.*" ;
				openFileDial.FilterIndex = 1;
				//openFileDial.DefaultExt = "xml";
				
				//openFileDial.InitialDirectory = m_Presets.m_ExpPth;

				openFileDial.RestoreDirectory = true;
				
				DialogResult dRes = openFileDial.ShowDialog();

				if( DialogResult.Cancel == dRes ) {

					setAppStat( AppStat.OK );
				}

				openFileDial.Dispose();
			}
			catch( Exception excp ) {
						
				handleExcp( excp );
			}
		}		
		
		//-------------------------------------------------------------------------
		private Rslt fileDialProc4OK(object sender, System.ComponentModel.CancelEventArgs e, dlgtFileDialOK p_FuncHandleFile ) {

			Rslt res = new Rslt( false );

			try {

				string fileName;

				Cursor oldCur = Cursor.Current;

				Cursor.Current = Cursors.WaitCursor;

				OpenFileDialog dial = (OpenFileDialog)sender;

				fileName = dial.FileName;

				res = p_FuncHandleFile( ref fileName );

				Cursor.Current = oldCur;

				if( res.notOK() ) {

					string msg = String.Format( m_Presets.m_QNphrase.errLoadOfFailed, fileName );

					handleErr( res, msg );
				}
				else {

					setAppStat( AppStat.OK );

					addLog( String.Format( m_Presets.m_QNphrase.msgLoaded, txtTitle.Text ) );
				}
			}
			catch( Exception excp ) {
		
				handleExcp( excp );
			}          

			return res;
		}

		//-------------------------------------------------------------------------
		private void handleExcp( Exception p_Excp ) {
		
			try {

				setAppStat( AppStat.Error );

				addLog( "Exception '" + p_Excp.Message + "'");

				m_Logger.write2Log( ref p_Excp );

				m_GUIservice.dispExcp( p_Excp );
			}
			catch {

				string msg = "An exception has occured at a very early stage of the"
					+ " initialization of the program. So early that the program is not able"
					+ " to provide you any aid. Errors at this stage are mostly due to"
					+ " bad configuration. If the configuration has been changed recently,"
					+ " switch back to the former version, if you can. In other cases please"
					+ " refer to the list of known problems in your documentation.";

				MessageBox.Show(  msg, "Early Error Occured", MessageBoxButtons.OK, MessageBoxIcon.Error );
			}
		}

		//-----------------------
		private void handleIfErr( IRslt p_Res, string p_Msg ) {

			if( p_Res.notOK() ) {

				handleErr( p_Res, p_Msg );
			}
		}

		//-----------------------
		private void handleErr( IRslt p_Res, string p_Msg ) {

			setAppStat( AppStat.Error );

			m_Logger.write2Log( ref p_Res );

			addLog( p_Msg );

			issueErr( p_Msg );
		}
				
		// -----------------------------
		/// <summary>
		/// Displays a text dialog. Detects, if the dialog is supposed to allow editing.
		/// If so, the Rslt is written back to p_Sender. 
		/// </summary>
		/// <param name="p_Sender"></param>
		/// <param name="p_Title"></param>
		private void handleTxtPop( object p_Sender, string p_Title ) {

			try {

				bool forEdit;
				DialogResult dRes;
				TextBox tBox;

				tBox = (TextBox)p_Sender;
				
				forEdit = !(tBox.ReadOnly);
			
				setAppStat( AppStat.Proc );

				frmTxtWin txtWin;
		
				txtWin = new frmTxtWin( forEdit, ref m_Presets );

				txtWin.Text = p_Title;

				txtWin.setText( tBox.Text );

				txtWin.TopMost = true;

				dRes = txtWin.ShowDialog();

				if( DialogResult.OK == dRes ) {

					if( forEdit ) {

						tBox.Text = txtWin.m_Text;
					}

					setAppStat( AppStat.OK );
				}

				txtWin.Dispose();

				setAppStat( AppStat.OK );
			}
			catch( Exception excp ) {
				
				handleExcp( excp );
			}	
		}

		//--------------------------
		private Rslt init() {
		frmTCEd.CheckForIllegalCrossThreadCalls = false;
			int x, y;

			Rslt res = new Rslt( RsltStatVal.isOK );

			m_QNTools = new QNTools();

			m_QNphrase = m_QNTools.m_QNphrase;

			m_GUIservice = new QNGUIservice( m_QNphrase );

			m_Presets = new QNpresets( m_QNTools );

			setClrs();

			// resizing stuff
			x = this.Width;
			y = this.Height;

			m_panel1DH = y - panel1.Height;

			m_butSaveNewDT = y - butSaveNew.Top;

			m_txtStatDT = y - txtStat.Top;
			m_comboLogDT = y - comboLog.Top;

			//setAppStat( AppStat.Proc );

			m_Logger.write2Log( m_QNTools.getApplInfo() );

			setLabels();

			// prepare lists
			m_PrecTCLst = new Ref2DocLst( "RefTC" );
			m_ReqsLst = new Ref2DocValLst( "ReqmntSrc" );
			m_TProcLst = new Ref2FileLst( "TestProcData" );
			m_TRefLst = new Ref2FileLst( "TestRsltData" );

			m_TCSelPresets = new TCSelPresets();

			if( res.checkRslt( m_Presets ) ) {

                m_DB = new RGSQL2( m_Presets.m_DBloc, m_Presets.m_DBname, m_Presets.m_DBuser, m_Presets.m_DBpw );

				if( res.checkRslt( m_DB ) ) {

    				m_xDB = new XMLDB( m_DB, m_Presets.m_UIDsrvLoc );

					reset();

					setUp4New();

					setAppStat( AppStat.OK );

					addLog( BTools.getNowISODate() + " " + m_Presets.m_QNphrase.msgSesStart );
				}
			}

			handleIfErr( res, m_Presets.m_QNphrase.errInitFailed );

			return res;
		}
	
		//------------------------------
		private void issueErr( string msg ) {

			MessageBox.Show( msg, m_Presets.m_QNphrase.lblErr, MessageBoxButtons.OK, MessageBoxIcon.Error );
		}

		//-------------------------------
		private Rslt loadTC( string p_loadUID ) {

			Rslt res = new Rslt( true );
			XMLDBNode xDBNode;

			xDBNode = m_xDB.getXMLsubTree( ref p_loadUID );

			if( res.checkRslt( xDBNode ) ) {

				res = loadTCdoc( xDBNode.m_xDoc );

				if( res.isOK() ) {

					setLoaded( true );
				}
			}

			if( res.notOK() ) {

				m_Logger.write2Log( "Loading of '" +  p_loadUID + "' failed." );
				handleErr( res, m_Presets.m_QNphrase.errTCLoadFailed );
			}

			return res;
		}
				
		//------------------------------
		private Rslt loadTCdoc( ref string p_FileName ) {
								
			Rslt res;

			XmlDocument xDoc = new XmlDocument();

			xDoc.Load( p_FileName );

			res = loadTCdoc( xDoc );

			return res;
		}

		//------------------------------
		private Rslt loadTCdoc( XmlDocument p_XDoc ) {

			Rslt res;
			RsltString xVal;
			RsltXMLNode xNode;
			RsltXMLNode aNode;

			string val;

			XmlNodeList nodeLst;

			/*
				<!ELEMENT TestCase ( Desc, RefTC+, ReqmntSrc+, TSysConfigRef, TestProc, ExpResult )>
					<!ATTLIST TestCase
						Appl			CDATA #IMPLIED
						Author		CDATA #REQUIRED
						DateTime	NMTOKEN #REQUIRED
						ExpDur		NMTOKEN #IMPLIED
						Fctn			CDATA #IMPLIED
						Section		CDATA #IMPLIED
						SoftRel		CDATA #IMPLIED
						TCType		(ac|co|do|fi|fu|io|lo|mo|pe|se|un|us|vo|aid|prep) ""
						Title			CDATA #REQUIRED
						UID				NMTOKEN #IMPLIED
						Ver				NMTOKEN #REQUIRED
						Prio			NMTOKEN #IMPLIED
					>
			*/

			reset();

			res = new Rslt( true );

			nodeLst = p_XDoc.SelectNodes( "/TestCase" );

			if( 1 != nodeLst.Count ) {

				m_Logger.write2Log( "Top node 'TestCase' not found." );

				res.setInError( m_Presets.m_QNphrase.errInvXML );
			}
			else {

				xNode = nodeLst.Item( 0 );

				m_uidTC = xNode.UID;

				xVal = xNode.getAttrValFromHtml( "Appl" );

				m_QNTools.selByDataLstVal( ref comboAppl, xVal.theString );

				xVal = xNode.getAttrValFromHtml( "Author" );
  
				txtAuthor.Text = xVal.theString;

				xVal = xNode.getAttrVal( "DateTime" );

				if( res.checkRslt( xVal ) ) {

					val = BTools.getDateFromISODateTime( xVal.theString );   
					txtDate.Text = val;

					val = BTools.getTimeFromISODateTime( xVal.theString );   
					txtTime.Text = val;

					xVal = xNode.getAttrVal( "ExpDur" );

					txtExpDuration.Text = xVal.theString;

					xVal = xNode.getAttrValFromHtml( "Fctn" );

					txtFctn.Text = xVal.theString;

					// The lock
					xVal = xNode.getAttrVal( "Lock" );

					if( xVal.theString.Length > 0 ) {

						menuSave.Enabled = false;
					}

					xVal = xNode.getAttrValFromHtml( "Prio" );
    
					m_QNTools.selByDataLstVal( ref comboTCPriority, xVal.theString );

					xVal = xNode.getAttrValFromHtml( "Section" );

					m_QNTools.selByDataLstVal( ref comboSection, xVal.theString );

					xVal = xNode.getAttrValFromHtml( "SoftRel" );

					m_QNTools.selByDataLstVal( ref comboApplRel, xVal.theString );

					xVal = xNode.getAttrValFromHtml( "TCType" );
    
					m_QNTools.selByDataLstVal( ref comboTCType, xVal.theString );

					// Title
					xVal = xNode.getAttrValFromHtml( "Title" );
				}

				if( res.checkRslt( xVal ) ) {

					txtTitle.Text = xVal.theString;

					xVal = xNode.getAttrValFromHtml( "Ver" );

					txtTCVer.Text = xVal.theString;

					/*
						<!ELEMENT RefTC (EMPTY)>
							<!ATTLIST RefTC
								Title				CDATA #REQUIRED
								RefUID		NMTOKEN #REQUIRED
								UID				NMTOKEN #IMPLIED
							>
					*/

					m_PrecTCLst.loadYSDown( xNode, lstPrecTC );

					/*<!ELEMENT ReqmntSrc (EMPTY)>
							<!ATTLIST ReqmntSrc
								Location	CDATA #IMPLIED
								RefUID		NMTOKEN #IMPLIED
								Title			CDATA #REQUIRED
								UID				NMTOKEN #IMPLIED
							> */

					m_ReqsLst.loadYSDown( xNode, lstReqs );


					/* <!ELEMENT TSysConfigRef (EMPTY)>
							<!ATTLIST TSysConfigRef
								RefUID	NMTOKEN #IMPLIED
								Title		CDATA #IMPLIED
								UID			NMTOKEN #IMPLIED
							> */

					if( res.checkRslt( m_PrecTCLst ) ) {

						nodeLst = xNode.m_xNode.SelectNodes( "./TSysConfigRef" );

						if( nodeLst.Count > 0 ) {

							aNode = nodeLst.Item( 0 );

							//m_TSysConf[0] = aNode.getAttrVal( "RefUID" );

							m_uidTSysConf = aNode.UID;

							xVal = aNode.getAttrValFromHtml( "Title" );
    
							m_QNTools.selByDataLstVal( ref comboTSysConf, xVal.theString );
						}
					}

					if( res.isOK() ) {

						/*	Desc, TestProc, ExpResult */
						aNode = xNode.getSubNode( "Desc" );

						if( res.checkRslt( aNode ) ) {

							txtDesc.Text = aNode.textFromHtml;
							m_uidDesc = aNode.UID;

							aNode = xNode.getSubNode( "TestProc" );
						}

						if( res.checkRslt( aNode ) ) {

							txtTstProc.Text = aNode.textFromHtml;
							m_uidTstProc = aNode.UID;
						}
					}

					m_TProcLst.loadYSDown( xNode, lstTstProcFile );

					if( res.checkRslt( m_TProcLst ) ) {

						aNode = xNode.getSubNode( "ExpResult" );
					
						if( res.checkRslt( aNode ) ) {

							txtExpRslt.Text = aNode.textFromHtml;
							m_uidExpRslt = aNode.UID;
						}
					}

					m_TRefLst.loadYSDown( xNode, lstTstRefFile );

					res.checkRslt( m_TRefLst );
				}
			}

			return res;
		}

		//---------------------------
		private void loadTCloader() {

			Rslt res = new Rslt();

			try {

				res = loadTC( m_loadUID );

				if( res.notOK() ) {

					handleErr( res, m_Presets.m_QNphrase.errTCLoadFailed );
				}
				else {

					setAppStatOK( res.isOK() );
					addLog( String.Format( m_Presets.m_QNphrase.msgLoaded, txtTitle.Text ) );
				}
			}
			catch( Exception e ) {
		
				handleExcp( e );
			}
		}

		//------------------------
		private void reset() {

			setUpAppl();

			txtAuthor.Text = "";
			txtDate.Text = "";
			txtDesc.Text = "";
			txtExpDuration.Text = "";	
			txtExpRslt.Text = "";
			txtFctn.Text = "";
		
			txtTCVer.Text = "";
			txtTime.Text = "";
			txtTitle.Text = "";			
			txtTstProc.Text = "";
	
			m_ReqsLst.Clear();
			lstReqs.Items.Clear();

			m_PrecTCLst.Clear();
			lstPrecTC.Items.Clear();

			m_TProcLst.Clear();
			lstTstProcFile.Items.Clear();

			m_TRefLst.Clear();
			lstTstRefFile.Items.Clear();

			resetUIDs();

			menuDel.Enabled = false;
			menuExport.Enabled = false;
			menuPrint.Enabled = false;
			menuSave.Enabled = true;
			menuSaveAsNew.Enabled = false;
			menuSaveNNew.Enabled = false;
			menuUID2Clip.Enabled = false;
		}

		//------------------------------
		private void setUp4New()  {

			txtAuthor.Text = m_Presets.m_Author;
		
			txtDate.Text = BTools.getNowISODate();
			txtTime.Text = BTools.getNowISOTime();
			
			txtTCVer.Text = "1.0";
		}
		
		//-----------------------------
		private void resetUIDs() {

			m_uidDesc = "";
			m_uidExpRslt = "";
			m_uidTC = "";
			m_uidTstProc = "";
			m_uidTSysConf = "";
		}

		//------------------------------
		private void resetUID4New() {

			resetUIDs();

			m_PrecTCLst.resetUIDs();
			m_ReqsLst.resetUIDs();
// ################
// copy BLOBs!
			m_TProcLst.resetUIDs();
// ################
// copy BLOBs!
			m_TRefLst.resetUIDs();
		}

		//-------------------------------
		private void setAppStat( AppStat p_Stat ) {

			switch( p_Stat) {

				case AppStat.OK:

					txtStat.BackColor = m_Presets.m_ClrStatOK;
					txtStat.ForeColor = Color.White;
					txtStat.Text = m_Presets.m_QNphrase.lblOK;
					break;

				case AppStat.Proc:

					txtStat.BackColor = m_Presets.m_ClrStatProc;
					txtStat.ForeColor = Color.Black;
					txtStat.Text = m_Presets.m_QNphrase.lblProcng;
					break;

				case AppStat.Error:

					txtStat.BackColor = m_Presets.m_ClrStatErr;
					txtStat.ForeColor = Color.White;
					txtStat.Text =  m_Presets.m_QNphrase.lblErr;
					break;
			}

			this.Refresh();			
		}

		//-----------------------------
		private void setAppStatOK( bool p_IsOK ) {

			if( p_IsOK ) {

				setAppStat( AppStat.OK );
			}
			else {

				setAppStat( AppStat.Error );
			}
		}

		//-----------------------------
		private void setClrs() {

			this.BackColor = m_Presets.m_ClrPnlBck;		
			this.ForeColor = m_Presets.m_ClrPnlFor;

			panel1.BackColor = m_Presets.m_ClrPnlBck;
			panel1.ForeColor = m_Presets.m_ClrPnlFor;
				
			butSaveNew.BackColor = m_Presets.m_ClrButBck;
			butSaveNew.ForeColor = m_Presets.m_ClrPnlFor;

			comboAppl.BackColor = m_Presets.m_ClrTxtBck;
			comboApplRel.BackColor = m_Presets.m_ClrTxtBck;
			comboLog.BackColor = m_Presets.m_ClrTxtBck;
			comboSection.BackColor = m_Presets.m_ClrTxtBck;
			comboTCPriority.BackColor = m_Presets.m_ClrTxtBck;
			comboTCType.BackColor = m_Presets.m_ClrTxtBck;
			comboTSysConf.BackColor = m_Presets.m_ClrTxtBck;

			lstPrecTC.BackColor = m_Presets.m_ClrTxtBck;
			lstReqs.BackColor = m_Presets.m_ClrTxtBck;
			lstTstProcFile.BackColor = m_Presets.m_ClrTxtBck;
			lstTstRefFile.BackColor = m_Presets.m_ClrTxtBck;

			txtAuthor.BackColor = m_Presets.m_ClrTxtBck;
			txtDate.BackColor = m_Presets.m_ClrTxtBck;
			txtDesc.BackColor = m_Presets.m_ClrTxtBck;
			txtExpDuration.BackColor = m_Presets.m_ClrTxtBck;
			txtExpRslt.BackColor = m_Presets.m_ClrTxtBck;
			txtFctn.BackColor = m_Presets.m_ClrTxtBck;
			txtTCVer.BackColor = m_Presets.m_ClrTxtBck;
			txtTime.BackColor = m_Presets.m_ClrTxtBck;
			txtTitle.BackColor = m_Presets.m_ClrTxtBck;
			txtTstProc.BackColor = m_Presets.m_ClrTxtBck;
		}

		//-------------------------------
		private RsltString saveTC() {
				
			RsltXMLNode xNode;

			RsltString UID = new RsltString();

			xNode = bldTCdoc();

			if( xNode.notOK() ) {

				string msg = String.Format( m_Presets.m_QNphrase.errCrtngFailed, txtTitle.Text );

				handleErr( xNode, msg );

				UID.setInError( msg );
			}
			else {

				UID = m_xDB.importXML( ref xNode.m_xDoc );

				if( m_xDB.notOK() ) {

					handleErr( m_xDB, m_Presets.m_QNphrase.errTCSaveFailed );

					m_xDB.resetRslt();
				}
			}

			return UID;
		}

		// -------------------------
		private void setUpAppl() {

			m_QNTools.reFillUpCombo( ref comboAppl, ref m_Presets.m_ApplLst, 0);

/*
comboAppl.Items.RemoveAt(0);
DatLstEntry dle = new DatLstEntry( "test", "test" );
comboAppl.Items.Insert( 0, dle );
*/
			m_QNTools.reFillUpCombo( ref comboApplRel,ref m_Presets.m_ApplSoftRelLst, 0);
			
			m_QNTools.reFillUpCombo( ref comboSection, ref m_Presets.m_SectionLst, 0);

			m_QNTools.reFillUpCombo( ref comboTCPriority, ref m_Presets.m_TstPrioLst, 0);

			m_QNTools.reFillUpCombo( ref comboTCType, ref m_Presets.m_TstTypeLst, 0);

			m_QNTools.reFillUpCombo( ref comboTSysConf, ref m_Presets.m_TstSysConfLst, 0);
		}

		#endregion

		#region I18N

		//---------------------------------
		private void setLabels() {

			Text = m_Presets.m_QNphrase.lblTCEditor;

			butSaveNew.Text = m_Presets.m_QNphrase.blblSaveNew;

			ctxtAddFileAdd.Text = m_Presets.m_QNphrase.mlblAdd;
			ctxtAddFileRem.Text = m_Presets.m_QNphrase.mlblRemove;
			ctxtProcAdd.Text = m_Presets.m_QNphrase.mlblAdd;
			ctxtProcRem.Text = m_Presets.m_QNphrase.mlblRemove;
			ctxtPrecTCAdd.Text = m_Presets.m_QNphrase.mlblAdd;
			ctxtPrecTCRem.Text = m_Presets.m_QNphrase.mlblRemove;
			ctxtRefAdd.Text = m_Presets.m_QNphrase.mlblAdd;
			ctxtRefRem.Text = m_Presets.m_QNphrase.mlblRemove;
			ctxtReqsEdit.Text = m_Presets.m_QNphrase.mlblEdit;
			ctxtTSysSel.Text = m_Presets.m_QNphrase.mlblSel;
			ctxtLogEntry.Text = m_Presets.m_QNphrase.mlblDispEntry;

			grpTCid.Text = m_Presets.m_QNphrase.lblTC;
			grpEditInfo.Text = m_Presets.m_QNphrase.lblEditInfo;
			grpTCAttr.Text = m_Presets.m_QNphrase.lblTCAttr;

			//lblAddFiles.Text = m_Presets.m_QNphrase.lblAddFiles;
			lblAuthor.Text = m_Presets.m_QNphrase.lblAuthor;
			lblAppl.Text = m_Presets.m_QNphrase.lblAppl;
			lblDate.Text = m_Presets.m_QNphrase.lblDate;
			lblDesc.Text = m_Presets.m_QNphrase.lblDesc;
			lblExpDur.Text = m_Presets.m_QNphrase.lblExpDur;
			lblFctn.Text = m_Presets.m_QNphrase.lblFctn;
			lblPrecTC.Text = m_Presets.m_QNphrase.lblPrecTC;
			lblPrio.Text = m_Presets.m_QNphrase.lblPrio;
			lblReqs.Text = m_Presets.m_QNphrase.lblReqs;
			lblSection.Text = m_Presets.m_QNphrase.lblSection;
			lblSoftRel.Text = m_Presets.m_QNphrase.lblRelease;
			lblTCType.Text = m_Presets.m_QNphrase.lblTCType;
			lblTCVer.Text = m_Presets.m_QNphrase.lblTCVer;
			lblTime.Text = m_Presets.m_QNphrase.lblTime;
			lblTitle.Text = m_Presets.m_QNphrase.lblTitle;
			lblTSysConfig.Text = m_Presets.m_QNphrase.lblTSysConfig12;

			menuAbout.Text = m_Presets.m_QNphrase.mlblAbout;
			menuFile.Text = m_Presets.m_QNphrase.mlblFile;
			menuDel.Text = m_Presets.m_QNphrase.mlblDel;
			menuExit.Text = m_Presets.m_QNphrase.mlblExit;
			menuExport.Text = m_Presets.m_QNphrase.mlblExp;
			menuHelp.Text = m_Presets.m_QNphrase.mlblHelp;
			menuImport.Text = m_Presets.m_QNphrase.mlblImp;
			menuNew.Text = m_Presets.m_QNphrase.mlblNew;
			menuOpen.Text = m_Presets.m_QNphrase.mlblOpen;
			menuPrint.Text = m_Presets.m_QNphrase.mlblPrint;
			menuSave.Text = m_Presets.m_QNphrase.mlblSave;
			menuSaveAsNew.Text = m_Presets.m_QNphrase.mlblSaveAsNew;
			menuSaveNNew.Text = m_Presets.m_QNphrase.mlblSaveNNew ;
			menuUID2Clip.Text = m_Presets.m_QNphrase.mlblUID2Clip;

			tabTstProcFile.Text = m_Presets.m_QNphrase.lblFile;
			tabTstProcTxt.Text = m_Presets.m_QNphrase.lblTstProc;
			tabRsltFile.Text = m_Presets.m_QNphrase.lblFile;
			tabRsltTxt.Text = m_Presets.m_QNphrase.lblRslt;
		}

		// ----------------------------------------------------
		private void setLoaded( bool p_InDB ) {

			if( p_InDB ) {

				menuDel.Enabled = true;
				menuExport.Enabled = true;
				menuUID2Clip.Enabled = true;
			}

			menuPrint.Enabled = true;
			menuSaveAsNew.Enabled = true;
			menuSaveNNew.Enabled = true;
		}

		#endregion

		#region event handling

		//-------------------------------
		private void butSaveNew_Click(object sender, System.EventArgs e) {
		
			try {

				RsltString UID;

				setAppStat( AppStat.Proc );
				
				UID = saveTC();

				if( UID.isOK() ) {

					reset();

					setUp4New();

					addLog( String.Format( m_Presets.m_QNphrase.msgSaved, txtTitle.Text ) );
					
					setAppStat( AppStat.OK );
				}
			}
			catch( Exception excp ) {
				
				handleExcp( excp );
			}
		}
				
		//------------------------
		private void ctxtLogEntry_Click(object sender, System.EventArgs e) {

			string msg;

			setAppStat( AppStat.Proc );

			msg = m_QNTools.getComboSelVal( ref comboLog );

			MessageBox.Show( msg, m_Presets.m_QNphrase.lblLogEntry, MessageBoxButtons.OK );	

			setAppStat( AppStat.OK );
		}
		
		//-------------------------
		private void ctxtPrecTCAdd_Click(object sender, System.EventArgs e) {
		
			try {

				int i, x;

				Ref2Doc refTC;

				DatLstEntry entry;

				DataLst RsltLst;
				
				setAppStat( AppStat.Proc );

				frmTCSel dialTCSel = new frmTCSel( m_xDB, false, m_Presets, m_TCSelPresets, "tc" );

				DialogResult dRes = dialTCSel.ShowDialog();

				if( DialogResult.OK == dRes ) {

					RsltLst = dialTCSel.m_RsltLst;

					x = RsltLst.Count;

					for( i = 0; i < x; ++i ) {

						entry = RsltLst[i];

						refTC = new Ref2Doc( entry.label, entry.val );

						m_PrecTCLst.Add( refTC );
										
						lstPrecTC.Items.Add( entry.label );
					}
				}

				dialTCSel.Dispose();

				setAppStat( AppStat.OK );
			}
			catch( Exception excp ) {
			
				handleExcp( excp );
			} 
		}

		//-------------------------
		private void ctxtPrecTCRem_Click(object sender, System.EventArgs e) {

			try {

				setAppStat( AppStat.Proc );

				int ix = lstPrecTC.SelectedIndex;
								
				if( ix > -1 ) {

					m_PrecTCLst.RemoveAt( ix );
					lstPrecTC.Items.RemoveAt( ix );
				}

				setAppStat( AppStat.OK );
			}
			catch( Exception excp ) {
			
				handleExcp( excp );
			} 
		}

		//-----------------------
		private void ctxtReqsEdit_Click(object sender, System.EventArgs e) {
		
			try {

				int i, x;

				Ref2DocVal aReq;

				/*
				<!ELEMENT ReqmntSrc (EMPTY)>
					<!ATTLIST ReqmntSrc
						Location	CDATA #IMPLIED
						RefUID		NMTOKEN #IMPLIED
						Title			CDATA #REQUIRED
						UID				NMTOKEN #IMPLIED
					> */
				
				setAppStat( AppStat.Proc );

				frmReqSel dialReqSel = new frmReqSel( m_Presets, m_ReqsLst );

				DialogResult dRes = dialReqSel.ShowDialog();

				if( DialogResult.OK == dRes ) {

					lstReqs.Items.Clear();

					x = m_ReqsLst.Count;

					for( i = 0; i < x; ++i ) {

						aReq = m_ReqsLst[i]; 

						lstReqs.Items.Add( aReq.Title );
					}
				}

				dialReqSel.Dispose();
				
				setAppStat( AppStat.OK );
			}
			catch( Exception excp ) {
					
				handleExcp( excp );
			} 
		}

		//------------------------
		private void ctxtReqsRem_Click(object sender, System.EventArgs e) {
				
			try {

				setAppStat( AppStat.Proc );

				int ix = lstReqs.SelectedIndex;
								
				if( ix > -1 ) {

					m_ReqsLst.RemoveAt( ix );
					lstReqs.Items.RemoveAt( ix );
				}

				setAppStat( AppStat.OK );
			}
			catch( Exception excp ) {
					
				handleExcp( excp );
			} 
		}

		//--------------------
		private void ctxtTSysSel_Click(object sender, System.EventArgs e) {

			try {

				string config;
				string ver;
				
				setAppStat( AppStat.Proc );

				frmTSys dialTSys = new frmTSys( ref m_Presets );
				DialogResult dRes = dialTSys.ShowDialog();

				if( DialogResult.OK == dRes ) {

					config = dialTSys.txtSys.Text;
					ver = "";
/*
					m_TSysConf[0] = config;
					m_TSysConf[1] = ver;
					m_TSysConf[2] = ""; // refUID
*/
					//txtTSysConfig.Text = config;
					
					if( ver.Length > 0 ) {
					
						//txtTSysConfig.Text += "; " + ver;
					}
				}

				dialTSys.Dispose();
				
				setAppStat( AppStat.OK );
			}
			catch( Exception excp ) {
					
				handleExcp( excp );
			}
		}
			
		//-------------------------
		private void menuAbout_Click(object sender, System.EventArgs e) {

			string title;
			string msg;

			setAppStat( AppStat.Proc );

			title = String.Format( m_Presets.m_QNphrase.msgAbout, m_Presets.m_QNphrase.lblTCEditor );

			msg = m_QNTools.getAboutInfoBlock();
			
			MessageBox.Show( msg, title, MessageBoxButtons.OK );	

			setAppStat( AppStat.OK );
		}

		//--------------------------
		private void menuDel_Click(object sender, System.EventArgs e) {

			DialogResult dRes;

			try {

				setAppStat( AppStat.Proc );

				string msg = String.Format( m_Presets.m_QNphrase.msgSure2dDel, txtTitle.Text );
			
				dRes = MessageBox.Show( msg, m_Presets.m_QNphrase.lblDeleting, MessageBoxButtons.YesNo, MessageBoxIcon.Question );

				if( DialogResult.Yes != dRes ) {
				
					setAppStat( AppStat.OK );
				}
				else {

					m_xDB.deleteXML( ref m_uidTC );

					if( m_xDB.notOK() ) {

						msg = String.Format( m_Presets.m_QNphrase.errDelFailed, txtTitle.Text );

						handleErr( m_xDB, msg );

						m_xDB.resetRslt();
					}
					else {

						addLog( String.Format( m_Presets.m_QNphrase.msgDeleted, txtTitle.Text ) );

						reset();

						setUp4New();

						setAppStat( AppStat.OK );
					}
				}
			}
			catch( Exception excp ) {
						
				handleExcp( excp );
			}
		}

		//-------------------------
		private void menuExit_Click(object sender, System.EventArgs e) {
		
			setAppStat( AppStat.Proc );

			Close();
		}

		//------------------------
		private void menuExport_Click(object sender, System.EventArgs e) {
		
			try {

				setAppStat( AppStat.Proc );

				SaveFileDialog saveFileDialTC;
				saveFileDialTC = new SaveFileDialog();
				saveFileDialTC.FileOk += new System.ComponentModel.CancelEventHandler(saveFileDialTC_FileOk);

				saveFileDialTC.Title = m_Presets.m_QNphrase.lblSaveTC;
				//openFileDialXML.Filter = "txt files (*.txt)|*.txt|All files (*.*)|*.*" ;
				//openFileDialXML.FilterIndex = 2 ;
				saveFileDialTC.DefaultExt = "xml";
				saveFileDialTC.Filter = "XML (*.xml)|*.xml";
				saveFileDialTC.FilterIndex = 1 ;

				string fName = m_QNTools.clearFileName( txtTitle.Text );

				saveFileDialTC.FileName = fName;

				saveFileDialTC.InitialDirectory = m_Presets.m_ExpPth;

				saveFileDialTC.RestoreDirectory = true ;
				
				DialogResult dRes = saveFileDialTC.ShowDialog();

				if( DialogResult.Cancel == dRes ) {

					setAppStat( AppStat.OK );
				}

				saveFileDialTC.Dispose();
			}
			catch( Exception excp ) {
						
				handleExcp( excp );
			}
		}

		//-----------------------
		private void menuImport_Click(object sender, System.EventArgs e) {
		
			try {

				setAppStat( AppStat.Proc );

				OpenFileDialog openFileDial;
				openFileDial = new OpenFileDialog();
				openFileDial.FileOk += new System.ComponentModel.CancelEventHandler(fileOKHdlr4TCImp);

				openFileDial.Title = m_Presets.m_QNphrase.lblOpenTC;
				//openFileDialXML.Filter = "txt files (*.txt)|*.txt|All files (*.*)|*.*" ;
				//openFileDialXML.FilterIndex = 2 ;
				openFileDial.DefaultExt = "xml";
				openFileDial.Filter = "XML (*.xml)|*.xml";
				openFileDial.FilterIndex = 1 ;

				openFileDial.InitialDirectory = m_Presets.m_ExpPth;

				openFileDial.RestoreDirectory = true ;
				
				DialogResult dRes = openFileDial.ShowDialog();

				if( DialogResult.Cancel == dRes ) {

					setAppStat( AppStat.OK );
				}

				openFileDial.Dispose();
			}
			catch( Exception excp ) {
						
				handleExcp( excp );
			}
		}

		//-------------------------
		private void menuNew_Click(object sender, System.EventArgs e) {
		
			setAppStat( AppStat.Proc );

			reset();

			setUp4New();

			setAppStat( AppStat.OK );
		}

		//------------------------
		private void menuOpen_Click(object sender, System.EventArgs e) {
		
			try {

				DataLst RsltLst;

				int x;
				DatLstEntry entry;
			
				setAppStat( AppStat.Proc );

				frmTCSel dialTCSel = new frmTCSel( m_xDB, true, m_Presets, m_TCSelPresets, "tc" );

				DialogResult dRes = dialTCSel.ShowDialog();

				if( DialogResult.Cancel == dRes ) {

					setAppStat( AppStat.OK );
				}
				else {

					RsltLst = dialTCSel.m_RsltLst;

					x = RsltLst.Count;

					if( x < 1 ) {

						setAppStat( AppStat.OK );
					}
					else {

						entry = RsltLst[0];

						m_loadUID = entry.val;

						//Thread loadThread = new Thread(new ThreadStart(loadTCloader));
						// Start the thread.
						//loadThread.Start();

						loadTCloader();
					}
				}

				dialTCSel.Dispose();
			}
			catch( Exception excp ) {
		
				handleExcp( excp );
			}
		}

		//----------------------
		private void menuSave_Click(object sender, System.EventArgs e) {
		
			try {

				RsltString UID;

				setAppStat( AppStat.Proc );
				
				UID = saveTC();

				if( UID.isOK() ) {

					Rslt res = loadTC( UID.theString );

					if( res.notOK() ) {

						handleErr( res, m_Presets.m_QNphrase.errTCLoadFailed );
					}
					else {

						addLog( String.Format( m_Presets.m_QNphrase.msgSaved, txtTitle.Text ) );
					
						setAppStat( AppStat.OK );
					}
				}
			}
			catch( Exception excp ) {
				
				handleExcp( excp );
			}  
		}

		//------------------------
		private void menuSaveNew_Click(object sender, System.EventArgs e) {
		
			try {

				RsltString UID;

				setAppStat( AppStat.Proc );

				resetUID4New();

				txtDate.Text = BTools.getNowISODate();
				txtTime.Text = BTools.getNowISOTime();

				UID = saveTC();

				if( UID.isOK() ) {

					Rslt res = loadTC( UID.theString );

					if( res.notOK() ) {

						handleErr( res, m_Presets.m_QNphrase.errTCLoadFailed );
					}
					else {

						addLog( String.Format( m_Presets.m_QNphrase.msgSavedNew, txtTitle.Text ) );
					
						setAppStat( AppStat.OK );
					}
				}
			}
			catch( Exception excp ) {
						
				handleExcp( excp );
			}
		}

		//-----------------------
		private void fileOKHdlr4TCImp(object sender, System.ComponentModel.CancelEventArgs e) {

			Rslt res;

			res = fileDialProc4OK(sender, e, new dlgtFileDialOK( loadTCdoc ) );

			if( res.isOK() ) {

				setLoaded( false );
			}
		}

		// ---------------------------
		/*
		private void fileOKHdlr4AddFile(object sender, System.ComponentModel.CancelEventArgs e) {

			Rslt res;

			res = fileDialProc4OK(sender, e, new dlgtFileDialOK( appAddFile ) );
		}
		*/

		// ---------------------------
		private void fileOKHdlr4ProcFile(object sender, System.ComponentModel.CancelEventArgs e) {

			Rslt res;

			res = fileDialProc4OK(sender, e, new dlgtFileDialOK( appProcFile ) );
		}


		// ---------------------------
		private void fileOKHdlr4RefFile(object sender, System.ComponentModel.CancelEventArgs e) {

			Rslt res;

			res = fileDialProc4OK(sender, e, new dlgtFileDialOK( appRefFile ) );
		}


		//----------------------------
		private void saveFileDialTC_FileOk(object sender, System.ComponentModel.CancelEventArgs e) {

			try {

				RsltXMLNode xNode;
				string fileName;

				SaveFileDialog DialXML = (SaveFileDialog)sender;

				fileName = DialXML.FileName;

				xNode = bldTCdoc();

				if( xNode.notOK() ) {

					string msg =  String.Format( m_Presets.m_QNphrase.errCrtngFailed, txtTitle.Text );

					handleErr( xNode, msg );
				}
				else {

					xNode.m_xDoc.Save( fileName );

					setAppStat( AppStat.OK );

					addLog( String.Format( m_Presets.m_QNphrase.msgSavedTo, txtTitle.Text, fileName ) );
				}
			}
			catch( Exception excp ) {
		
				handleExcp( excp );
			}          
		}

		//----------------------------
		private void txtDesc_DoubleClick(object sender, System.EventArgs e) {

			handleTxtPop( sender, lblDesc.Text );
		}

		//----------------------------
		private void txtTstProc_DoubleClick(object sender, System.EventArgs e) {

			handleTxtPop( sender, tabTstProcTxt.Text );
		}

		//----------------------------
		private void txtExpRslt_DoubleClick(object sender, System.EventArgs e) {

			handleTxtPop( sender, tabRsltTxt.Text );
		}

		// ----------------------------
		private void menuDocu_Click(object sender, System.EventArgs e) {
				
			try {

				setAppStat( AppStat.OK );

				System.Diagnostics.Process proc = new System.Diagnostics.Process();

				proc.EnableRaisingEvents = false;

				proc.StartInfo.FileName = QNDefs.manQN;

				proc.Start();
			}
			catch( Exception excp ) {
	
				handleExcp( excp );
			}
		}

		// ----------------------------
		private void butSaveNew_MouseLeave(object sender, System.EventArgs e) {
		
			butSaveNew.BackColor = m_Presets.m_ClrButBck;
		}

		// ----------------------------
		private void butSaveNew_MouseEnter(object sender, System.EventArgs e) {
		
			butSaveNew.BackColor = m_Presets.m_ClrButHigh;
		}

		// ----------------------------
		private void menuPrint_Click(object sender, System.EventArgs e) {

			string title;

			setAppStat( AppStat.Proc );

			Rslt res = new Rslt( true );

			RsltXMLNode xNode = bldTCdoc();

			string xsltName = QNDefs.xlsTestCase + "_" + m_QNphrase.m_Loc + ".xsl";

			if( res.checkRslt( xNode ) ) {

				title = m_QNphrase.lblTC + " '" + txtTitle.Text + "'";

				res = m_GUIservice.printXDoc( xNode.m_xDoc, ref xsltName, ref title );
			}

			if( res.notOK() ) {

				handleErr( res, m_Presets.m_QNphrase.errPrintFailed );
			}
			else {

				setAppStat( AppStat.OK );
			}
		}

		// -----------------------
		private void menuUID2Clip_Click(object sender, System.EventArgs e) {
		
			Clipboard.SetDataObject( m_uidTC );
		}

		// -----------------------
		private void frmTCEd_Resize(object sender, System.EventArgs e) {
		
			int x, y;

			y = this.Height;
			x = this.Width;

			panel1.Height = y - m_panel1DH;

			butSaveNew.Top = y - m_butSaveNewDT;

			txtStat.Top = y - m_txtStatDT;
			comboLog.Top = y - m_comboLogDT;
		}

		//-------------------------------
		private void ctxtAddFileAdd_Click(object sender, System.EventArgs e) {

			System.ComponentModel.CancelEventHandler fileOkHandler;

			fileOkHandler = new System.ComponentModel.CancelEventHandler( fileOKHdlr4ProcFile );

			doDialForAnyType( fileOkHandler );
		}

		//-------------------------------
		private void ctxtAddFileRem_Click(object sender, System.EventArgs e) {
		
		}

		//-------------------------------
		private void ctxtMenuPrecTC_Popup(object sender, System.EventArgs e) {
		
			if( lstPrecTC.SelectedIndex <  0 ) {

				ctxtPrecTCRem.Enabled = false;
			}
			else {

				ctxtPrecTCRem.Enabled = true;
			}
		}

		// -------------------------
		private void ctxtProcAdd_Click(object sender, System.EventArgs e) {
		
			System.ComponentModel.CancelEventHandler fileOkHandler;

			fileOkHandler = new System.ComponentModel.CancelEventHandler( fileOKHdlr4ProcFile );

			doDialForAnyType( fileOkHandler );
		}

		// -------------------------
		private void ctxtMenuProcFile_Popup(object sender, System.EventArgs e) {

			if( lstTstProcFile.Items.Count > 0 ) {

				ctxtProcAdd.Enabled = false;
			}
			else {

				ctxtProcAdd.Enabled = true;
			}

			if( lstTstProcFile.SelectedIndex <  0 ) {

				ctxtProcRem.Enabled = false;
			}
			else {

				ctxtProcRem.Enabled = true;
			}
		}

		// -------------------
		private void ctxtProcRem_Click(object sender, System.EventArgs e) {
		
			int ix;

			Ref2File r2f;

			string blobUID;

			ix = lstTstProcFile.SelectedIndex; 

			if( ix > -1 ) {

				r2f = m_TProcLst[ix];

				blobUID = r2f.RefUID;

				if( blobUID.Length > 0 ) {
// #######################
// delete Blob?!
				}

				m_TProcLst.RemoveAt( ix );

				lstTstProcFile.Items.RemoveAt( ix );
			}
		}

		// -------------------
		private void ctxtMenuRefFile_Popup(object sender, System.EventArgs e) {
		
			if( lstTstRefFile.Items.Count > 0 ) {

				ctxtRefAdd.Enabled = false;
			}
			else {

				ctxtRefAdd.Enabled = true;
			}

			if( lstTstRefFile.SelectedIndex <  0 ) {

				ctxtRefRem.Enabled = false;
			}
			else {

				ctxtRefRem.Enabled = true;
			}
		}

		// -------------------
		private void ctxtRefAdd_Click(object sender, System.EventArgs e) {
		
			System.ComponentModel.CancelEventHandler fileOkHandler;

			fileOkHandler = new System.ComponentModel.CancelEventHandler( fileOKHdlr4RefFile );

			doDialForAnyType( fileOkHandler );
		}

		// -------------------
		private void ctxtRefRem_Click(object sender, System.EventArgs e) {
		
			int ix;

			Ref2File r2f;

			string blobUID;

			ix = lstTstRefFile.SelectedIndex; 

			if( ix > -1 ) {

				r2f = m_TRefLst[ix];

				blobUID = r2f.RefUID;

				if( blobUID.Length > 0 ) {
					
// #######################
// delete Blob?!
				}

				m_TRefLst.RemoveAt( ix );

				lstTstRefFile.Items.RemoveAt( ix );
			}
		}

		#endregion

	}
}
