//---------------------------------------------------------
// QA Navigator Suite
// Copyright  2003 - 2007 Ronald D. Grindle
//---------------------------------------------------------

using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using System.Data;
using System.Threading;
using System.Xml;

using RG.Basics;
using RG.SQL;
using RG.XMLDB;

using QN.Basics;

namespace QN.TCSet
{
  
	/// <summary>
	/// QA Navigator suite
	/// Copyright  2003 - 2007 Ronald D. Grindle
	///
	/// Compiling TC Sets
	/// </summary>
	public class frmTCSetComp : System.Windows.Forms.Form
	{

		#region My Members

		private int m_lstTCDH, m_lstTCDW;
		private int m_txtStatDT;
		private int m_comboLogDT;
		private int m_panel1DH, m_panel1DW;
		private int m_colDW;

		private int m_tExpDur;
		private int m_TCCnt;

		private QNGUIservice m_GUIservice;
		private QNphrase m_QNphrase;
		private QNpresets m_Presets;
		private QNTools m_QNTools;

		private RefTCInfoLst m_TCLst; 

		private static SimpleLog m_Logger;

		private string m_ApplPth;

		private string m_loadUID;

		private string m_uidDesc;
		private string m_uidTCCol;
		private string m_uidTCS;

		private	TCSelPresets m_TCSSelPresets;
		private	TCSelPresets m_TCSelPresets;

        private RGSQL2 m_DB;
		private XMLDB m_xDB;

		#endregion

		#region Form Members

		private System.Windows.Forms.MainMenu mainMenu;
		private System.Windows.Forms.MenuItem menuItem9;
		private System.Windows.Forms.MenuItem menuItem10;
		private System.Windows.Forms.ComboBox comboLog;
		private System.Windows.Forms.TextBox txtStat;
		private System.Windows.Forms.Panel panel1;
		private System.Windows.Forms.Label lblTitle;
		private System.Windows.Forms.TextBox txtTitle;
		private System.Windows.Forms.TextBox txtDesc;
		private System.Windows.Forms.Label lblDesc;
		private System.Windows.Forms.TextBox txtAuthor;
		private System.Windows.Forms.Label lblAuthor;
		private System.Windows.Forms.TextBox txtDate;
		private System.Windows.Forms.Label lblDate;
		private System.Windows.Forms.TextBox txtTime;
		private System.Windows.Forms.Label lblTime;
		private System.Windows.Forms.Label lblTotTCs;
		private System.Windows.Forms.TextBox txtTotTCs;
		private System.Windows.Forms.Label lblSoftRel;
		private System.Windows.Forms.TextBox txtFctn;
		private System.Windows.Forms.Label lblFctn;
		private System.Windows.Forms.Label lblSection;
		private System.Windows.Forms.Label lblAppl;
		private System.Windows.Forms.ComboBox comboSection;
		private System.Windows.Forms.ComboBox comboAppl;
		private System.Windows.Forms.Label lblTCLst;
		private System.Windows.Forms.TextBox txtExpDur;
		private System.Windows.Forms.Label lblDuration;
		private System.Windows.Forms.ComboBox comboTCType;
		private System.Windows.Forms.Label lblTCType;
		private System.Windows.Forms.MenuItem menuItem12;
		private System.Windows.Forms.MenuItem menuOpen;
		private System.Windows.Forms.MenuItem menuNew;
		private System.Windows.Forms.MenuItem menuSave;
		private System.Windows.Forms.MenuItem menuFile;
		private System.Windows.Forms.MenuItem menuSaveNew;
		private System.Windows.Forms.MenuItem menuExit;
		private System.Windows.Forms.MenuItem menuHelp;
		private System.Windows.Forms.MenuItem menuAbout;
		private System.Windows.Forms.ContextMenu ctxtMenuLog;
		private System.Windows.Forms.MenuItem ctxtLogEntry;
		private System.Windows.Forms.MenuItem menuDel;
		private System.Windows.Forms.ColumnHeader colHeadTitle;
		private System.Windows.Forms.ListView lstVTC;
		private System.Windows.Forms.ColumnHeader columnHeader1;
		private System.Windows.Forms.ColumnHeader colHeadDur;
		private System.Windows.Forms.ContextMenu ctxtMenuTCLst;
		private System.Windows.Forms.MenuItem ctxtTCLstRem;
		private System.Windows.Forms.MenuItem ctxtTCLstView;
		private System.Windows.Forms.MenuItem menuItem2;
		private System.Windows.Forms.MenuItem menuDocu;
		private System.Windows.Forms.MenuItem ctxtTCLstAddTC;

		#endregion

		private System.Windows.Forms.ImageList imgLstTCType;
		private System.Windows.Forms.MenuItem ctxtTCLstAddTCS;
		private System.Windows.Forms.MenuItem menuItem3;
		private System.Windows.Forms.MenuItem menuPrint;
		private System.Windows.Forms.ComboBox comboApplRel;
		private System.ComponentModel.IContainer components;

		#region Construction/De-construction
    
		// --------------------
		public frmTCSetComp()
		{

			try {
				
				m_Logger = new SimpleLog( "TCSetComp" );

				InitializeComponent();

				init();
			}
			catch( Exception e ) {
		
				handleExcp( e );
			}
		}

		// --------------------
		public frmTCSetComp( string p_UID ) {

			try {

				Rslt res;
				
				m_Logger = new SimpleLog( "TCSetComp" );

				InitializeComponent();

				res = init();

				if( res.isOK() ) {

					loadTCS( p_UID );
				}
			}
			catch( Exception e ) {
		
				handleExcp( e );
			}
		}

		// -----------------------------------
		/// <summary>
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if (components != null) 
				{
					components.Dispose();
				}
                m_DB.close();
			}
			base.Dispose( disposing );
		}

		#endregion

		#region Windows Form Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
            this.components = new System.ComponentModel.Container();
            System.ComponentModel.ComponentResourceManager resources = new System.ComponentModel.ComponentResourceManager( typeof( frmTCSetComp ) );
            this.mainMenu = new System.Windows.Forms.MainMenu( this.components );
            this.menuFile = new System.Windows.Forms.MenuItem();
            this.menuNew = new System.Windows.Forms.MenuItem();
            this.menuOpen = new System.Windows.Forms.MenuItem();
            this.menuItem10 = new System.Windows.Forms.MenuItem();
            this.menuSave = new System.Windows.Forms.MenuItem();
            this.menuSaveNew = new System.Windows.Forms.MenuItem();
            this.menuItem12 = new System.Windows.Forms.MenuItem();
            this.menuDel = new System.Windows.Forms.MenuItem();
            this.menuItem9 = new System.Windows.Forms.MenuItem();
            this.menuPrint = new System.Windows.Forms.MenuItem();
            this.menuItem3 = new System.Windows.Forms.MenuItem();
            this.menuExit = new System.Windows.Forms.MenuItem();
            this.menuHelp = new System.Windows.Forms.MenuItem();
            this.menuDocu = new System.Windows.Forms.MenuItem();
            this.menuItem2 = new System.Windows.Forms.MenuItem();
            this.menuAbout = new System.Windows.Forms.MenuItem();
            this.comboLog = new System.Windows.Forms.ComboBox();
            this.ctxtMenuLog = new System.Windows.Forms.ContextMenu();
            this.ctxtLogEntry = new System.Windows.Forms.MenuItem();
            this.txtStat = new System.Windows.Forms.TextBox();
            this.panel1 = new System.Windows.Forms.Panel();
            this.comboApplRel = new System.Windows.Forms.ComboBox();
            this.comboTCType = new System.Windows.Forms.ComboBox();
            this.lblTCType = new System.Windows.Forms.Label();
            this.txtExpDur = new System.Windows.Forms.TextBox();
            this.lblDuration = new System.Windows.Forms.Label();
            this.lblTCLst = new System.Windows.Forms.Label();
            this.lblSoftRel = new System.Windows.Forms.Label();
            this.txtFctn = new System.Windows.Forms.TextBox();
            this.lblFctn = new System.Windows.Forms.Label();
            this.lblSection = new System.Windows.Forms.Label();
            this.lblAppl = new System.Windows.Forms.Label();
            this.comboSection = new System.Windows.Forms.ComboBox();
            this.comboAppl = new System.Windows.Forms.ComboBox();
            this.txtTotTCs = new System.Windows.Forms.TextBox();
            this.lblTotTCs = new System.Windows.Forms.Label();
            this.txtTime = new System.Windows.Forms.TextBox();
            this.lblTime = new System.Windows.Forms.Label();
            this.txtAuthor = new System.Windows.Forms.TextBox();
            this.lblAuthor = new System.Windows.Forms.Label();
            this.txtDate = new System.Windows.Forms.TextBox();
            this.lblDate = new System.Windows.Forms.Label();
            this.txtDesc = new System.Windows.Forms.TextBox();
            this.lblDesc = new System.Windows.Forms.Label();
            this.lblTitle = new System.Windows.Forms.Label();
            this.txtTitle = new System.Windows.Forms.TextBox();
            this.lstVTC = new System.Windows.Forms.ListView();
            this.columnHeader1 = new System.Windows.Forms.ColumnHeader();
            this.colHeadTitle = new System.Windows.Forms.ColumnHeader();
            this.colHeadDur = new System.Windows.Forms.ColumnHeader();
            this.ctxtMenuTCLst = new System.Windows.Forms.ContextMenu();
            this.ctxtTCLstAddTC = new System.Windows.Forms.MenuItem();
            this.ctxtTCLstAddTCS = new System.Windows.Forms.MenuItem();
            this.ctxtTCLstRem = new System.Windows.Forms.MenuItem();
            this.ctxtTCLstView = new System.Windows.Forms.MenuItem();
            this.imgLstTCType = new System.Windows.Forms.ImageList( this.components );
            this.panel1.SuspendLayout();
            this.SuspendLayout();
            // 
            // mainMenu
            // 
            this.mainMenu.MenuItems.AddRange( new System.Windows.Forms.MenuItem[] {
            this.menuFile,
            this.menuHelp} );
            // 
            // menuFile
            // 
            this.menuFile.Index = 0;
            this.menuFile.MenuItems.AddRange( new System.Windows.Forms.MenuItem[] {
            this.menuNew,
            this.menuOpen,
            this.menuItem10,
            this.menuSave,
            this.menuSaveNew,
            this.menuItem12,
            this.menuDel,
            this.menuItem9,
            this.menuPrint,
            this.menuItem3,
            this.menuExit} );
            this.menuFile.Text = "File";
            // 
            // menuNew
            // 
            this.menuNew.Index = 0;
            this.menuNew.Text = "New";
            this.menuNew.Click += new System.EventHandler( this.menuNew_Click );
            // 
            // menuOpen
            // 
            this.menuOpen.Index = 1;
            this.menuOpen.Text = "Open";
            this.menuOpen.Click += new System.EventHandler( this.menuOpen_Click );
            // 
            // menuItem10
            // 
            this.menuItem10.Index = 2;
            this.menuItem10.Text = "-";
            // 
            // menuSave
            // 
            this.menuSave.Index = 3;
            this.menuSave.Shortcut = System.Windows.Forms.Shortcut.CtrlS;
            this.menuSave.Text = "Save";
            this.menuSave.Click += new System.EventHandler( this.menuSave_Click );
            // 
            // menuSaveNew
            // 
            this.menuSaveNew.Index = 4;
            this.menuSaveNew.Text = "Save As New";
            this.menuSaveNew.Click += new System.EventHandler( this.menuSaveNew_Click );
            // 
            // menuItem12
            // 
            this.menuItem12.Index = 5;
            this.menuItem12.Text = "-";
            // 
            // menuDel
            // 
            this.menuDel.Index = 6;
            this.menuDel.Text = "Delete";
            this.menuDel.Click += new System.EventHandler( this.menuDel_Click );
            // 
            // menuItem9
            // 
            this.menuItem9.Index = 7;
            this.menuItem9.Text = "-";
            // 
            // menuPrint
            // 
            this.menuPrint.Index = 8;
            this.menuPrint.Text = "Print";
            this.menuPrint.Click += new System.EventHandler( this.menuPrint_Click );
            // 
            // menuItem3
            // 
            this.menuItem3.Index = 9;
            this.menuItem3.Text = "-";
            // 
            // menuExit
            // 
            this.menuExit.Index = 10;
            this.menuExit.Text = "Exit";
            this.menuExit.Click += new System.EventHandler( this.menuExit_Click );
            // 
            // menuHelp
            // 
            this.menuHelp.Index = 1;
            this.menuHelp.MenuItems.AddRange( new System.Windows.Forms.MenuItem[] {
            this.menuDocu,
            this.menuItem2,
            this.menuAbout} );
            this.menuHelp.Text = "Help";
            // 
            // menuDocu
            // 
            this.menuDocu.Index = 0;
            this.menuDocu.Text = "Docu";
            this.menuDocu.Click += new System.EventHandler( this.menuDocu_Click );
            // 
            // menuItem2
            // 
            this.menuItem2.Index = 1;
            this.menuItem2.Text = "-";
            // 
            // menuAbout
            // 
            this.menuAbout.Index = 2;
            this.menuAbout.Text = "About";
            this.menuAbout.Click += new System.EventHandler( this.menuAbout_Click );
            // 
            // comboLog
            // 
            this.comboLog.ContextMenu = this.ctxtMenuLog;
            this.comboLog.DropDownStyle = System.Windows.Forms.ComboBoxStyle.DropDownList;
            this.comboLog.Font = new System.Drawing.Font( "Microsoft Sans Serif", 9.75F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ( (byte)( 0 ) ) );
            this.comboLog.ItemHeight = 20;
            this.comboLog.Location = new System.Drawing.Point( 155, 789 );
            this.comboLog.Name = "comboLog";
            this.comboLog.Size = new System.Drawing.Size( 736, 28 );
            this.comboLog.TabIndex = 0;
            this.comboLog.TabStop = false;
            // 
            // ctxtMenuLog
            // 
            this.ctxtMenuLog.MenuItems.AddRange( new System.Windows.Forms.MenuItem[] {
            this.ctxtLogEntry} );
            this.ctxtMenuLog.Popup += new System.EventHandler( this.ctxtMenuLog_Popup );
            // 
            // ctxtLogEntry
            // 
            this.ctxtLogEntry.Index = 0;
            this.ctxtLogEntry.Text = "Display Entry";
            // 
            // txtStat
            // 
            this.txtStat.BackColor = System.Drawing.Color.Yellow;
            this.txtStat.Font = new System.Drawing.Font( "Microsoft Sans Serif", 9.75F, System.Drawing.FontStyle.Bold, System.Drawing.GraphicsUnit.Point, ( (byte)( 0 ) ) );
            this.txtStat.Location = new System.Drawing.Point( 5, 790 );
            this.txtStat.Name = "txtStat";
            this.txtStat.ReadOnly = true;
            this.txtStat.Size = new System.Drawing.Size( 144, 26 );
            this.txtStat.TabIndex = 0;
            this.txtStat.TabStop = false;
            this.txtStat.Text = "Initializing...";
            this.txtStat.TextAlign = System.Windows.Forms.HorizontalAlignment.Center;
            // 
            // panel1
            // 
            this.panel1.BorderStyle = System.Windows.Forms.BorderStyle.Fixed3D;
            this.panel1.Controls.Add( this.comboApplRel );
            this.panel1.Controls.Add( this.comboTCType );
            this.panel1.Controls.Add( this.lblTCType );
            this.panel1.Controls.Add( this.txtExpDur );
            this.panel1.Controls.Add( this.lblDuration );
            this.panel1.Controls.Add( this.lblTCLst );
            this.panel1.Controls.Add( this.lblSoftRel );
            this.panel1.Controls.Add( this.txtFctn );
            this.panel1.Controls.Add( this.lblFctn );
            this.panel1.Controls.Add( this.lblSection );
            this.panel1.Controls.Add( this.lblAppl );
            this.panel1.Controls.Add( this.comboSection );
            this.panel1.Controls.Add( this.comboAppl );
            this.panel1.Controls.Add( this.txtTotTCs );
            this.panel1.Controls.Add( this.lblTotTCs );
            this.panel1.Controls.Add( this.txtTime );
            this.panel1.Controls.Add( this.lblTime );
            this.panel1.Controls.Add( this.txtAuthor );
            this.panel1.Controls.Add( this.lblAuthor );
            this.panel1.Controls.Add( this.txtDate );
            this.panel1.Controls.Add( this.lblDate );
            this.panel1.Controls.Add( this.txtDesc );
            this.panel1.Controls.Add( this.lblDesc );
            this.panel1.Controls.Add( this.lblTitle );
            this.panel1.Controls.Add( this.txtTitle );
            this.panel1.Controls.Add( this.lstVTC );
            this.panel1.Location = new System.Drawing.Point( 0, 0 );
            this.panel1.Name = "panel1";
            this.panel1.Size = new System.Drawing.Size( 896, 785 );
            this.panel1.TabIndex = 0;
            // 
            // comboApplRel
            // 
            this.comboApplRel.Font = new System.Drawing.Font( "Microsoft Sans Serif", 9.75F );
            this.comboApplRel.ItemHeight = 20;
            this.comboApplRel.Location = new System.Drawing.Point( 732, 84 );
            this.comboApplRel.Name = "comboApplRel";
            this.comboApplRel.Size = new System.Drawing.Size( 148, 28 );
            this.comboApplRel.TabIndex = 15;
            // 
            // comboTCType
            // 
            this.comboTCType.Font = new System.Drawing.Font( "Microsoft Sans Serif", 9.75F );
            this.comboTCType.ItemHeight = 20;
            this.comboTCType.Location = new System.Drawing.Point( 732, 46 );
            this.comboTCType.Name = "comboTCType";
            this.comboTCType.Size = new System.Drawing.Size( 148, 28 );
            this.comboTCType.TabIndex = 3;
            // 
            // lblTCType
            // 
            this.lblTCType.Font = new System.Drawing.Font( "Microsoft Sans Serif", 9.75F );
            this.lblTCType.ImeMode = System.Windows.Forms.ImeMode.NoControl;
            this.lblTCType.Location = new System.Drawing.Point( 635, 47 );
            this.lblTCType.Name = "lblTCType";
            this.lblTCType.Size = new System.Drawing.Size( 93, 28 );
            this.lblTCType.TabIndex = 14;
            this.lblTCType.Text = "TC Type";
            this.lblTCType.TextAlign = System.Drawing.ContentAlignment.MiddleLeft;
            // 
            // txtExpDur
            // 
            this.txtExpDur.BackColor = System.Drawing.Color.LightSteelBlue;
            this.txtExpDur.Font = new System.Drawing.Font( "Microsoft Sans Serif", 9.75F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ( (byte)( 0 ) ) );
            this.txtExpDur.Location = new System.Drawing.Point( 741, 162 );
            this.txtExpDur.Name = "txtExpDur";
            this.txtExpDur.Size = new System.Drawing.Size( 70, 26 );
            this.txtExpDur.TabIndex = 0;
            this.txtExpDur.TabStop = false;
            this.txtExpDur.Text = "999";
            // 
            // lblDuration
            // 
            this.lblDuration.Font = new System.Drawing.Font( "Microsoft Sans Serif", 9.75F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ( (byte)( 0 ) ) );
            this.lblDuration.Location = new System.Drawing.Point( 635, 163 );
            this.lblDuration.Name = "lblDuration";
            this.lblDuration.Size = new System.Drawing.Size( 106, 26 );
            this.lblDuration.TabIndex = 12;
            this.lblDuration.Text = "Duration";
            this.lblDuration.TextAlign = System.Drawing.ContentAlignment.MiddleLeft;
            // 
            // lblTCLst
            // 
            this.lblTCLst.Location = new System.Drawing.Point( 11, 269 );
            this.lblTCLst.Name = "lblTCLst";
            this.lblTCLst.Size = new System.Drawing.Size( 149, 25 );
            this.lblTCLst.TabIndex = 0;
            this.lblTCLst.Text = "TC List";
            this.lblTCLst.TextAlign = System.Drawing.ContentAlignment.MiddleLeft;
            // 
            // lblSoftRel
            // 
            this.lblSoftRel.Font = new System.Drawing.Font( "Microsoft Sans Serif", 9.75F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ( (byte)( 0 ) ) );
            this.lblSoftRel.ImeMode = System.Windows.Forms.ImeMode.NoControl;
            this.lblSoftRel.Location = new System.Drawing.Point( 635, 87 );
            this.lblSoftRel.Name = "lblSoftRel";
            this.lblSoftRel.Size = new System.Drawing.Size( 93, 26 );
            this.lblSoftRel.TabIndex = 0;
            this.lblSoftRel.Text = "Release";
            this.lblSoftRel.TextAlign = System.Drawing.ContentAlignment.MiddleLeft;
            // 
            // txtFctn
            // 
            this.txtFctn.Font = new System.Drawing.Font( "Microsoft Sans Serif", 9.75F );
            this.txtFctn.Location = new System.Drawing.Point( 149, 127 );
            this.txtFctn.Name = "txtFctn";
            this.txtFctn.Size = new System.Drawing.Size( 475, 26 );
            this.txtFctn.TabIndex = 6;
            this.txtFctn.Text = "Function";
            // 
            // lblFctn
            // 
            this.lblFctn.Font = new System.Drawing.Font( "Microsoft Sans Serif", 9.75F );
            this.lblFctn.ImeMode = System.Windows.Forms.ImeMode.NoControl;
            this.lblFctn.Location = new System.Drawing.Point( 5, 128 );
            this.lblFctn.Name = "lblFctn";
            this.lblFctn.Size = new System.Drawing.Size( 144, 25 );
            this.lblFctn.TabIndex = 0;
            this.lblFctn.Text = "Function";
            this.lblFctn.TextAlign = System.Drawing.ContentAlignment.MiddleLeft;
            // 
            // lblSection
            // 
            this.lblSection.Font = new System.Drawing.Font( "Microsoft Sans Serif", 9.75F );
            this.lblSection.ImeMode = System.Windows.Forms.ImeMode.NoControl;
            this.lblSection.Location = new System.Drawing.Point( 5, 48 );
            this.lblSection.Name = "lblSection";
            this.lblSection.Size = new System.Drawing.Size( 144, 25 );
            this.lblSection.TabIndex = 0;
            this.lblSection.Text = "Section";
            this.lblSection.TextAlign = System.Drawing.ContentAlignment.MiddleLeft;
            // 
            // lblAppl
            // 
            this.lblAppl.Font = new System.Drawing.Font( "Microsoft Sans Serif", 9.75F );
            this.lblAppl.ImeMode = System.Windows.Forms.ImeMode.NoControl;
            this.lblAppl.Location = new System.Drawing.Point( 5, 87 );
            this.lblAppl.Name = "lblAppl";
            this.lblAppl.Size = new System.Drawing.Size( 144, 26 );
            this.lblAppl.TabIndex = 0;
            this.lblAppl.Text = "Application";
            this.lblAppl.TextAlign = System.Drawing.ContentAlignment.MiddleLeft;
            // 
            // comboSection
            // 
            this.comboSection.DropDownStyle = System.Windows.Forms.ComboBoxStyle.DropDownList;
            this.comboSection.Font = new System.Drawing.Font( "Microsoft Sans Serif", 9.75F );
            this.comboSection.ItemHeight = 20;
            this.comboSection.Location = new System.Drawing.Point( 149, 46 );
            this.comboSection.Name = "comboSection";
            this.comboSection.Size = new System.Drawing.Size( 475, 28 );
            this.comboSection.TabIndex = 2;
            // 
            // comboAppl
            // 
            this.comboAppl.DropDownStyle = System.Windows.Forms.ComboBoxStyle.DropDownList;
            this.comboAppl.Font = new System.Drawing.Font( "Microsoft Sans Serif", 9.75F );
            this.comboAppl.ItemHeight = 20;
            this.comboAppl.Location = new System.Drawing.Point( 149, 85 );
            this.comboAppl.Name = "comboAppl";
            this.comboAppl.Size = new System.Drawing.Size( 475, 28 );
            this.comboAppl.TabIndex = 4;
            // 
            // txtTotTCs
            // 
            this.txtTotTCs.BackColor = System.Drawing.Color.LightSteelBlue;
            this.txtTotTCs.Font = new System.Drawing.Font( "Microsoft Sans Serif", 9.75F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ( (byte)( 0 ) ) );
            this.txtTotTCs.Location = new System.Drawing.Point( 741, 127 );
            this.txtTotTCs.Name = "txtTotTCs";
            this.txtTotTCs.ReadOnly = true;
            this.txtTotTCs.Size = new System.Drawing.Size( 70, 26 );
            this.txtTotTCs.TabIndex = 0;
            this.txtTotTCs.TabStop = false;
            this.txtTotTCs.Text = "99999";
            // 
            // lblTotTCs
            // 
            this.lblTotTCs.Font = new System.Drawing.Font( "Microsoft Sans Serif", 9.75F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ( (byte)( 0 ) ) );
            this.lblTotTCs.Location = new System.Drawing.Point( 635, 128 );
            this.lblTotTCs.Name = "lblTotTCs";
            this.lblTotTCs.Size = new System.Drawing.Size( 106, 25 );
            this.lblTotTCs.TabIndex = 0;
            this.lblTotTCs.Text = "TCs";
            this.lblTotTCs.TextAlign = System.Drawing.ContentAlignment.MiddleLeft;
            // 
            // txtTime
            // 
            this.txtTime.Font = new System.Drawing.Font( "Microsoft Sans Serif", 9.75F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ( (byte)( 0 ) ) );
            this.txtTime.Location = new System.Drawing.Point( 533, 162 );
            this.txtTime.Name = "txtTime";
            this.txtTime.Size = new System.Drawing.Size( 54, 26 );
            this.txtTime.TabIndex = 9;
            this.txtTime.TabStop = false;
            this.txtTime.Text = "01:59";
            // 
            // lblTime
            // 
            this.lblTime.Font = new System.Drawing.Font( "Microsoft Sans Serif", 9.75F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ( (byte)( 0 ) ) );
            this.lblTime.ImeMode = System.Windows.Forms.ImeMode.NoControl;
            this.lblTime.Location = new System.Drawing.Point( 448, 163 );
            this.lblTime.Name = "lblTime";
            this.lblTime.Size = new System.Drawing.Size( 85, 26 );
            this.lblTime.TabIndex = 0;
            this.lblTime.Text = "Time";
            this.lblTime.TextAlign = System.Drawing.ContentAlignment.MiddleLeft;
            // 
            // txtAuthor
            // 
            this.txtAuthor.Font = new System.Drawing.Font( "Microsoft Sans Serif", 9.75F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ( (byte)( 0 ) ) );
            this.txtAuthor.Location = new System.Drawing.Point( 149, 162 );
            this.txtAuthor.Name = "txtAuthor";
            this.txtAuthor.Size = new System.Drawing.Size( 96, 26 );
            this.txtAuthor.TabIndex = 7;
            this.txtAuthor.Text = "xxx";
            // 
            // lblAuthor
            // 
            this.lblAuthor.Font = new System.Drawing.Font( "Microsoft Sans Serif", 9.75F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ( (byte)( 0 ) ) );
            this.lblAuthor.ImeMode = System.Windows.Forms.ImeMode.NoControl;
            this.lblAuthor.Location = new System.Drawing.Point( 5, 163 );
            this.lblAuthor.Name = "lblAuthor";
            this.lblAuthor.Size = new System.Drawing.Size( 144, 26 );
            this.lblAuthor.TabIndex = 0;
            this.lblAuthor.Text = "Author";
            this.lblAuthor.TextAlign = System.Drawing.ContentAlignment.MiddleLeft;
            // 
            // txtDate
            // 
            this.txtDate.Font = new System.Drawing.Font( "Microsoft Sans Serif", 9.75F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ( (byte)( 0 ) ) );
            this.txtDate.Location = new System.Drawing.Point( 341, 162 );
            this.txtDate.Name = "txtDate";
            this.txtDate.Size = new System.Drawing.Size( 96, 26 );
            this.txtDate.TabIndex = 8;
            this.txtDate.TabStop = false;
            this.txtDate.Text = "2003-12-31";
            // 
            // lblDate
            // 
            this.lblDate.Font = new System.Drawing.Font( "Microsoft Sans Serif", 9.75F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ( (byte)( 0 ) ) );
            this.lblDate.ImeMode = System.Windows.Forms.ImeMode.NoControl;
            this.lblDate.Location = new System.Drawing.Point( 256, 163 );
            this.lblDate.Name = "lblDate";
            this.lblDate.Size = new System.Drawing.Size( 85, 26 );
            this.lblDate.TabIndex = 0;
            this.lblDate.Text = "Date";
            this.lblDate.TextAlign = System.Drawing.ContentAlignment.MiddleLeft;
            // 
            // txtDesc
            // 
            this.txtDesc.AcceptsReturn = true;
            this.txtDesc.AcceptsTab = true;
            this.txtDesc.Font = new System.Drawing.Font( "Microsoft Sans Serif", 9.75F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ( (byte)( 0 ) ) );
            this.txtDesc.Location = new System.Drawing.Point( 149, 198 );
            this.txtDesc.Multiline = true;
            this.txtDesc.Name = "txtDesc";
            this.txtDesc.ScrollBars = System.Windows.Forms.ScrollBars.Vertical;
            this.txtDesc.Size = new System.Drawing.Size( 731, 65 );
            this.txtDesc.TabIndex = 10;
            this.txtDesc.Text = "Description";
            this.txtDesc.DoubleClick += new System.EventHandler( this.txtDesc_DoubleClick );
            // 
            // lblDesc
            // 
            this.lblDesc.Font = new System.Drawing.Font( "Microsoft Sans Serif", 9.75F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ( (byte)( 0 ) ) );
            this.lblDesc.ImeMode = System.Windows.Forms.ImeMode.NoControl;
            this.lblDesc.Location = new System.Drawing.Point( 5, 203 );
            this.lblDesc.Name = "lblDesc";
            this.lblDesc.Size = new System.Drawing.Size( 144, 25 );
            this.lblDesc.TabIndex = 0;
            this.lblDesc.Text = "Description";
            this.lblDesc.TextAlign = System.Drawing.ContentAlignment.MiddleLeft;
            // 
            // lblTitle
            // 
            this.lblTitle.Font = new System.Drawing.Font( "Microsoft Sans Serif", 9.75F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ( (byte)( 0 ) ) );
            this.lblTitle.ImeMode = System.Windows.Forms.ImeMode.NoControl;
            this.lblTitle.Location = new System.Drawing.Point( 5, 11 );
            this.lblTitle.Name = "lblTitle";
            this.lblTitle.Size = new System.Drawing.Size( 144, 26 );
            this.lblTitle.TabIndex = 0;
            this.lblTitle.Text = "Title";
            this.lblTitle.TextAlign = System.Drawing.ContentAlignment.MiddleLeft;
            // 
            // txtTitle
            // 
            this.txtTitle.Font = new System.Drawing.Font( "Microsoft Sans Serif", 9.75F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ( (byte)( 0 ) ) );
            this.txtTitle.Location = new System.Drawing.Point( 149, 10 );
            this.txtTitle.Name = "txtTitle";
            this.txtTitle.Size = new System.Drawing.Size( 731, 26 );
            this.txtTitle.TabIndex = 1;
            this.txtTitle.Text = "Title";
            // 
            // lstVTC
            // 
            this.lstVTC.Columns.AddRange( new System.Windows.Forms.ColumnHeader[] {
            this.columnHeader1,
            this.colHeadTitle,
            this.colHeadDur} );
            this.lstVTC.ContextMenu = this.ctxtMenuTCLst;
            this.lstVTC.Font = new System.Drawing.Font( "Microsoft Sans Serif", 9.75F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ( (byte)( 0 ) ) );
            this.lstVTC.FullRowSelect = true;
            this.lstVTC.GridLines = true;
            this.lstVTC.Location = new System.Drawing.Point( 5, 299 );
            this.lstVTC.Name = "lstVTC";
            this.lstVTC.Size = new System.Drawing.Size( 880, 476 );
            this.lstVTC.SmallImageList = this.imgLstTCType;
            this.lstVTC.TabIndex = 11;
            this.lstVTC.UseCompatibleStateImageBehavior = false;
            this.lstVTC.View = System.Windows.Forms.View.Details;
            // 
            // columnHeader1
            // 
            this.columnHeader1.Text = "";
            this.columnHeader1.Width = 20;
            // 
            // colHeadTitle
            // 
            this.colHeadTitle.Width = 564;
            // 
            // colHeadDur
            // 
            this.colHeadDur.Text = "duration";
            this.colHeadDur.Width = 72;
            // 
            // ctxtMenuTCLst
            // 
            this.ctxtMenuTCLst.MenuItems.AddRange( new System.Windows.Forms.MenuItem[] {
            this.ctxtTCLstAddTC,
            this.ctxtTCLstAddTCS,
            this.ctxtTCLstRem,
            this.ctxtTCLstView} );
            this.ctxtMenuTCLst.Popup += new System.EventHandler( this.ctxtMenuTCLst_Popup );
            // 
            // ctxtTCLstAddTC
            // 
            this.ctxtTCLstAddTC.Index = 0;
            this.ctxtTCLstAddTC.Text = "addTC";
            this.ctxtTCLstAddTC.Click += new System.EventHandler( this.ctxtTCLstAdd_Click );
            // 
            // ctxtTCLstAddTCS
            // 
            this.ctxtTCLstAddTCS.Index = 1;
            this.ctxtTCLstAddTCS.Text = "add TCS";
            this.ctxtTCLstAddTCS.Click += new System.EventHandler( this.ctxtTCLstAddTCS_Click );
            // 
            // ctxtTCLstRem
            // 
            this.ctxtTCLstRem.Index = 2;
            this.ctxtTCLstRem.Text = "remove";
            this.ctxtTCLstRem.Click += new System.EventHandler( this.ctxtTCLstRem_Click );
            // 
            // ctxtTCLstView
            // 
            this.ctxtTCLstView.Index = 3;
            this.ctxtTCLstView.Text = "view";
            this.ctxtTCLstView.Click += new System.EventHandler( this.ctxtTCLstView_Click );
            // 
            // imgLstTCType
            // 
            this.imgLstTCType.ImageStream = ( (System.Windows.Forms.ImageListStreamer)( resources.GetObject( "imgLstTCType.ImageStream" ) ) );
            this.imgLstTCType.TransparentColor = System.Drawing.Color.Transparent;
            this.imgLstTCType.Images.SetKeyName( 0, "" );
            this.imgLstTCType.Images.SetKeyName( 1, "" );
            // 
            // frmTCSetComp
            // 
            this.AutoScaleBaseSize = new System.Drawing.Size( 8, 19 );
            this.ClientSize = new System.Drawing.Size( 901, 825 );
            this.Controls.Add( this.panel1 );
            this.Controls.Add( this.comboLog );
            this.Controls.Add( this.txtStat );
            this.Font = new System.Drawing.Font( "Microsoft Sans Serif", 9.75F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ( (byte)( 0 ) ) );
            this.Icon = ( (System.Drawing.Icon)( resources.GetObject( "$this.Icon" ) ) );
            this.Menu = this.mainMenu;
            this.Name = "frmTCSetComp";
            this.Text = "TC Set Compiler";
            this.Resize += new System.EventHandler( this.frmTCSetComp_Resize );
            this.panel1.ResumeLayout( false );
            this.panel1.PerformLayout();
            this.ResumeLayout( false );
            this.PerformLayout();

		}
		#endregion

		/// <summary>
		/// The main entry point for the application.
		/// </summary>
		[STAThread]
		static void Main( string[] args ) 
		{

			if( args.Length < 1 ) {

				Application.Run(new frmTCSetComp());
			}
			else {

				Application.Run(new frmTCSetComp( args[0] ));
			}
		}

		#region My Stuff

		//-------------------------------------------------------------------------					
		private void addLog( IRslt p_Res ) {

			int ix;

			string msg = BTools.getNowISOTime() + " " + p_Res.getRsltLastMsg();

			ix = comboLog.Items.Add( msg );

			m_Logger.write2Log( ref p_Res );

			comboLog.SelectedIndex = ix;
		}

		//--------------------------
		private void addLog( string p_Msg ) {

			int ix;

			string msg = BTools.getNowISOTime() + " " + p_Msg;

			ix = comboLog.Items.Add( msg );

			m_Logger.write2Log( msg );

			comboLog.SelectedIndex = ix;
		}

		//--------------------------
		private void addTCDial( string p_TCType ) {

			try {
				
				int i, x;

				DatLstEntry entry;
				DataLst RsltLst;

				RefTCInfo refTC;

				string expDur;
				string ttc;
	
				setAppStat( AppStat.Proc );

				frmTCSel dialTCSel = new frmTCSel( m_xDB, false, m_Presets, m_TCSelPresets, p_TCType );

				DialogResult dRes = dialTCSel.ShowDialog();

				if( DialogResult.Cancel == dRes ) {

					setAppStat( AppStat.OK );
				}
				else {

					RsltLst = dialTCSel.m_RsltLst;

					x = RsltLst.Count;

					if( x > 0 ) {

						for( i = 0; i < x; i++ ) {

							entry = RsltLst[i];

							refTC = new RefTCInfo( entry.label, entry.val, p_TCType );
							
							expDur = m_xDB.getAttrVal( entry.val, "ExpDur" );							
							refTC.m_ExpDur = expDur;

							if( refTC.IsSet ) {

								ttc = m_xDB.getAttrVal( entry.val, "TotTCs" );							
								refTC.m_TotTCs = ttc;
							}

							addTCtoLst( ref refTC );
						}

						updateCounter();
					}
				}

				dialTCSel.Dispose();

				//lstVTC.Refresh();

				setAppStat( AppStat.OK );
			}
			catch( Exception excp ) {
		
				handleExcp( excp );
			}
		}

		//---------------------------------
		private void addTCtoLst( ref RefTCInfo p_Entry ) {
			
			string expDur;

			ListViewItem lvItem;

			expDur = p_Entry.m_ExpDur;

			m_TCLst.Add( p_Entry );

			lvItem = new ListViewItem();

			lvItem.SubItems.Add( p_Entry.Title );
			lvItem.SubItems.Add( expDur );

			if( expDur.Length > 0 ) {

				m_tExpDur += Convert.ToInt32( expDur );
			}

			lstVTC.Items.Add( lvItem );

			if( p_Entry.IsSet ) {

				lvItem.ImageIndex = 1;

				if( p_Entry.m_TotTCs.Length > 0 ) {

					m_TCCnt += Convert.ToInt32( p_Entry.m_TotTCs );
				}
			}
			else {

				lvItem.ImageIndex = 0;

				m_TCCnt += 1;
			}
		}

		//---------------------------------
		private RsltXMLNode bldTCSdoc() {

			int i, x;

			RefTCInfo refTC;

			RsltXMLNode aNode;
			RsltXMLNode bNode;
			RsltXMLNode TCSNode;

			string val;
			RsltString rVal;

			/*
			<!ELEMENT TCSet ( Desc, ReqmntSrc*, TCCol, TSysConfigRef?, ProcLog* )>
				<!ATTLIST TCSet
					Appl			CDATA #IMPLIED
					Author		CDATA #REQUIRED
					Date			NMTOKEN #REQUIRED
					ExpDur		NMTOKEN #IMPLIED
					Fctn			CDATA #IMPLIED
					Section		CDATA #IMPLIED
					SoftRel		CDATA #IMPLIED
					TCType		(ac|co|do|fi|fu|lo|mo|pe|se|un|us|vo|prep) ""
					Time			NMTOKEN #REQUIRED
					Title			CDATA #REQUIRED
					UID				NMTOKEN #IMPLIED
				>

				<!ELEMENT TCCol (RefTC*)>
					<!ATTLIST TCCol
						UID			NMTOKEN #IMPLIED
					>
			*/	

			string nodeName = "TCSet";
			TCSNode = new RsltXMLNode( nodeName );

			if( TCSNode.isOK() ) {

				//<?xml version="1.0"?>
				TCSNode.createXmlDecl( "1.0", "", "" );

				//<!DOCTYPE TestCase SYSTEM "RG.TCSetV01.dtd">
				TCSNode.addSysDocTypeInfo( nodeName, QNDefs.dtdNameTCS );

				//<!-- Generated by QA Navigator Test Set Compiler V1.0.0.0-->
				string genCom = QNDefs.phrGenBy + m_QNTools.getApplInfo();
				TCSNode.addComment( genCom );

				TCSNode.addUID( m_uidTCS );
				
				val = m_QNTools.getComboSelVal( ref comboAppl );
				TCSNode.addAttribAsHtml( "Appl", val );

				TCSNode.addAttribAsHtml( "Author", txtAuthor.Text );

				rVal = m_QNTools.chkVal( txtDate.Text, lblDate.Text );

				if( TCSNode.checkRslt( rVal ) ) {

					val = rVal.theString;

					rVal = m_QNTools.chkVal( txtTime.Text, lblTime.Text );

					if( TCSNode.checkRslt( rVal ) ) {

						val += "T" + rVal.theString;

						TCSNode.addAttrib( "DateTime", val );
					}				
				}
			}

			if( TCSNode.isOK() ) {

				TCSNode.addAttrib( "ExpDur", txtExpDur.Text );

				TCSNode.addAttribAsHtml( "Fctn", txtFctn.Text );

				val = m_QNTools.getComboSelVal( ref comboSection );
				TCSNode.addAttribAsHtml( "Section", val );

				val = m_QNTools.getComboSelVal( ref comboApplRel );
				TCSNode.addAttribAsHtml( "SoftRel", val );

				val = m_QNTools.getComboSelVal( ref comboTCType );
				TCSNode.addAttribAsHtml( "TCType", val );

				rVal = m_QNTools.chkVal( txtTitle.Text, lblTitle.Text );

				if( TCSNode.checkRslt( rVal ) ) {

					TCSNode.addAttribAsHtml( "Title", rVal );

					TCSNode.addAttrib( "TotTCs", txtTotTCs.Text );
				}
			}
					
			if( TCSNode.isOK() ) {

				aNode = TCSNode.appTxtNodeAsHTML( "Desc", txtDesc.Text );
				aNode.addUID( m_uidDesc );

				/*
				<!ELEMENT TCCol (RefTC*)>
					<!ATTLIST TCCol
						UID			NMTOKEN #IMPLIED
					>
				*/

				aNode = TCSNode.appNode( "TCCol" );

				aNode.addUID( m_uidTCCol );

				x = m_TCLst.Count;

				for( i = 0; i < x; ++i ) {

					refTC = m_TCLst[ i ];

					bNode = aNode.appNode( "RefTC" );

					bNode.addUID( refTC.UID );

					bNode.addAttrib( "ExpDur", refTC.m_ExpDur );
					bNode.addAttrib( "RefUID", refTC.RefUID );
					bNode.addAttribAsHtml( "Title", refTC.Title );
					bNode.addAttribAsHtml( "Type", refTC.Type );

					if( refTC.IsSet ) {

						bNode.addAttrib( "TotTCs", refTC.m_TotTCs );
					}
				}
			}

			return TCSNode;
		}

		//----------------------------
		private void handleExcp( Exception p_Excp ) {
			
			try {
				setAppStat( AppStat.Error );

				addLog( "Exception '" + p_Excp.Message + "'");

				m_Logger.write2Log( ref p_Excp );

				m_GUIservice.dispExcp( p_Excp );
			}
			catch {

				string msg = "An exception has occured at a very early stage of the"
					+ " initialization of the program. So early that the program is not able"
					+ " to provide you any aid. Errors at this stage are mostly due to"
					+ " bad configuration. If the configuration has been changed recently,"
					+ " switch back to the former version, if you can. In other cases please"
					+ " refer to the list of known problems in your documentation.";

				MessageBox.Show(  msg, "Early Error Occured", MessageBoxButtons.OK, MessageBoxIcon.Error );
			}
		}

		//------------------------------
		private void handleErr( IRslt p_Res, string p_Msg ) {

			setAppStat( AppStat.Error );

			m_Logger.write2Log( ref p_Res );

			addLog( p_Msg );

			issueErr( p_Msg );
		}

		//-----------------------
		private void handleIfErr( IRslt p_Res, string p_Msg ) {

			if( p_Res.notOK() ) {

				handleErr( p_Res, p_Msg );
			}
		}
				
		// -----------------------------
		/// <summary>
		/// Displays a text dialog. Detects, if the dialog is supposed to allow editing.
		/// If so, the Rslt is written back to p_Sender. 
		/// </summary>
		/// <param name="p_Sender"></param>
		/// <param name="p_Title"></param>
		private void handleTxtPop( object p_Sender, string p_Title ) {

			try {

				bool forEdit;
				DialogResult dRes;
				TextBox tBox;

				tBox = (TextBox)p_Sender;
				
				forEdit = !(tBox.ReadOnly);
			
				setAppStat( AppStat.Proc );

				frmTxtWin txtWin;
		
				txtWin = new frmTxtWin( forEdit, ref m_Presets );

				txtWin.Text = p_Title;

				txtWin.setText( tBox.Text );

				txtWin.TopMost = true;

				dRes = txtWin.ShowDialog();

				if( DialogResult.OK == dRes ) {

					if( forEdit ) {

						tBox.Text = txtWin.m_Text;
					}

					setAppStat( AppStat.OK );
				}

				txtWin.Dispose();

				setAppStat( AppStat.OK );
			}
			catch( Exception excp ) {
				
				handleExcp( excp );
			}	
		}
	
		// -----------------------------------
		private Rslt init() {
    frmTCSetComp.CheckForIllegalCrossThreadCalls = false;
			int x, y;
			Rslt res = new Rslt( true );

			m_QNTools = new QNTools();

			m_QNphrase = m_QNTools.m_QNphrase;

			m_GUIservice = new QNGUIservice( m_QNphrase );

			m_ApplPth = m_QNTools.getApplPth();

			m_Presets = new QNpresets( m_QNTools );

			setClrs();

			// resizing stuff
			x = this.Width;
			y = this.Height;	

			m_panel1DH = y - panel1.Height;
			m_panel1DW = x - panel1.Width;
			
			m_lstTCDW = x - lstVTC.Width;
			m_lstTCDH = y - lstVTC.Height;
			m_colDW = x - lstVTC.Columns[1].Width;

			m_txtStatDT = y - txtStat.Top;
			m_comboLogDT = y - comboLog.Top;

			//init services 	

			m_Logger.write2Log( m_QNTools.getApplInfo() );

			setLabels();

			//pre-sets
			m_tExpDur = 0;
			m_TCCnt = 0;

			m_TCLst = new RefTCInfoLst( "" );

			m_TCSelPresets = new TCSelPresets();
			m_TCSSelPresets = new TCSelPresets();

			if( m_Presets.notOK() ) {

				res.setRsltStat( m_Presets, m_QNphrase.errPresetFrm );
			}
			else {
			
				setUpAppl();

                m_DB = new RGSQL2( m_Presets.m_DBloc, m_Presets.m_DBname, m_Presets.m_DBuser, m_Presets.m_DBpw );

                if( res.checkRslt( m_DB ) )
                {
                    m_xDB = new XMLDB( m_DB, m_Presets.m_UIDsrvLoc );

                    res.setRsltStat( m_xDB );
                }
			}

			if( res.notOK() ) {

				handleErr( res, m_QNphrase.errInitFailed );
			}
			else {

				reset();

				setAppStat( AppStat.OK );

				addLog( BTools.getNowISODate() + " " + m_QNphrase.msgSesStart );
			}

			return res;
		}

		//------------------------------
		private void issueErr( string msg ) {

			MessageBox.Show( msg, m_QNphrase.lblErr, MessageBoxButtons.OK, MessageBoxIcon.Error );
		}

		//------------------------------
		private Rslt loadTCS( string p_loadUID ) {

			Rslt res = new Rslt( true );
			XMLDBNode xDBNode;

			xDBNode = m_xDB.getXMLsubTree( ref p_loadUID );

			if( res.checkRslt( xDBNode ) ) {

				reset();

				res = loadTCSdoc( xDBNode.m_xDoc );
			}

			if( res.isOK() ) {

				setLoaded();
			}
			else {

				m_Logger.write2Log( "Loading of '" +  p_loadUID + "' failed." );

				handleErr( res, m_QNphrase.errTCSLoadFailed );
			}

			return res;
		}
		
		//---------------------------
		private Rslt loadTCSdoc( XmlDocument p_XDoc ) {

			Rslt res = new Rslt( true );

			int i, x;

			RefTCInfo tcRefDat;

			RsltString xVal;
			RsltXMLNode xNode;
			RsltXMLNode aNode;

			string refUID;
			string expDur;
			string title;
			string ttc;
			string type;
			string val;
			
			XmlNodeList nodeLst;

			/*
				<!ELEMENT TCSet ( Desc, ReqmntSrc*, TCCol, TSysConfigRef?, ProcLog* )>
					<!ATTLIST TCSet
						Appl			CDATA #IMPLIED
						Author		CDATA #REQUIRED
						Date			NMTOKEN #REQUIRED
						ExpDur		NMTOKEN #IMPLIED
						Fctn			CDATA #IMPLIED
						Section		CDATA #IMPLIED
						SoftRel		CDATA #IMPLIED
						TCType		(ac|co|do|fi|fu|lo|mo|pe|se|un|us|vo|prep) ""
						Time			NMTOKEN #REQUIRED
						Title			CDATA #REQUIRED
						UID				NMTOKEN #IMPLIED
					>
			*/

			nodeLst = p_XDoc.SelectNodes( "/TCSet" );

			if( 1 != nodeLst.Count ) {

				m_Logger.write2Log( "Top node 'TCSet' not found." );
				res.setInError( m_QNphrase.errInvXML );
			}
			else {

				xNode = nodeLst.Item( 0 );

				m_uidTCS = xNode.UID;

				xVal = xNode.getAttrValFromHtml( "Appl" );

				m_QNTools.selByDataLstVal( ref comboAppl, xVal.theString );

				xVal = xNode.getAttrValFromHtml( "Author" );
    
				txtAuthor.Text = xVal.theString;

				xVal = xNode.getAttrVal( "DateTime" );

				if( res.checkRslt( xVal ) ) {

					val = BTools.getDateFromISODateTime( xVal.theString );   
					txtDate.Text = val;

					val = BTools.getTimeFromISODateTime( xVal.theString );   
					txtTime.Text = val;

					xVal = xNode.getAttrVal( "ExpDur" );

					txtExpDur.Text = xVal.theString;

					xVal = xNode.getAttrValFromHtml( "Fctn" );

					txtFctn.Text = xVal.theString;

					// The lock
					xVal = xNode.getAttrVal( "Lock" );

					if( xVal.theString.Length > 0 ) {

						menuSave.Enabled = false;
					}

					xVal = xNode.getAttrValFromHtml( "Section" );

					m_QNTools.selByDataLstVal( ref comboSection, xVal.theString );

					xVal = xNode.getAttrValFromHtml( "SoftRel" );

					m_QNTools.selByDataLstVal( ref comboApplRel, xVal.theString );

					xVal = xNode.getAttrValFromHtml( "TCType" );
    
					m_QNTools.selByDataLstVal( ref comboTCType, xVal.theString );

					// Title
					xVal = xNode.getAttrValFromHtml( "Title" );
				}

				if( res.checkRslt( xVal ) ) {

					txtTitle.Text = xVal.theString;

					aNode = xNode.getSubNode( "Desc" );
					
					if( aNode.isOK() ) {
					
						txtDesc.Text = aNode.textFromHtml;
						m_uidDesc = aNode.UID;
					}

					/*
						<!ELEMENT TCCol (RefTC*)>
							<!ATTLIST TCCol
								UID			NMTOKEN #IMPLIED
							>
					*/

					nodeLst = xNode.m_xNode.SelectNodes( "./TCCol" );

					if( nodeLst.Count == 1 ) {

						aNode = nodeLst.Item( 0 );

						m_uidTCCol = aNode.UID;

						nodeLst = aNode.m_xNode.SelectNodes( "./RefTC" );

						x = nodeLst.Count;

						for( i = 0; i < x; ++ i ) {

							aNode = nodeLst.Item( i );

							refUID = aNode.getAttrVal( "RefUID" );
							expDur = aNode.getAttrVal( "ExpDur" );
							title = aNode.getAttrValFromHtml( "Title" );
							type = aNode.getAttrValFromHtml( "Type" );

							tcRefDat = new RefTCInfo( aNode.UID, title, refUID, type );

							tcRefDat.m_ExpDur = expDur;

							if( tcRefDat.IsSet ) {

								ttc = aNode.getAttrVal( "TotTCs" );

								tcRefDat.m_TotTCs = ttc;
							}

							addTCtoLst( ref tcRefDat );
						}
					}
				}
			}

			updateCounter();

			return res;
		}

		//--------------------------
		private void loadTCSloader() {

			Rslt res = new Rslt();

			try {

				reset();

				res = loadTCS( m_loadUID );

				if( res.notOK() ) {

					handleErr( res, m_QNphrase.errTCSLoadFailed );
				}
				else {

					setLoaded();

					setAppStatOK( res.isOK() );
					addLog( String.Format( m_QNphrase.msgLoaded, txtTitle.Text ) );
				}
			}
			catch( Exception e ) {
		
				handleExcp( e );
			}
		}

		//-------------------------------------------------------------------------
		private void reset() {

			m_tExpDur = 0;
			m_TCCnt = 0;

			txtDesc.Text = "";
			txtExpDur.Text = "";
			txtFctn.Text = "";
			txtTitle.Text = "";
			txtTotTCs.Text = "";

			m_TCLst.Clear();
			lstVTC.Items.Clear();

			// txtAuthor is left as is

			txtDate.Text = BTools.getNowISODate();
			txtTime.Text = BTools.getNowISOTime();
	
			txtDesc.Text = "";

			menuDel.Enabled = false;
			menuPrint.Enabled = false;
			menuSave.Enabled = true;
			menuSaveNew.Enabled = false;

			resetUIDs();
		}
	
		//--------------------------
		private void resetUIDs() {

			int i, x;
			RefTCInfo refTC;

			m_uidDesc = "";
			m_uidTCS = "";
			m_uidTCCol = "";
			
			x = m_TCLst.Count;

			for( i = 0; i < x; ++i ) {

				refTC = m_TCLst[ i ];

				refTC.UID = "";

				m_TCLst[ i ] = refTC;
			}
		}

		//---------------------------
		private void setAppStat( AppStat p_Stat ) {

			switch( p_Stat) {

				case AppStat.OK:

					txtStat.BackColor = m_Presets.m_ClrStatOK;
					txtStat.ForeColor = Color.White;
					txtStat.Text = m_QNphrase.lblOK;
					break;

				case AppStat.Proc:

					txtStat.BackColor = m_Presets.m_ClrStatProc;
					txtStat.ForeColor = Color.Black;
					txtStat.Text = m_QNphrase.lblProcng;
					break;

				case AppStat.Error:

					txtStat.BackColor = m_Presets.m_ClrStatErr;
					txtStat.ForeColor = Color.White;
					txtStat.Text =  m_QNphrase.lblErr;
					break;
			}

			this.Refresh();			
		}

		//-------------------------------------------------------------------------
		private void setAppStatOK( bool p_IsOK ) {

			if( p_IsOK ) {

				setAppStat( AppStat.OK );
			}
			else {

				setAppStat( AppStat.Error );
			}
		}

		//-------------------------------------------------------------------------
		private void setClrs() {

			this.BackColor = m_Presets.m_ClrPnlBck;
			panel1.BackColor = m_Presets.m_ClrPnlBck;
			panel1.ForeColor = m_Presets.m_ClrPnlFor;

			comboAppl.BackColor = m_Presets.m_ClrTxtBck;
			comboApplRel.BackColor = m_Presets.m_ClrTxtBck;
			comboLog.BackColor = m_Presets.m_ClrTxtBck;
			comboSection.BackColor = m_Presets.m_ClrTxtBck;
			comboTCType.BackColor = m_Presets.m_ClrTxtBck;

			lstVTC.BackColor = m_Presets.m_ClrTxtBck;
						
			txtAuthor.BackColor = m_Presets.m_ClrTxtBck;
			txtDate.BackColor = m_Presets.m_ClrTxtBck;
			txtDesc.BackColor = m_Presets.m_ClrTxtBck;
			txtFctn.BackColor = m_Presets.m_ClrTxtBck;
			txtTime.BackColor = m_Presets.m_ClrTxtBck;
			txtTitle.BackColor = m_Presets.m_ClrTxtBck;
			//txtExpDur.BackColor = m_Presets.m_ClrTxtBck;
			//txtTotTCs.BackColor = m_Presets.m_ClrTxtBck;
		}

		//-------------------------------------------------------------------------
		private RsltString saveTCSet() {
				
			RsltXMLNode xNode;

			RsltString UID = new RsltString();

			xNode = bldTCSdoc();

			if( xNode.notOK() ) {

				string msg = String.Format( m_QNphrase.errCrtngFailed, txtTitle.Text );

				handleErr( xNode, msg );

				UID.setInError( msg );
			}
			else {

				UID = m_xDB.importXML( ref xNode.m_xDoc );
 
				if( m_xDB.notOK() ) {

					handleErr( m_xDB, m_QNphrase.errTCSSaveFailed );

					m_xDB.resetRslt();
				}
			}

			return UID;
		}

		// ----------------------------------------------------
		private void setUpAppl() {

			txtAuthor.Text = m_Presets.m_Author;		

			m_QNTools.reFillUpCombo( ref comboAppl, ref m_Presets.m_ApplLst, 0);

			m_QNTools.reFillUpCombo( ref comboApplRel,ref m_Presets.m_ApplSoftRelLst, 0);
			
			m_QNTools.reFillUpCombo( ref comboSection, ref m_Presets.m_SectionLst, 0);

			m_QNTools.reFillUpCombo( ref comboTCType, ref m_Presets.m_TstTypeLst, 0);		
		}

		// --------------------
		private void updateCounter() {

			txtTotTCs.Text = Convert.ToString( m_TCCnt );
			txtExpDur.Text = Convert.ToString( m_tExpDur );
		}

		#endregion
		
		#region I18N

		//---------------------------------
		private void setLabels() {

			Text = m_QNphrase.lblTCSetComp;

			ctxtTCLstAddTC.Text = m_QNphrase.mlblAddTC;
			ctxtTCLstAddTCS.Text = m_QNphrase.mlblAddTCSet;
			ctxtTCLstRem.Text = m_QNphrase.mlblRemove;
			ctxtTCLstView.Text = m_QNphrase.mlblView;

			ctxtLogEntry.Text = m_QNphrase.mlblDispEntry;

			lblAuthor.Text = m_QNphrase.lblAuthor;
			lblAppl.Text = m_QNphrase.lblAppl;
			lblDate.Text = m_QNphrase.lblDate;
			lblDesc.Text = m_QNphrase.lblDesc;
			lblDuration.Text = m_QNphrase.lblDuration;
			lblFctn.Text = m_QNphrase.lblFctn;
			lblSection.Text = m_QNphrase.lblSection;
			lblSoftRel.Text = m_QNphrase.lblRelease;
			lblTCType.Text = m_QNphrase.lblTCType;
			lblTCLst.Text = m_QNphrase.lblTCLst;
			lblTime.Text = m_QNphrase.lblTime;
			lblTitle.Text = m_QNphrase.lblTitle;
			lblTotTCs.Text = m_QNphrase.lblTotTCs;

			lstVTC.Columns[1].Text = m_QNphrase.lblTitle;
			lstVTC.Columns[2].Text = m_QNphrase.lblDuration;
			
			menuAbout.Text = m_QNphrase.mlblAbout;
			menuDocu.Text = m_QNphrase.mlblDocu;
			menuDel.Text = m_QNphrase.mlblDel;
			menuExit.Text = m_QNphrase.mlblExit;
			menuFile.Text = m_QNphrase.mlblFile;
			menuHelp.Text = m_QNphrase.mlblHelp;
			menuNew.Text = m_QNphrase.mlblNew;
			menuOpen.Text = m_QNphrase.mlblOpen;
			menuPrint.Text = m_QNphrase.mlblPrint;
			menuSave.Text = m_QNphrase.mlblSave;
			menuSaveNew.Text = m_QNphrase.mlblSaveAsNew;
		}

		// ----------------------------------------------------
		private void setLoaded() {

			menuDel.Enabled = true;
			menuPrint.Enabled = true;
			menuSaveNew.Enabled = true;
		}

		#endregion

		#region Event Handling
    
		//-------------------------
		private void ctxtLogEntry_Click(object sender, System.EventArgs e) {

			string msg;

			setAppStat( AppStat.Proc );

			msg = m_QNTools.getComboSelVal( ref comboLog );

			MessageBox.Show( msg, m_QNphrase.lblLogEntry, MessageBoxButtons.OK );	

			setAppStat( AppStat.OK );
		}
		
		// --------------------------
		private void frmTCSetComp_Resize(object sender, System.EventArgs e) {
		
			int x, y;

			y = this.Height;
			x = this.Width;

			panel1.Height = y - m_panel1DH;
			panel1.Width = x - m_panel1DW;

			lstVTC.Height = y - m_lstTCDH;
			lstVTC.Width = x - m_lstTCDW;
			lstVTC.Columns[1].Width = x - m_colDW;

			txtStat.Top = y - m_txtStatDT;
			comboLog.Top = y - m_comboLogDT;
		}

		//---------------------------
		private void menuAbout_Click(object sender, System.EventArgs e) {

			string title;
			string msg;

			setAppStat( AppStat.Proc );

			title = String.Format( m_QNphrase.msgAbout, m_QNphrase.lblTCEditor );

			msg = m_QNTools.getAboutInfoBlock();
			
			MessageBox.Show( msg, title, MessageBoxButtons.OK );	

			setAppStat( AppStat.OK );
		}

		//------------------------------
		private void menuDel_Click(object sender, System.EventArgs e) {

			DialogResult dRes;

			try {

				setAppStat( AppStat.Proc );

				string msg = String.Format( m_QNphrase.msgSure2dDel, txtTitle.Text );
			
				dRes = MessageBox.Show( msg, m_QNphrase.lblDeleting, MessageBoxButtons.YesNo, MessageBoxIcon.Question );

				if( DialogResult.Yes != dRes ) {

					setAppStat( AppStat.OK );
				}
				else {

					setAppStat( AppStat.Proc );

					m_xDB.deleteXML( ref m_uidTCS );

					if( m_xDB.notOK() ) {

						msg = String.Format( m_QNphrase.errDelFailed, txtTitle.Text );

						handleErr( m_xDB, msg );

						m_xDB.resetRslt();
					}
					else {

						addLog( String.Format( m_QNphrase.msgDeleted, txtTitle.Text ) );

						reset();

						setAppStat( AppStat.OK );
					}
				}
			}
			catch( Exception excp ) {
						
				handleExcp( excp );
			}
		}

		//-------------------------
		private void menuExit_Click(object sender, System.EventArgs e) {
		
			setAppStat( AppStat.Proc );

			Close();
		}

		
		//-----------------------------------------------------
		private void menuNew_Click(object sender, System.EventArgs e) {
			
			setAppStat( AppStat.Proc );

			reset();

			setAppStat( AppStat.OK );
		}

		//-----------------------------------------------------
		private void menuOpen_Click(object sender, System.EventArgs e) {
		
			try {

				DataLst RsltLst;

				int x;
				DatLstEntry entry;
			
				setAppStat( AppStat.Proc );

				frmTCSel dialTCSSel = new frmTCSel( m_xDB, true, m_Presets, m_TCSSelPresets, "tcs" );

				DialogResult dRes = dialTCSSel.ShowDialog();

				if( DialogResult.Cancel == dRes ) {

					setAppStat( AppStat.OK );
				}
				else {

					RsltLst = dialTCSSel.m_RsltLst;

					x = RsltLst.Count;

					if( x < 1 ) {

						setAppStat( AppStat.OK );
					}
					else {

						entry = RsltLst[0];

						m_loadUID = entry.val;

						Thread loadThread = new Thread(new ThreadStart(loadTCSloader));

						// Start the thread.
						loadThread.Start();
					}
				}

				dialTCSSel.Dispose();
			}
			catch( Exception excp ) {
		
				handleExcp( excp );
			}
		}

		//-------------------------------------------------------------------------
		private void menuSave_Click(object sender, System.EventArgs e) {
		
			try {

				RsltString UID;

				setAppStat( AppStat.Proc );
				
				UID = saveTCSet();

				if( UID.isOK() ) {

					Rslt res = loadTCS( UID.theString );

					if( res.notOK() ) {

						handleErr( res, m_QNphrase.errTCLoadFailed );
					}
					else {

						addLog( String.Format( m_QNphrase.msgSaved, txtTitle.Text ) );
					
						setAppStat( AppStat.OK );
					}
				}
			}
			catch( Exception excp ) {
				
				handleExcp( excp );
			}  
		}

		//-----------------------
		private void menuSaveNew_Click(object sender, System.EventArgs e) {
		
			try {

				RsltString UID;

				setAppStat( AppStat.Proc );

				resetUIDs();

				txtDate.Text = BTools.getNowISODate();
				txtTime.Text = BTools.getNowISOTime();

				UID = saveTCSet();

				if( UID.isOK() ) {

					Rslt res = loadTCS( UID.theString );

					if( res.notOK() ) {

						handleErr( res, m_QNphrase.errTCLoadFailed );
					}
					else {

						addLog( String.Format( m_QNphrase.msgSaved, txtTitle.Text ) );
					
						setAppStat( AppStat.OK );
					}
				}
			}
			catch( Exception excp ) {
						
				handleExcp( excp );
			}
		}

		// --------------------------------
		private void txtDesc_DoubleClick(object sender, System.EventArgs e) {
		
			handleTxtPop( sender, lblDesc.Text );
		}

		// --------------------------------
		private void ctxtTCLstAdd_Click(object sender, System.EventArgs e) {

			addTCDial( "tc" );	
		}

		// --------------------------------
		private void ctxtTCLstAddTCS_Click(object sender, System.EventArgs e) {
		
			addTCDial( "tcs" );	
		}

		// --------------------------------
		private void ctxtTCLstRem_Click(object sender, System.EventArgs e) {
				
			try {

				RefTCInfo refTC;

				int i, x, idx;

				string dur;

				setAppStat( AppStat.Proc );

				x = lstVTC.SelectedIndices.Count;

				//int y;

				for( i = 0; i < x; ++i ) {

					//y = lstVTC.SelectedIndices.Count;

					idx = lstVTC.SelectedIndices[0];

					refTC = m_TCLst[ idx ];

					dur = refTC.m_ExpDur;

					if( dur.Length > 0 ) {

						m_tExpDur -= Convert.ToInt32( dur );
					}

					if( refTC.IsSet ) {
						
						if( refTC.m_TotTCs.Length > 0 ) {

							m_TCCnt -= Convert.ToInt32( refTC.m_TotTCs );
						}
					}
					else {

						m_TCCnt -= 1;
					}

					m_TCLst.RemoveAt( idx );

					lstVTC.Items.RemoveAt( idx );
				}

				updateCounter();

				setAppStat( AppStat.OK );
			}
			catch( Exception excp ) {
		
				handleExcp( excp );
			}
		}

		// --------------------------------
		private void ctxtTCLstView_Click(object sender, System.EventArgs e) {

			try {

				string UID;

				int idx;

				RefTCInfo refTC;

				setAppStat( AppStat.Proc );

				idx = lstVTC.SelectedIndices[0];

				refTC = m_TCLst[ idx ];

				UID = refTC.RefUID;

				System.Diagnostics.Process proc = new System.Diagnostics.Process();

				proc.EnableRaisingEvents = false;

				if( refTC.IsSet ) {

					proc.StartInfo.FileName = QNDefs.exeTCSEdit;
				}
				else {

					proc.StartInfo.FileName = QNDefs.exeTCEdit;
				}

				proc.StartInfo.Arguments = UID;
				proc.Start();

				setAppStat( AppStat.OK );
			}
			catch( Exception excp ) {
	
				handleExcp( excp );
			}
		}

		// ------------------------
		private void menuDocu_Click(object sender, System.EventArgs e) {
		
			try {

				setAppStat( AppStat.Proc );

				System.Diagnostics.Process proc = new System.Diagnostics.Process();

				proc.EnableRaisingEvents = false;

				proc.StartInfo.FileName = QNDefs.manQN;

				proc.Start();

				setAppStat( AppStat.OK );
			}
			catch( Exception excp ) {
	
				handleExcp( excp );
			}
		}

		// ------------------------
		private void menuPrint_Click(object sender, System.EventArgs e) {
		
			string title;

			Rslt res = new Rslt( true );

			setAppStat( AppStat.Proc );

			RsltXMLNode xNode = bldTCSdoc();

			string xsltName = QNDefs.xlsTCSet + "_" + m_QNphrase.m_Loc + ".xsl";

			if( res.checkRslt( xNode ) ) {

				title = m_QNphrase.lblTCSet + " '" + txtTitle.Text + "'";

				res = m_GUIservice.printXDoc( xNode.m_xDoc, ref xsltName, ref title );
			}

			if( res.notOK() ) {

				handleErr( res, m_Presets.m_QNphrase.errPrintFailed );
			}
			else {

				setAppStat( AppStat.OK );
			}
		}



		// -------------------------
		private void ctxtMenuLog_Popup(object sender, System.EventArgs e) {
		
			if( comboLog.SelectedIndex > -1 ) {

				ctxtLogEntry.Enabled = true;
			}
			else {

				ctxtLogEntry.Enabled = true;
			}
		}

		// -------------------------
		private void ctxtMenuTCLst_Popup(object sender, System.EventArgs e) {
		
			if( lstVTC.SelectedIndices.Count > 0 ) {

				ctxtTCLstRem.Enabled = true;
				ctxtTCLstView.Enabled = true;
			}
			else {

				ctxtTCLstRem.Enabled = false;
				ctxtTCLstView.Enabled = false;
			}
		}

		#endregion
	}
}
